@@ -52,7 +52,7 @@ static void mime_part_set_header (GMimeObject *object, const char *header, const
 static const char *mime_part_get_header (GMimeObject *object, const char *header);
 static void mime_part_remove_header (GMimeObject *object, const char *header);
 static char *mime_part_get_headers (GMimeObject *object);
-static ssize_t mime_part_write_to_stream (GMimeObject *object, GMimeStream *stream);
+static gssize mime_part_write_to_stream (GMimeObject *object, GMimeStream *stream);
 
 #define NEEDS_DECODING(encoding) (((GMimePartEncodingType) encoding) == GMIME_PART_ENCODING_BASE64 ||   \
 				  ((GMimePartEncodingType) encoding) == GMIME_PART_ENCODING_UUENCODE || \
@@ -269,10 +269,10 @@ mime_part_get_headers (GMimeObject *object)
 	return GMIME_OBJECT_CLASS (parent_class)->get_headers (object);
 }
 
-static ssize_t
+static gssize
 write_content (GMimePart *part, GMimeStream *stream)
 {
-	ssize_t nwritten, total = 0;
+	gssize nwritten, total = 0;
 	
 	if (!part->content)
 		return 0;
@@ -350,11 +350,11 @@ write_content (GMimePart *part, GMimeStream *stream)
 	return total;
 }
 
-static ssize_t
+static gssize
 mime_part_write_to_stream (GMimeObject *object, GMimeStream *stream)
 {
 	GMimePart *mime_part = (GMimePart *) object;
-	ssize_t nwritten, total = 0;
+	gssize nwritten, total = 0;
 	
 	/* write the content headers */
 	nwritten = g_mime_header_write_to_stream (object->headers, stream);
@@ -1233,7 +1233,7 @@ g_mime_part_get_content (const GMimePart *mime_part, size_t *len)
  *
  * Returns the number of bytes written or -1 on fail.
  **/
-ssize_t
+gssize
 g_mime_part_write_to_stream (GMimePart *mime_part, GMimeStream *stream)
 {
 	g_return_val_if_fail (GMIME_IS_PART (mime_part), -1);
