@@ -29,7 +29,7 @@ extern "C" {
 #pragma }
 #endif /* __cplusplus */
 
-#include <glib.h>
+#include <glib/gtypes.h>
 #include <stdio.h>
 
 #include "gmime-object.h"
