@@ -33,6 +33,11 @@
 #include <pan/base/serverlist.h>
 #include <pan/base/status-item.h>
 
+#include <pan/filters/filter-aggregate.h>
+#include <pan/filters/filter-phrase.h>
+#include <pan/filters/filter-score.h>
+#include <pan/filters/score.h>
+
 #include <pan/article-actions.h>
 #include <pan/articlelist.h>
 #include <pan/message-window.h>
@@ -45,7 +50,6 @@
 #include <pan/task-save.h>
 #include <pan/util.h>
 
-
 /***
 ****  Finding a match in pan.sent
 ***/
@@ -234,7 +238,9 @@ article_copy_articles_to_folder_cb (gpointer obj, gpointer arg, gpointer data)
 gboolean
 article_copy_articles_to_folder (Group * folder, const Article ** articles, int qty)
 {
-	Task * task;
+	int i;
+	int missing_qty;
+	MessageIdentifier ** missing;
 	debug_enter ("article_copy_articles_to_folder");
 
 	/* sanity clause */
@@ -244,15 +250,43 @@ article_copy_articles_to_folder (Group * folder, const Article ** articles, int
 	g_return_val_if_fail (qty>0, FALSE);
 	g_return_val_if_fail (articles_are_valid(articles, qty), FALSE);
 
-	/* make a task to download the bodies in case we don't already have them */
-	task = TASK(task_bodies_new_from_articles (articles, qty));
-	if (task != NULL) {
-		const char * source_key = group_get_acache_key (articles[0]->group);
-		ArgSet * args = argset_new2 (g_strdup(source_key), folder);
-		pan_callback_add (task->task_ran_callback, article_copy_articles_to_folder_cb, args);
-		queue_add (task);
+	/* copy anything that's already cached, and make a list of those not cached */
+	missing = g_newa (MessageIdentifier*, qty);
+	missing_qty = 0;
+	for (i=0; i<qty; ++i)
+	{
+		const Article * article = articles[i];
+		const char * acache_key = group_get_acache_key (article->group);
+		MessageIdentifier * mid = message_identifier_new_from_article (article);
+		GMimeMessage * message = acache_get_message (acache_key, &mid, 1);
+
+		if (message == NULL)
+			missing[missing_qty++] = mid;
+		else
+		{
+			folder_add_message (folder, message);
+			g_object_unref (message);
+			g_object_unref (mid);
+		}
 	}
 
+	/* queue anything that's missing */
+	if (missing_qty != 0)
+	{
+		Task * task = TASK (task_bodies_new (articles[0]->group->server, missing, missing_qty));
+
+		if (task != NULL)
+		{
+			const char * source_key = group_get_acache_key (articles[0]->group);
+			ArgSet * args = argset_new2 (g_strdup(source_key), folder);
+			pan_callback_add (task->task_ran_callback, article_copy_articles_to_folder_cb, args);
+			queue_add (task);
+		}
+	}
+
+	/* cleanup */
+	for (i=0; i<missing_qty; ++i)
+		g_object_unref (missing[i]);
 	debug_exit ("article_copy_articles_to_folder");
 	return TRUE;
 }
@@ -357,132 +391,9 @@ article_action_selected_save_as (void)
 }
 
 /***
-****
-****
 ****   DELETING ARTICLES
-****
-****
 ***/
 
-static void
-remove_duplicate_articles (GPtrArray * articles)
-{
-	int i;
-	GHashTable * hash;
-
-	/* sanity claus */
-	g_return_if_fail (articles!=NULL);
-	g_return_if_fail (articles->len>0);
-	if (articles->len==1) return;
-
-	/* remove duplicates */
-	hash = g_hash_table_new (g_direct_hash, g_direct_equal);
-	for (i=0; i<articles->len; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(articles,i));
-		g_hash_table_insert (hash, a, a);
-	}
-	pan_hash_to_ptr_array  (hash, articles);
-
-	/* cleanup */
-	g_hash_table_destroy (hash);
-}
-
-
-/**
- * Pairs of Group* and GArrays of gulong article numbers to remove from
- * the group.  We pass these arrays on to group_remove_crossposts(),
- * then free the array because we're done with it.
- * @see articlelist_delete_articles
- */
-static void
-articlelist_remove_xrefs_ghfunc (gpointer key,
-                                 gpointer value,
-                                 gpointer user_data)
-{
-	Group * g = GROUP(key);
-	GArray * numbers = (GArray*) value;
-	group_remove_crossposts (g, (gulong*)numbers->data, numbers->len);
-	g_array_free (numbers, TRUE);
-}
-
-/**
- * data is a GHashTable of (key=group,val=GArray) where we fill those
- * GArrays with the article numbers of this crosspost.
- */
-static void
-articlelist_remove_xrefs_xreffunc (Server   * server,
-                                   Group    * group,
-                                   gulong     number,
-                                   gpointer   data)
-{
-	GHashTable * hash = (GHashTable*) data;
-
-	if (group_is_subscribed (group))
-	{
-		GArray * a = g_hash_table_lookup (hash, group);
-		if (a == NULL)
-			g_hash_table_insert (hash,
-			                     group,
-			                     a = g_array_new (FALSE, FALSE, sizeof(gulong)));
-		g_array_append_val (a, number);
-	}
-}
-
-/**
- * @see remove_duplicate_articles
- * @see articlelist_remove_xrefs_xreffunc
- * @see articlelist_remove_xrefs_ghfunc
- */
-static void
-delete_articles (GPtrArray * articles)
-{
-	debug_enter ("article_action_delete_articles");
-
-	g_return_if_fail (articles!=NULL);
-
-	if (articles->len)
-	{
-		Group * group = ARTICLE(g_ptr_array_index(articles,0))->group;
-
-		/* uniqueness check */
-		remove_duplicate_articles (articles);
-
-		/* remove these articles from the cache */
-		if (1) {
-			guint i;
-			GPtrArray * ids = g_ptr_array_new ();
-			for (i=0; i<articles->len; ++i) {
-				Article * a = ARTICLE(g_ptr_array_index(articles,i));
-				g_ptr_array_add (ids, (gpointer)article_get_message_id(a));
-			}
-			acache_expire_messages (group_get_acache_key(group), (const gchar **)ids->pdata, ids->len);
-			g_ptr_array_free (ids, TRUE);
-		}
-
-		/* if these articles are posted elsewhere, delete those crossposts too */
-		if (1) {
-			guint i;
-			GHashTable * hash = g_hash_table_new (g_direct_hash, g_direct_equal);
-			for (i=0; i<articles->len; ++i) {
-				Article * a = ARTICLE(g_ptr_array_index(articles,i));
-				article_xref_foreach (a, articlelist_remove_xrefs_xreffunc, hash, SERVER_GROUPS_SUBSCRIBED, TRUE);
-			}
-			g_hash_table_foreach (hash, articlelist_remove_xrefs_ghfunc, NULL);
-			g_hash_table_destroy (hash);
-		}
-
-		/* delete the articles in this group */
-		group_remove_articles (group, articles);
-
-		/* log */
-		log_add_va (LOG_INFO, _("Deleted %u articles from \"%s\""),
-		            articles->len,
-		            group_get_name(group));
-	}
-
-	debug_exit ("article_action_delete_articles");
-}
-
 void
 article_action_delete_selected_articles (void)
 {
@@ -503,7 +414,26 @@ article_action_delete_selected_articles (void)
 	}
 
 	/* delete the articles */
-	delete_articles (articles);
+	if (articles->len)
+	{
+		Group * group;
+		MessageIdentifier ** mids;
+
+		/* remove the headers */
+		mids = g_new (MessageIdentifier*, articles->len);
+		for (i=0; i<articles->len; ++i)
+			mids[i] = message_identifier_new_from_article (ARTICLE(g_ptr_array_index(articles,i)));
+		message_identifiers_delete ((const MessageIdentifier**)mids, articles->len, SERVER_GROUPS_SUBSCRIBED);
+
+		/* remove the cached bodies */
+	       	group = articlelist_get_group ();
+		acache_expire_message_ids (group_get_acache_key(group), (const MessageIdentifier**)mids, articles->len);
+
+		/* cleanup */
+		for (i=0; i<articles->len; ++i)
+			g_object_unref (mids[i]);
+		g_free (mids);
+	}
 
 	/* cleanup */
 	g_ptr_array_free (articles, TRUE);
@@ -529,3 +459,92 @@ article_action_edit_selected (void)
 	g_object_unref (mid);
 	debug_exit ("article_action_edit_selected");
 }
+
+/***
+****
+***/
+
+enum {
+	DO_WATCH,
+	DO_IGNORE,
+	DO_LIST
+};
+
+static void
+article_action_something_thread (const Article * article, int action)
+{
+	ScoreAddItem add_items[2];
+	GPtrArray * articles;
+	char * score_name;
+	char * thread_message_id;
+	struct {
+		int score;
+		const char * verb;
+	} do_list[DO_LIST] = {
+		{ 9999, "watch" },
+		{ -9999, "ignore" }
+	};
+
+	/* sanity checking */
+	g_return_if_fail (article_is_valid (article));
+	g_return_if_fail (action==DO_WATCH || action==DO_IGNORE);
+
+	/* get the new score's name */
+	thread_message_id = article_get_thread_message_id (article);
+	score_name = g_strdup_printf ("%s thread \"%s\"",
+		do_list[action].verb,
+		article_get_subject (article_get_root (article)));
+
+	/* if the article's Message-ID is the top article, 
+	 * the article's Referencess contains the top article,
+	 * set the score to watch/ignore */
+	add_items[0].on = TRUE;
+	add_items[0].negate = FALSE;
+	add_items[0].key = HEADER_MESSAGE_ID;
+	add_items[0].value = filter_phrase_create_regex (thread_message_id, PHRASE_MATCH_IS);
+	add_items[1].on = TRUE;
+	add_items[1].negate = FALSE;
+	add_items[1].key = HEADER_REFERENCES;
+	add_items[1].value = filter_phrase_create_regex (thread_message_id, PHRASE_MATCH_STARTS_WITH);
+	score_add (score_name, NULL, do_list[action].score, TRUE, 31, AGGREGATE_TYPE_OR, add_items, 2);
+
+	/* rescore the thread */
+	articles = g_ptr_array_new ();
+	article_get_thread (article, articles);
+	ensure_articles_scored (article->group, (Article**)articles->pdata, articles->len);
+
+	/* cleanup */
+	g_ptr_array_free (articles, TRUE);
+	g_free (add_items[1].value);
+	g_free (add_items[0].value);
+	g_free (score_name);
+	g_free (thread_message_id);
+}
+
+void
+article_action_watch_thread (const Article * article)
+{
+	article_action_something_thread (article, DO_WATCH);
+}
+
+void
+article_action_watch_selected_thread (void)
+{
+	Article * article = articlelist_get_selected_article_nolock ();
+	if (article != NULL)
+		article_action_watch_thread (article);
+}
+
+void
+article_action_ignore_thread (const Article * article)
+{
+	article_action_something_thread (article, DO_IGNORE);
+}
+
+void
+article_action_ignore_selected_thread (void)
+{
+	Article * article = articlelist_get_selected_article_nolock ();
+	if (article != NULL)
+		article_action_ignore_thread (article);
+}
