@@ -41,4 +41,13 @@ void      article_action_delete_selected_articles    (void);
 
 void      article_action_edit_selected               (void);
 
+void      article_action_watch_thread                (const Article   * article);
+
+void      article_action_watch_selected_thread       (void);
+
+void      article_action_ignore_thread               (const Article   * article);
+
+void      article_action_ignore_selected_thread      (void);
+
+
 #endif
