@@ -35,7 +35,6 @@
 #include <pan/article-toolbar.h>
 #include <pan/filter-mediator.h>
 #include <pan/globals.h>
-#include <pan/gui.h>
 #include <pan/util.h>
 
 static void fire_user_changed_filter (void);
@@ -109,7 +108,8 @@ merge_filter_with_phrase (Filter * filter)
 		filter_phrase_set (FILTER_PHRASE(phrase_filter),
 		                   PHRASE_MATCH_CONTAINS,
 		                   get_phrase_key_type(),
-		                   key);
+		                   key,
+		                   FALSE);
 
 		retval = filter_aggregate_new ();
 		filter_aggregate_add (FILTER_AGGREGATE(retval), children, G_N_ELEMENTS(children));
@@ -272,7 +272,7 @@ article_toolbar_new (void)
 	***  One-time initialization of local fields
 	**/
 
-	bits = ~(gulong)STATE_FILTER_IGNORED;
+	bits = ~(gulong)STATE_FILTER_SCORE_LOW;
 	g_snprintf (buf, sizeof(buf), "/Display/filter_bits=%lu", bits);
 	bits = pan_config_get_ulong (buf);
 
