@@ -158,20 +158,24 @@ acache_message_id_to_basename (char * buf, int len, const char * message_id)
 	g_return_val_if_fail (buf!=NULL, NULL);
 	g_return_val_if_fail (len>0, NULL);
 
-	/* transform */
+	/* some characters in message-ids are illegal on older Windows boxes,
+	 * so we transform those illegal characters using URL encoding */
 	for (in=message_id, out=buf; *in; ++in) {
 		switch (*in) {
-			case '_': *out++ = '_'; *out++ = '_'; break;
-			case '%': *out++ = '_'; *out++ = 'p'; break;
-			case '$': *out++ = '_'; *out++ = 'd'; break;
-			case '/': *out++ = '_'; *out++ = 's'; break;
-			case '\\': *out++ = '_'; *out++ = 'b'; break;
-			case '<': break;
-			case '>': break;
-			default: *out++ = *in; break;
+			case '%': /* this is the escape character */
+			case '"': case '*': case '/': case ':': case '?': case '|':
+			case '\\': /* these are illegal on vfat, fat32 */
+				g_snprintf (out, len-(out-buf), "%%%02x", (int)*in);
+				out += 3;
+				break;
+			case '<': case '>': /* these are illegal too, but rather than encoding
+			                       them, follow the convention of omitting them */
+				break;
+			default:
+				*out++ = *in;
 		}
 	}
-	strcpy (out, ".msg");
+	g_snprintf (out, len-(out-buf), ".msg");
 
 	debug_exit ("acache_message_id_to_basename");
 	return buf;
@@ -206,14 +210,14 @@ acache_basename_to_message_id (char * buf, int len, const char * basename)
 	out = buf;
 	*out++ = '<';
 	for (in=tmp_basename; *in; ++in) {
-		if (*in != '_')
+		if (in[0]!='%' || !g_ascii_isxdigit(in[1]) || !g_ascii_isxdigit(in[2]))
 			*out++ = *in;
-		else switch (*++in) {
-			case '_': *out++ = '_'; break;
-			case 'p': *out++ = '%'; break;
-			case 'd': *out++ = '$'; break;
-			case 's': *out++ = '/'; break;
-			case 'b': *out++ = '\\'; break;
+		else {
+			char buf[3];
+			buf[0] = *++in;
+			buf[1] = *++in;
+			buf[2] = '\0';
+			*out++ = (char) strtoul (buf, NULL, 16);
 		}
 	}
 	*out++ = '>';
@@ -516,6 +520,26 @@ acache_expire_messages (const char   *  path_key,
 	g_static_mutex_unlock (&entries_mutex);
 }
 
+void
+acache_expire_message_ids (const char                * path_key,
+                           const MessageIdentifier  ** mids,
+                           int                         mid_qty)
+{
+	int i;
+	const char ** message_ids;
+
+	/* sanity clause */
+	g_return_if_fail (is_nonempty_string (path_key));
+	g_return_if_fail (mid_qty >= 0);
+	g_return_if_fail (message_identifiers_are_valid ((const MessageIdentifier **)mids, mid_qty));
+
+	/* let acache_expire_messages() do all the work */
+	message_ids = g_newa (const char*, mid_qty);
+	for (i=0; i<mid_qty; ++i)
+		message_ids[i] = mids[i]->message_id;
+	acache_expire_messages (path_key, message_ids, mid_qty);
+}
+
 static int
 acache_expire_to_size (AcachePath * apath, double max_megs)
 {
@@ -523,6 +547,9 @@ acache_expire_to_size (AcachePath * apath, double max_megs)
 	const size_t cache_max = (size_t) max_megs * (1024 * 1024);
 	debug_enter ("acache_expire_to_size");
 
+	/* sanity clause */
+	g_return_val_if_fail (apath!=NULL, 0);
+
 	g_static_mutex_lock (&entries_mutex);
 	{
 		debug3 (DEBUG_ACACHE, "expiring to %lu; current size is %lu (%ld free)", cache_max, apath->size, (long)cache_max-(long)apath->size);
@@ -805,7 +832,7 @@ acache_get_message  (const char          * path_key,
 	g_return_val_if_fail (message_identifiers_are_valid ((const MessageIdentifier**)mids, mid_qty), NULL);
 
 	/* get streams */
-	streams = g_alloca (sizeof(GMimeStream*) * mid_qty);
+	streams = g_newa (GMimeStream*, mid_qty);
 	for (i=stream_qty=0; i<mid_qty; ++i)
 	{
 		GMimeStream * stream;
