@@ -19,7 +19,7 @@
 
 #include <config.h>
 
-#include <glib.h>
+#include <glib/gmem.h>
 
 #include <ctype.h>
 #include <limits.h>
@@ -58,12 +58,18 @@ typedef struct
 Norm;
 
 /**
- * Normalizing a subject header involves tearing out the multipart
- * substrings ("(21/42)" or "[12|213]") and converting it all to
- * one case so that we can use strcmp instead of g_strcasecmp.
+ * Normalizing a subject header involves:
  *
- * When we're threading articles, it's a big speedup to normalize the
- * subjects at the outset instead of normalizing them in each comparison.
+ * 1. tearing out the numerator from multipart indicators
+ *    (i.e., remove "21" from (21/42))
+ *    (why?)
+ * 2. convert it to lowercase so that we can use strcmp
+ *    instead of g_strcasecmp, which is slower.
+ * 3. strip out all the leading noise that breaks sorting.
+ *
+ * When we're threading articles, it's much faster to
+ * normalize the * subjects at the outset instead of
+ * normalizing them for each comparison.
  */
 static size_t
 normalize_subject (char * buf, const Article * a)
@@ -90,16 +96,13 @@ normalize_subject (char * buf, const Article * a)
 
 	while (*in)
 	{
-		/* strip multipart information */
+		/* strip numerator out of multipart information */
 		if (multipart && (*in=='('||*in=='[') && isdigit(in[1])) {
-			const guchar * start = in;
-			while (*in && *in!=']' && *in!=')') {
-				if (isalpha(*in)) { /* oops, not multipart information */
-					in = ++start;
-					break;
-				} 
+			const guchar * start = ++in;
+			while (*in && isdigit(*in))
 				++in;
-			}
+			if (*in!='/' && *in!='|') /* oops, not a multipart */
+				in = start;
 			continue;
 		}
 
@@ -169,7 +172,7 @@ compare_pA_to_pA_by_part (const void * va, const void * vb)
 	gboolean        b_is_reply;
 
 	/* already normalized, so check multipart first */
-	if (a->parts == b->parts)
+	if (a->parts!=0 && (a->parts == b->parts))
 		return a->part - b->part;
 
 	/* order by subject */
@@ -226,6 +229,36 @@ compare_pN_to_pN_by_subject (const void * va, const void * vb)
 	return value;
 }
 
+static int
+compare_ppA_to_ppA_by_score (const void* va, const void* vb)
+{
+        register const Article * a = *(const Article**)va;
+        register const Article * b = *(const Article**)vb;
+	return a->score - b->score;
+}
+
+static gulong
+article_get_combined_linecount (const Article * a)
+{
+	gulong retval = 0;
+
+	g_return_val_if_fail (article_is_valid(a), FALSE);
+
+	retval = a->linecount;
+	if (a->part!=0 && a->parts!=0 && a->threads!=NULL) {
+		GSList * l;
+		int part = a->part + 1;
+		for (l=a->threads; l; l=l->next) {
+			Article * child = ARTICLE(l->data);
+			if (child->part == part) {
+				retval += child->linecount;
+				++part;
+			}
+		}
+	}
+
+	return retval;
+}
 
 static int
 compare_ppA_to_ppA_by_linecount (const void* va, const void* vb)
@@ -242,8 +275,8 @@ compare_ppA_to_ppA_by_action (const void * va, const void * vb)
 	register const Article * b = *(const Article **)vb;
 	int ia, ib;
 
-	ia = article_flag_on (a, STATE_DECODED) ? 1 : 0;
-	ib = article_flag_on (b, STATE_DECODED) ? 1 : 0;
+	ia = article_get_decode_state (a);
+	ib = article_get_decode_state (b);
 	if (ia != ib)
 		return ib - ia;
 
@@ -281,13 +314,8 @@ compare_ppA_to_ppA_by_read (const void * va, const void * vb)
 	if (ia != ib)
 		return ib - ia;
 
-	ia = article_flag_on (a, STATE_MULTIPART_ALL) ? 1 : 0;
-	ib = article_flag_on (b, STATE_MULTIPART_ALL) ? 1 : 0;
-	if (ia != ib)
-		return ib - ia;
-
-	ia = article_flag_on (a, STATE_MULTIPART_SOME) ? 1 : 0;
-	ib = article_flag_on (b, STATE_MULTIPART_SOME) ? 1 : 0;
+	ia = article_get_multipart_state (a);
+	ib = article_get_multipart_state (b);
 	if (ia != ib)
 		return ib - ia;
 
@@ -391,6 +419,11 @@ sort_articles (Article      ** buf,
 			msort (buf, article_qty, sizeof(Article*), compare_ppA_to_ppA_by_read);
 			break;
 		}
+		case ARTICLE_SORT_SCORE:
+		{
+			msort (buf, article_qty, sizeof(Article*), compare_ppA_to_ppA_by_score);
+			break;
+		}
 		case ARTICLE_SORT_SUBJECT:
 		default:
 		{
@@ -436,10 +469,10 @@ is_child_of (const Article * child,
 	}
 }
 
-static guint
+static int
 article_root_get_part_state (const Article * a)
 {
-	guint retval;
+	int retval;
 
 	/* sanity clause */
 	g_return_val_if_fail (a!=NULL, 0u);
@@ -447,19 +480,19 @@ article_root_get_part_state (const Article * a)
 
 	/* not a multipart */
 	if (a->parts<1)
-		retval = 0;
+		retval = MULTIPART_STATE_NONE;
 
 	/* someone's posted a "00/124" nfo message */
 	else if (a->part==0)
-		retval = 0;
+		retval = MULTIPART_STATE_NONE;
 
 	/* incomplete multipart */
 	else if (a->part>1)
-		retval = STATE_MULTIPART_SOME;
+		retval = MULTIPART_STATE_SOME;
 
 	/* someone's posted a followup to a multipart */
 	else if (a->linecount<250 && !g_strncasecmp(a->subject,"Re:", 3))
-		retval = 0;
+		retval = MULTIPART_STATE_NONE;
 
 	/* a multipart */
 	else {
@@ -468,25 +501,25 @@ article_root_get_part_state (const Article * a)
 		for (l=a->threads; l!=NULL; l=l->next)
 			if (ARTICLE(l->data)->parts==a->parts && ARTICLE(l->data)->part==part)
 				++part;
-		retval = part==a->parts+1 ? STATE_MULTIPART_ALL : STATE_MULTIPART_SOME;
+		retval = part==a->parts+1 ? MULTIPART_STATE_ALL : MULTIPART_STATE_SOME;
 	}
 
 	return retval;
 }
 
 static void
-set_children_part_state (Article * a, const guint state)
+set_children_part_state (Article * a, const int state)
 {
 	gint part;
 	GSList * l;
 
-	a->state |= state;
+	a->multipart_state = state;
 
 	part = a->part + 1;
 	for (l=a->threads; l!=NULL; l=l->next) {
 		Article * child = ARTICLE(l->data);
 		if (child->parts==a->parts && child->part==part) {
-			child->state |= state;
+			child->multipart_state = state;
 			++part;
 		}
 	}
@@ -564,7 +597,7 @@ thread_articles (GPtrArray    * articles)
 		references = a->references;
 		if (a->parts<2 && references!=NULL && *references=='<')
 		{
-			gchar * message_id;
+			char * message_id;
 			g_string_assign (tmp_references, references);
 		       	message_id = strrchr (tmp_references->str, '<');
 
@@ -700,7 +733,7 @@ thread_articles (GPtrArray    * articles)
 
 	for (i=0; i!=articles->len; ++i) {
 		Article * a = ARTICLE(g_ptr_array_index(articles,i));
-		a->state &= ~(STATE_MULTIPART_ALL|STATE_MULTIPART_SOME);
+		a->multipart_state = MULTIPART_STATE_NONE;
 	}
 	for (i=0; i!=articles->len; ++i) {
 		Article * a = ARTICLE(g_ptr_array_index(articles,i));
