@@ -32,7 +32,6 @@
 #include <pan/base/group.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-i18n.h>
-/*#include <pan/base/run.h>*/
 #include <pan/base/server.h>
 #include <pan/base/group.h>
 #include <pan/base/util-mime.h>
@@ -48,112 +47,104 @@ static void fire_articles_changed (Group*, Article**, int article_qty, ArticleCh
 ****
 ***/
 
+#define DEAD_POINTER ((void*)(0xDEADBEEF))
+
 void
 article_destructor (Article * a)
 {
+	static Article dead_article;
+	static gboolean dead_article_inited = FALSE;
+
+	/* sanity clause */
         g_return_if_fail (a != NULL);
 
-	/* clean up article parts */
-	if (a->threads != NULL) {
-		g_slist_free (a->threads);
-		a->threads = NULL;
+	/* make sure dead_article is initialized */
+	if (!dead_article_inited)
+	{
+		dead_article_inited = TRUE;
+
+		dead_article.is_new           = ~0;
+		dead_article.passes_filter    = ~0;
+		dead_article.error_flag       = ~0;
+		dead_article.multipart_state  = ~0;
+		dead_article.decode_state     = ~0;
+		dead_article.part             = ~0;
+		dead_article.parts            = ~0;
+		dead_article.score            = ~0;
+		dead_article.score_date       = ~0;
+		dead_article.byte_qty         = ~0;
+		dead_article.linecount        = ~0;
+		dead_article.unread_children  = ~0;
+		dead_article.date             = ~0;
+		dead_article.threads          = DEAD_POINTER;
+		dead_article.parent           = DEAD_POINTER;
+		dead_article.xref             = DEAD_POINTER;
+		dead_article.subject          = DEAD_POINTER;
+		dead_article.author_addr      = DEAD_POINTER;
+		dead_article.author_real      = DEAD_POINTER;
+		dead_article.message_id       = DEAD_POINTER;
+		dead_article.references       = DEAD_POINTER;
 	}
-	a->date = 0;
-	a->number = 0;
-	a->linecount = 0;
-	a->message_id = NULL;
-	a->xref = NULL;
-	a->references = NULL;
-	a->subject = NULL;
-	a->author_addr = NULL;
-	a->author_real = NULL;
-	a->group = NULL;
-}
 
-static void
-article_constructor (Article * article, Group * group)
-{
-	g_return_if_fail (article!=NULL);
-	g_return_if_fail (group!=NULL);
-
-	/* init this class */
-	article->group = group;
-	article->is_new = FALSE;
-	article->self_passes_filter = FALSE;
-	article->tree_passes_filter = FALSE;
-	article->part = (gint16)0;
-	article->parts = (gint16)0;
-	article->score = (gint16)0;
-	article->byte_qty = (gulong)0;
-	article->linecount = (guint16)0;
-	article->state = (guint16)0;
-	article->unread_children = (guint16)0;
-	article->new_children = (guint16)0;
-	article->date = (time_t)0;
-	article->number = (gulong)0;
-	article->subject = NULL;
-	article->author_real = NULL;
-	article->author_addr = NULL;
-	article->message_id = NULL;
-	article->xref = NULL;
-	article->references = NULL;
-	article->parent = NULL;
-	article->threads = NULL;
+	/* destroy the article */
+	g_slist_free (a->threads);
+	*a = dead_article;
 }
 
 Article*
 article_new (Group * group)
 {
 	Article * article;
+	static Article empty_article;
+	static gboolean empty_article_inited = FALSE;
 
+	/* sanity clause */
 	g_return_val_if_fail (group!=NULL, NULL);
 
+	/* make sure the zero struct is initialized */
+	if (!empty_article_inited)
+	{
+		empty_article_inited = TRUE;
+
+		empty_article.is_new           = FALSE;
+		empty_article.passes_filter    = FALSE;
+		empty_article.error_flag       = FALSE;
+		empty_article.multipart_state  = (guint8)0;
+		empty_article.decode_state     = (guint8)0;
+		empty_article.part             = (gint16)0;
+		empty_article.parts            = (gint16)0;
+		empty_article.score            = (gint16)0;
+		empty_article.byte_qty         = (gulong)0;
+		empty_article.linecount        = (guint16)0;
+		empty_article.unread_children  = (guint16)0;
+		empty_article.new_children     = (guint16)0;
+		empty_article.score_date       = (time_t)0;
+		empty_article.date             = (time_t)0;
+		empty_article.number           = (gulong)0;
+		empty_article.subject          = NULL;
+		empty_article.author_real      = NULL;
+		empty_article.author_addr      = NULL;
+		empty_article.message_id       = NULL;
+		empty_article.xref             = NULL;
+		empty_article.references       = NULL;
+		empty_article.parent           = NULL;
+		empty_article.threads          = NULL;
+		empty_article.group            = DEAD_POINTER;
+	}
+
+	/* create the new article */
 	article = group_alloc_new_article (group);
-	article_constructor (article, group);
+	*article = empty_article;
+	article->group = group;
+
 	return article;
 }
 
-
-Article*
-article_dup (Group * group, const Article * article)
-{
-	Article * retval;
-
-	g_return_val_if_fail (article_is_valid(article), NULL);
-	g_return_val_if_fail (group!=NULL, NULL);
-
-#define chunk_if_exists(A, share) \
-	is_nonempty_string (A) ? group_chunk_string (group, A, share) : NULL
-
-	retval                      = article_new (group);
-	retval->self_passes_filter  = FALSE;
-	retval->tree_passes_filter  = FALSE;
-	retval->unread_children     = 0;
-	retval->new_children        = 0;
-	retval->group               = group;
-	retval->parts               = article->parts;
-	retval->part                = article->part;
-	retval->linecount           = article->linecount;
-	retval->state               = article->state;
-	retval->date                = article->date;
-	retval->number              = article->number;
-	retval->xref                = chunk_if_exists (article->xref, FALSE);
-	retval->subject             = chunk_if_exists (article->subject, TRUE);
-	retval->author_real         = chunk_if_exists (article->author_real, TRUE);
-	retval->author_addr         = chunk_if_exists (article->author_addr, TRUE);
-	retval->message_id          = chunk_if_exists (article->message_id, FALSE);
-	retval->references          = chunk_if_exists (article->references, TRUE);
-	retval->parent              = NULL;
-	retval->threads             = NULL;
-
-	return retval;
-}
-
 /***
 ****
 ***/
 
-static void
+void
 articles_set_dirty (Article ** articles, int qty)
 {
 	Group * group;
@@ -161,9 +152,9 @@ articles_set_dirty (Article ** articles, int qty)
 
 	/* sanity check */
 	g_return_if_fail (qty >= 1);
-	g_return_if_fail (articles_are_valid ((const Article **)articles, qty));
+	g_return_if_fail (articles_are_valid_in_group ((const Article **)articles, qty));
 
-	/* mark the articles' group as read */
+	/* mark the articles' group as dirty */
 	group = articles[0]->group;
 	group_set_articles_dirty (group);
 	fire_articles_changed (group, articles, qty, ARTICLE_CHANGED_DIRTY);
@@ -193,7 +184,7 @@ articles_set_new_impl (Article ** articles, int article_qty, gboolean is_new, gb
 
 	/* sanity check */
 	g_return_if_fail (article_qty > 0);
-	g_return_if_fail (articles_are_valid ((const Article **)articles, article_qty));
+	g_return_if_fail (articles_are_valid_in_group ((const Article **)articles, article_qty));
 
 	/* mark each article's newness field */
 	changed = g_ptr_array_sized_new (article_qty);
@@ -205,16 +196,19 @@ articles_set_new_impl (Article ** articles, int article_qty, gboolean is_new, gb
 		}
 	}
 
-	/* update parent counts of changed articles */
-	for (i=0; i<changed->len; ++i) {
-		Article * a = ARTICLE(g_ptr_array_index(changed, i));
-		for (a=a->parent; a!=NULL; a=a->parent)
-			a->new_children += is_new ? 1 : -1;
-	}
+	if (changed->len)
+	{
+		/* update parent counts of changed articles */
+		for (i=0; i<changed->len; ++i) {
+			Article * a = ARTICLE(g_ptr_array_index(changed, i));
+			for (a=a->parent; a!=NULL; a=a->parent)
+				a->new_children += is_new ? 1 : -1;
+		}
 
-	/* fire notification of changed articles */
-	if (fire && changed->len!=0)
-		fire_articles_changed (articles[0]->group, (Article**)changed->pdata, changed->len, ARTICLE_CHANGED_NEW);
+		/* fire notification of changed articles */
+		if (fire)
+			fire_articles_changed (articles[0]->group, (Article**)changed->pdata, changed->len, ARTICLE_CHANGED_NEW);
+	}
 
 	/* cleanup */
 	g_ptr_array_free (changed, TRUE);
@@ -242,40 +236,6 @@ article_is_read (const Article * a)
 	return newsrc_is_article_read (group_get_newsrc(a->group), a->number);
 }
 
-static void
-articles_set_read_numbers_ghfunc (gpointer key, gpointer value, gpointer user_data)
-{
-	int i;
-	int changed_qty;
-	Group * g = GROUP(key);
-	GArray * numbers = (GArray*) value;
-	gboolean read = user_data!=NULL;
-	Newsrc * newsrc = group_get_newsrc (g);
-
-	/**
-	***  Mark each number as read/unread
-	**/
-
-	for (i=changed_qty=0; i<numbers->len; ++i)
-	{
-		const gulong number = g_array_index (numbers, gulong, i);
-
-		if (read != newsrc_mark_article (newsrc, number, read))
-			++changed_qty;
-	}
-
-	/**
-	***  If any changed, update the group's count
-	**/
-
-	if (changed_qty != 0)
-	{
-		group_inc_article_read_qty (g, read ? changed_qty : -changed_qty);
-	}
-
-	g_array_free (numbers, TRUE);
-}
-
 void
 articles_set_read_simple (Article    ** articles,
                           int           article_qty,
@@ -284,21 +244,23 @@ articles_set_read_simple (Article    ** articles,
 	guint i;
 	Group * g;
 	Newsrc * newsrc;
-	GPtrArray * changed;
+	Article ** changed;
+	int changed_qty;
 
 	/* sanity clause */
 	g_return_if_fail (article_qty>0);
-	g_return_if_fail (articles_are_valid ((const Article **)articles, article_qty));
+	g_return_if_fail (articles_are_valid_in_group ((const Article **)articles, article_qty));
 
 	/* prep local variables */
 	g = articles[0]->group;
 	newsrc = group_get_newsrc (g);
-       	changed = g_ptr_array_sized_new (article_qty);
+	changed = g_new (Article*, article_qty);
+	changed_qty = 0;
 
 	/* find the articles that will be changed by this action */
 	for (i=0; i<article_qty; ++i)
 		if (!!read != !!newsrc_mark_article (newsrc, articles[i]->number, read))
-			g_ptr_array_add (changed, articles[i]);
+			changed[changed_qty++] = articles[i];
 
 	/**
 	***  If anything is to be changed:
@@ -307,46 +269,52 @@ articles_set_read_simple (Article    ** articles,
 	***  (3) fire an event
 	**/
 
-	if (changed->len != 0)
+	if (changed_qty != 0)
 	{
 		/* update parent counts */
-		for (i=0; i!=changed->len; ++i)
+		for (i=0; i!=changed_qty; ++i)
 		{
-			Article * a = ARTICLE(g_ptr_array_index(changed,i));
+			Article * a = changed[i];
 			for (a=a->parent; a!=NULL; a=a->parent)
 				a->unread_children += read ? -1 : 1;
 		}
 		if (read)
-			articles_set_new_impl ((Article**)changed->pdata, changed->len, FALSE, FALSE);
+			articles_set_new_impl (changed, changed_qty, FALSE, FALSE);
 
 		/* update the group's count */
-		group_inc_article_read_qty (g, read ? changed->len : -changed->len);
+		group_inc_article_read_qty (g, read ? changed_qty : -changed_qty);
 
 		/* fire an event */
-		fire_articles_changed (g, (Article**)changed->pdata, changed->len, ARTICLE_CHANGED_READ);
+		fire_articles_changed (g, changed, changed_qty, ARTICLE_CHANGED_READ);
 	}
+
+	/* cleanup */
+	g_free (changed);
 }
 
 static void
-articles_set_read_articles_ghfunc (gpointer key, gpointer value, gpointer user_data)
+articles_set_read_numbers_ghfunc (gpointer key, gpointer value, gpointer user_data)
 {
-	GArray * articles = (GArray*) value;
-	const gboolean read = user_data != NULL;
-	articles_set_read_simple ((Article**)articles->data, articles->len, read);
-	g_array_free (articles, TRUE);
-}
+	int i;
+	int changed_qty;
+	Group * g = GROUP(key);
+	GArray * numbers = (GArray*) value;
+	const gboolean read = user_data!=NULL;
+	Newsrc * newsrc = group_get_newsrc (g);
 
-/**
- * The hashtable maps from a Group* to a GArray*, which is returned.
- * If no matching GArray* exists, it's created and added to the hash before returning.
- */
-static GArray*
-get_group_array_from_hash (GHashTable * hash, Group * group, int size)
-{
-	GArray * arr = (GArray*) g_hash_table_lookup (hash, group);
-	if (arr == NULL)
-		g_hash_table_insert (hash, group, arr = g_array_new (FALSE, FALSE, size));
-	return arr;
+	/* Mark each number as read/unread */
+	for (i=changed_qty=0; i<numbers->len; ++i) {
+		const gulong number = g_array_index (numbers, gulong, i);
+		if (read != newsrc_mark_article (newsrc, number, read))
+			++changed_qty;
+	}
+
+	/* If any changed, update the group's count */
+	if (changed_qty != 0)
+		group_inc_article_read_qty (g, read ? changed_qty : -changed_qty);
+
+	/* cleanup */
+	g_array_free (numbers, TRUE);
 }
 
 /**
@@ -361,73 +329,49 @@ mark_read_xreffunc (Server * server, Group * g, gulong number, gpointer user_dat
 	{
 		GArray * group_numbers;
 
+		/* make sure *group_to_numbers holds a hash table */
 		if (*group_to_numbers == NULL)
 			*group_to_numbers = g_hash_table_new (g_direct_hash, g_direct_equal);
 
-		group_numbers = get_group_array_from_hash (*group_to_numbers, g, sizeof(gulong));
+		/* make sure group_numbers is a GArray of ulongs in *group_to_numbers */
+		group_numbers = (GArray*) g_hash_table_lookup (*group_to_numbers, g);
+		if (group_numbers == NULL) {
+			group_numbers = g_array_new (FALSE, FALSE, sizeof(gulong));
+			g_hash_table_insert (*group_to_numbers, g, group_numbers);
+		}
+
 		g_array_append_val (group_numbers, number);
 	}
 }
 
-void
-articles_set_read (Article ** articles, int article_qty, gboolean read)
+static void
+articles_set_read_and_crossposts (Article ** articles, int article_qty, gboolean read)
 {
 	int i;
-	GHashTable * group_to_articles;
 	GHashTable * group_to_numbers = NULL;
-	debug_enter ("articles_set_read");
+	debug_enter ("articles_set_read_and_crossposts");
 
 	/* sanity check */
 	g_return_if_fail (article_qty > 0);
-	g_return_if_fail (articles_are_valid ((const Article **)articles, article_qty));
+	g_return_if_fail (articles != NULL);
+	g_return_if_fail (articles_are_valid_in_group ((const Article **)articles, article_qty));
 
-	/* get a big hash, keyed by Group*,  of all the articles that we're
-	   marking read.  We have two hashes -- one for the articles passed
-	   in, and one for article numbers of the xposts.  */
-	group_to_articles = g_hash_table_new (g_direct_hash, g_direct_equal);
+	/* mark the Articles as read */
+	articles_set_read_simple (articles, article_qty, read);
+
+	/* collect the crossposts.
+	 * this is done with a hash of Group* key to an array of gulong article indices
+	 * from the article's xref header. */
 	for (i=0; i<article_qty; ++i)
-	{
-		GArray * arr;
-		Article * a = articles[i];
+		article_xref_foreach (articles[i],
+		                      mark_read_xreffunc,
+		                      &group_to_numbers,
+		                      SERVER_GROUPS_SUBSCRIBED,
+		                      TRUE);
 
-		/* first add this article to the group's array */
-	       	arr = get_group_array_from_hash (group_to_articles, a->group, sizeof(Article*));
-		g_array_append_val (arr, a);
-
-		/* secondly, add the xrefs to their group's arrays */
-		article_xref_foreach (a, mark_read_xreffunc, &group_to_numbers,
-		                      SERVER_GROUPS_SUBSCRIBED, TRUE);
-
-		/* lastly, if it's a multipart, process the child parts too */
-		if (a->part==1 && a->parts>1 && a->threads!=NULL)
-		{
-			GSList * l;
-			for (l=a->threads; l!=NULL; l=l->next)
-			{
-				Article * child = ARTICLE(l->data);
-				if (child->part>a->part && child->parts==a->parts)
-				{
-					/* add this article to the group's array */
-					g_array_append_val (arr, child);
-
-					/* add the xrefs to their group's arrays */
-					article_xref_foreach (child,
-						mark_read_xreffunc,
-						&group_to_numbers,
-						SERVER_GROUPS_SUBSCRIBED, TRUE);
-				}
-			}
-		}
-	}
-
-	/* mark the passed-in articles as read/unread */
-	g_hash_table_foreach (group_to_articles,
-	                      articles_set_read_articles_ghfunc,
-	                      GINT_TO_POINTER(read));
-	g_hash_table_destroy (group_to_articles);
-
-
-	/* mark their crosspost counterparts, if any, as read/unread */
+	/* now that we've organized the crossposts by group,
+	 * mark them read with one call per group
+	 * to minimize the number of events fired. */
 	if (group_to_numbers != NULL) {
 		g_hash_table_foreach (group_to_numbers,
 		                      articles_set_read_numbers_ghfunc,
@@ -435,18 +379,65 @@ articles_set_read (Article ** articles, int article_qty, gboolean read)
 		g_hash_table_destroy (group_to_numbers);
 	}
 
+	debug_exit ("articles_set_read_and_crossposts");
+}
+
+void
+articles_set_read (Article ** articles, int article_qty, gboolean read)
+{
+	int i;
+	GPtrArray * all_articles;
+	debug_enter ("articles_set_read");
+
+	/* sanity check */
+	g_return_if_fail (article_qty > 0);
+	g_return_if_fail (articles_are_valid_in_group ((const Article **)articles, article_qty));
+
+	/* add all the articles passed in */
+	all_articles = g_ptr_array_new ();
+	pan_g_ptr_array_assign (all_articles, (gpointer*)articles, article_qty);
+
+	/* multiparts are marked read/unread as a set,
+	 * so if any of the articles passed in are multiparts,
+	 * add the children to the list too */
+	for (i=0; i<article_qty; ++i) {
+		const Article * a = articles[i];
+		if (a->part==1 && a->parts>1 && a->threads!=NULL) {
+			GSList * l;
+			for (l=a->threads; l!=NULL; l=l->next) {
+				Article * child = ARTICLE(l->data);
+				if (child->part>a->part && child->parts==a->parts)
+					g_ptr_array_add (all_articles, child);
+			}
+		}
+	}
+
+	/* mark them as read/unread */
+	articles_set_read_and_crossposts ((Article**)all_articles->pdata, all_articles->len, read);
+
+	/* cleanup */
+	g_ptr_array_free (all_articles, TRUE);
 	debug_exit ("articles_set_read");
 }
 
 
+
 /**
 *** PUBLIC MUTATORS
 **/
 
+int
+article_get_multipart_state (const Article * article)
+{
+	g_return_val_if_fail (article_is_valid (article), 0);
+
+	return article->multipart_state;
+}
+
 void
-articles_add_flag (Article  ** articles,
-                   int         qty,
-                   gushort     flag)
+articles_set_multipart_state (Article  ** articles,
+                              int         qty,
+                              int         state)
 {
 	int i;
 	int changed_qty;
@@ -454,50 +445,77 @@ articles_add_flag (Article  ** articles,
 
 	/* sanity clause */
 	g_return_if_fail (qty >= 1);
-	g_return_if_fail (articles_are_valid ((const Article **)articles, qty));
+	g_return_if_fail (articles_are_valid_in_group ((const Article **)articles, qty));
+	g_return_if_fail (state==MULTIPART_STATE_NONE || state==MULTIPART_STATE_SOME || state==MULTIPART_STATE_ALL);
 
 	/* mark 'em */
-	changed = g_malloc (sizeof(Article*) * qty);
+	changed = g_newa (Article*, qty);
 	for (changed_qty=i=0; i<qty; ++i) {
-		if (!(articles[i]->state & flag)) {
-			articles[i]->state |= flag;
+		Article * a = articles[i];
+		guint8 * pstate = &a->multipart_state;
+		if (*pstate != state) {
+			*pstate = state;
+			changed[changed_qty++] = a;
+		}
+		if (articles[i]->multipart_state != state) {
+			articles[i]->multipart_state = state;
 			changed[changed_qty++] = articles[i];
 		}
 	}
 
 	if (changed_qty > 0)
 		articles_set_dirty (changed, changed_qty);
+}
 
-	g_free (changed);
+int
+article_get_decode_state (const Article * article)
+{
+	g_return_val_if_fail (article_is_valid (article), 0);
+
+	return article->decode_state;
+}
+
+void
+articles_set_decode_state (Article  ** articles,
+                           int         qty,
+                           int         state)
+{
+	int i;
+	int changed_qty;
+	Article ** changed;
+
+	/* sanity clause */
+	g_return_if_fail (qty >= 1);
+	g_return_if_fail (articles_are_valid_in_group ((const Article **)articles, qty));
+	g_return_if_fail (state==DECODE_STATE_NONE || state==DECODE_STATE_DECODED || state==DECODE_STATE_FAILED);
+
+	/* mark 'em */
+	changed = g_newa (Article*, qty);
+	for (changed_qty=i=0; i<qty; ++i) {
+		Article * a = articles[i];
+		guint8 * pstate = &a->decode_state;
+		if (*pstate != state) {
+			*pstate = state;
+			changed[changed_qty++] = a;
+		}
+	}
+
+	if (changed_qty > 0)
+		articles_set_dirty (changed, changed_qty);
 }
 
 
 void
-articles_remove_flag (Article         ** articles,
-                      int                qty,
-                      gushort            flag)
+article_set_error_flag (Article * article, gboolean error_flag)
 {
-	int i;
-	int changed_qty;
-	Article ** changed;
+	g_return_if_fail (article_is_valid (article));
 
-	/* sanity clause */
-	g_return_if_fail (qty >= 1);
-	g_return_if_fail (articles_are_valid ((const Article **)articles, qty));
+	if (article->error_flag != error_flag)
+	{
+		article->error_flag = error_flag;
 
-	/* mark 'em */
-       	changed = g_malloc (sizeof(Article*) * qty);
-	for (changed_qty=i=0; i<qty; ++i) {
-		if (articles[i]->state & flag) {
-			articles[i]->state &= ~flag;
-			changed[changed_qty++] = articles[i];
-		}
+		articles_set_dirty (&article, 1);
 	}
-
-	if (changed_qty > 0)
-		articles_set_dirty (changed, changed_qty);
-
-	g_free (changed);
 }
 
 
@@ -508,8 +526,19 @@ articles_remove_flag (Article         ** articles,
 ****
 ***/
 
+Article*
+article_get_root (const Article* article)
+{
+	g_return_val_if_fail (article_is_valid(article), (Article*)article);
+
+	while (article->parent != NULL)
+		article = article->parent;
+
+	return (Article*) article;
+}
+
 char*
-article_get_thread_message_id (const Article* article)
+article_get_thread_message_id (const Article * article)
 {
 	char * pch;
 	const char * refs;
@@ -519,8 +548,7 @@ article_get_thread_message_id (const Article* article)
 
 	/* go up as high as we can from our own threading, because sometimes
 	 * the References: header is broken by other newreaders. */
-	while (article->parent != NULL)
-		article = article->parent;
+	article = article_get_root (article);
 
 	refs = article->references;
 	if (is_nonempty_string(refs))
@@ -529,66 +557,51 @@ article_get_thread_message_id (const Article* article)
 	}
 	else /* top of the thread */
 	{
-		const char * msg_id = article_get_message_id (article);
-		g_return_val_if_fail (is_nonempty_string(msg_id), NULL);
-		pch = g_strdup (msg_id);
+		pch = g_strdup (article_get_message_id (article));
 	}
 
 	return pch;
 }
 
 static void
-article_get_entire_thread_impl (Article* top, GPtrArray* setme)
+article_get_thread_impl (const Article * top, GPtrArray * setme)
 {
 	GSList *l;
-	g_ptr_array_add (setme, top);
+	g_ptr_array_add (setme, (gpointer)top);
 	for (l=top->threads; l!=NULL; l=l->next)
-		article_get_entire_thread_impl (ARTICLE(l->data), setme);
+		article_get_thread_impl (ARTICLE(l->data), setme);
 }
 
-static void
-article_get_thread_impl (Article * article,
-                         GPtrArray * setme,
-                         gboolean children_only)
+void
+article_get_subthread (const Article * article, GPtrArray * setme)
 {
-	Article* top = article;
-
 	/* sanity clause */
 	g_return_if_fail (article_is_valid(article));
 	g_return_if_fail (setme!=NULL);
 
-	if (top != NULL)
-	{
-		if (!children_only)
-			while (top->parent != NULL)
-				top = top->parent;
-
-		pan_g_ptr_array_reserve (setme, 128);
-		article_get_entire_thread_impl (top, setme);
-	}
+	pan_g_ptr_array_reserve (setme, 128);
+	article_get_thread_impl (article, setme);
 }
 
 void
-article_get_subthread (Article* article, GPtrArray* setme)
+article_get_thread (const Article * article, GPtrArray * setme)
 {
-	article_get_thread_impl (article, setme, TRUE);
+	/* sanity clause */
+	g_return_if_fail (article_is_valid(article));
+	g_return_if_fail (setme!=NULL);
+
+	article_get_subthread (article_get_root(article), setme);
 }
 
 void
-article_get_entire_thread (Article* article, GPtrArray* setme)
-{
-	article_get_thread_impl (article, setme, FALSE);
-}
-
-void
-article_get_references (Article* article, GPtrArray* setme)
+article_get_references (const Article * article, GPtrArray * setme)
 {
 	g_return_if_fail (article_is_valid(article));
 	g_return_if_fail (setme!=NULL);
 
 	pan_g_ptr_array_reserve (setme, 128);
 	while (article != NULL) {
-		g_ptr_array_add (setme, article);
+		g_ptr_array_add (setme, (gpointer)article);
 		article = article->parent;
 	}
 }
@@ -605,7 +618,7 @@ article_get_unique_threads (const GPtrArray    * articles,
 
 	/* sanity clause */
 	retval = g_ptr_array_sized_new (128);
-	g_return_val_if_fail (articles_are_valid ((const Article **)articles->pdata, articles->len), retval);
+	g_return_val_if_fail (articles_are_valid_in_group ((const Article **)articles->pdata, articles->len), retval);
 
 	/* get the thread for each article passed in */
 	thread = g_ptr_array_sized_new (128);
@@ -624,7 +637,7 @@ article_get_unique_threads (const GPtrArray    * articles,
 		if (thread_get == GET_SUBTHREAD)
 			article_get_subthread (a, thread);
 		else
-			article_get_entire_thread (a, thread);
+			article_get_thread (a, thread);
 
 		/* add the articles into "all" hash */
 		for (j=0; j<thread->len; ++j) {
@@ -641,101 +654,10 @@ article_get_unique_threads (const GPtrArray    * articles,
 	return retval;
 }
 
-
-const char **
-article_get_message_ids  (const Article ** articles,
-                          int              article_qty)
-{
-	int i;
-	const char ** message_ids;
-
-	g_return_val_if_fail (articles!=NULL, NULL);
-	g_return_val_if_fail (article_qty>=0, NULL);
-	g_return_val_if_fail (articles_are_valid (articles, article_qty), NULL);
-
-	message_ids = g_new (const char *, article_qty);
-	for (i=0; i<article_qty; ++i)
-		message_ids[i] = article_get_message_id (ARTICLE(articles[i]));
-
-	return message_ids;
-}
-
-
 /***
-****
-****  OTHER HEADERS
-****
+****  XREF
 ***/
 
-int
-article_get_crosspost_qty (const Article  * article)
-{
-	int retval;
-
-	g_return_val_if_fail (article_is_valid (article), 0);
-
-	if (!is_nonempty_string (article->xref))
-		retval = 1;
-	else {
-		const char * pch;
-		retval = 0;
-		for (pch=article->xref+6; *pch; ++pch)
-			if (*pch==':')
-				++retval;
-	}
-
-	return retval;
-}
-
-void
-article_remove_header (Article * a, const char * key)
-{
-	g_return_if_fail (a!=NULL);
-	g_return_if_fail (is_nonempty_string(key));
-		 
-	if (!strcmp(key, HEADER_MESSAGE_ID))
-		a->message_id = NULL;
-	else if (!strcmp(key, HEADER_XREF))
-		a->xref = NULL;
-	else if (!strcmp(key, HEADER_SUBJECT))
-		a->subject = NULL;
-	else if (!strcmp(key, HEADER_REFERENCES))
-		a->references = NULL;
-	else if (!strcmp(key, HEADER_FROM))
-	{
-		a->author_addr = NULL;
-		a->author_real = NULL;
-	}
-	else g_error (_("Don't know what to do with header \"%s\""), key);
-}
-
-gboolean
-article_header_is_internal (const char * key)
-{
-	return is_nonempty_string(key) && !g_strncasecmp(key,"X-Pan-Internal-",15);
-}
-
-gboolean
-article_header_is_extra (const char * key)
-{
-	/* sanity check */
-	if (!is_nonempty_string(key)) return FALSE;
-
-	/* pan internals aren't user-specified headers */
-	if (article_header_is_internal(key)) return FALSE;
-
-	/* other headers that are handled explicitly elsewhere */
-	if (!strcmp(key,HEADER_FOLLOWUP_TO)) return FALSE;
-	if (!strcmp(key,HEADER_NEWSGROUPS)) return FALSE;
-	if (!strcmp(key,HEADER_ORGANIZATION)) return FALSE;
-	if (!strcmp(key,HEADER_REPLY_TO)) return FALSE;
-	if (!strcmp(key,HEADER_DATE)) return FALSE;
-	if (!strcmp(key,HEADER_XREF)) return FALSE;
-	if (!strcmp(key,HEADER_FROM)) return FALSE;
-
-	return TRUE;
-}
-
 static void
 xref_foreach (const char         * xref_string,
               Server             * server,
@@ -790,85 +712,132 @@ article_xref_foreach (const Article       * a,
 	xref_foreach (a->xref, a->group->server, set, skip_group_a?a->group:NULL, func, user_data);
 }
 
-static void
-article_set_header_impl (Article        * a,
-                         const char     * key,
-                         const char     * val,
-                         HeaderAction     action,
-                         gboolean         init)
-{
-	g_return_if_fail (a!=NULL);
-	g_return_if_fail (is_nonempty_string(key));
-
-	/* chunk if necessary */
-	if (action==DO_CHUNK || action==DO_CHUNK_SHARE)
-		val = group_chunk_string (a->group, val, action==DO_CHUNK_SHARE);
-
-	/* if we're possibly erasing a header, remove the old value */
-	if (!init && !is_nonempty_string (val))
-		article_remove_header (a, key);
-
-	/* add the new header */
-	if (is_nonempty_string (val))
-	{
-		if      (!strcmp (key, HEADER_MESSAGE_ID)) a->message_id = val;
-		else if (!strcmp (key, HEADER_SUBJECT)) a->subject = val;
-		else if (!strcmp (key, HEADER_REFERENCES)) a->references = val;
-		else if (!strcmp (key, HEADER_XREF)) a->xref = val;
-		else if (!strcmp (key, HEADER_FROM))
-		{
-			if (init)
-				article_init_author_from_header (a, val);
-			else
-				article_set_author_from_header (a, val);
-		}
-		else pan_warn_if_reached ();
-	}
-}
-
-void
-article_init_header (Article         * a,
-                     const char      * key,
-                     const char      * val,
-                     HeaderAction      action)
-{
-	article_set_header_impl (a, key, val, action, TRUE);
-}
-
-void
-article_set_header (Article         * a,
-                    const char      * key,
-                    const char      * val,
-                    HeaderAction      action)
-{
-	article_set_header_impl (a, key, val, action, FALSE);
-}
-
 /***
 ****
-****  BODY
+****  OTHER HEADERS
 ****
 ***/
 
+int
+article_get_crosspost_qty (const Article  * article)
+{
+	int retval;
+
+	g_return_val_if_fail (article_is_valid (article), 0);
+
+	if (!is_nonempty_string (article->xref))
+		retval = 1;
+	else {
+		const char * pch;
+		retval = 0;
+		for (pch=article->xref; *pch; ++pch)
+			if (*pch==':')
+				++retval;
+	}
+
+	return retval;
+}
+
+gboolean
+article_header_is_internal (const char * key)
+{
+	return is_nonempty_string(key) && !strncmp(key,"X-Pan-Internal-",15);
+}
+
+gboolean
+article_header_is_extra (const char * key)
+{
+	/* sanity check */
+	if (!is_nonempty_string(key)) return FALSE;
+
+	/* pan internals aren't user-specified headers */
+	if (article_header_is_internal(key)) return FALSE;
+
+	/* other headers that are handled explicitly elsewhere */
+	if (!strcmp(key,HEADER_FOLLOWUP_TO)) return FALSE;
+	if (!strcmp(key,HEADER_NEWSGROUPS)) return FALSE;
+	if (!strcmp(key,HEADER_ORGANIZATION)) return FALSE;
+	if (!strcmp(key,HEADER_REPLY_TO)) return FALSE;
+	if (!strcmp(key,HEADER_DATE)) return FALSE;
+	if (!strcmp(key,HEADER_XREF)) return FALSE;
+	if (!strcmp(key,HEADER_FROM)) return FALSE;
+
+	return TRUE;
+}
+
+void
+article_set_subject (Article        * a,
+	             const char     * subject)
+{
+	g_return_if_fail (a!=NULL);
+	g_return_if_fail (is_nonempty_string(subject));
+
+	a->subject = (char*) group_chunk_string (a->group, subject, TRUE);
+}
+ 
+const char*
+article_get_subject (const Article * a)
+{
+	g_return_val_if_fail (article_is_valid(a), "");
+	return is_nonempty_string(a->subject) ? a->subject : "";
+}
+
+void
+article_set_message_id (Article        * a,
+	                const char     * message_id)
+{
+	g_return_if_fail (a!=NULL);
+	g_return_if_fail (is_nonempty_string(message_id));
+
+	a->message_id = (char*) group_chunk_string (a->group, message_id, TRUE);
+}
+
+const char*
+article_get_message_id (const Article * a)
+{
+	g_return_val_if_fail (article_is_valid(a), "");
+	return a->message_id;
+}
+
+void
+article_set_references (Article        * a,
+	                const char     * references)
+{
+	g_return_if_fail (a!=NULL);
+	g_return_if_fail (is_nonempty_string(references));
+
+	a->references = (char*) group_chunk_string (a->group, references, TRUE);
+}
+
+void
+article_set_xrefs (Article        * a,
+	           const char     * xrefs)
+{
+	g_return_if_fail (a!=NULL);
+	g_return_if_fail (is_nonempty_string(xrefs));
+
+	a->xref = (char*) group_chunk_string (a->group, xrefs, FALSE);
+}
+
+
 char*
 article_format_author_str (const char * author_addr,
                            const char * author_real,
                            char       * buf,
                            int          bufsize)
 {
-	const gboolean have_addr = is_nonempty_string (author_addr);
-	const gboolean have_name = is_nonempty_string (author_real);
-	const int name_len = have_name ? strlen (author_real) : 0;
-	const int addr_len = have_addr ? strlen (author_addr) : 0;
+	const int name_len = author_real ? strlen (author_real) : 0;
+	const int addr_len = author_addr ? strlen (author_addr) : 0;
 
 	*buf = '\0';
 
-	if (have_addr && have_name)
+	if (name_len && addr_len)
 	{
 		if (name_len + addr_len + 6 > bufsize)
 			g_snprintf (buf, bufsize, "\"%s\" <%s>", author_real, author_addr);
 		else
 		{
+			/* doing this by hand because profiling says it's a hotspot */
 			char * pch = buf;
 			*pch++ = '"';
 			memcpy (pch, author_real, name_len);
@@ -882,9 +851,9 @@ article_format_author_str (const char * author_addr,
 			*pch++ = '\0';
 		}
 	}
-	else if (have_addr)
+	else if (addr_len)
 		g_strlcpy (buf, author_addr, bufsize);
-	else if (have_name)
+	else if (name_len)
 		g_strlcpy (buf, author_real, bufsize);
 
 	g_strstrip (buf);
@@ -894,12 +863,11 @@ article_format_author_str (const char * author_addr,
 char*
 article_get_author_str (const Article * a, char * buf, int bufsize)
 {
-	*buf = '\0';
-
 	/* sanity clause */
 	g_return_val_if_fail (buf!=NULL, buf);
-	g_return_val_if_fail (a!=NULL, buf);
 	g_return_val_if_fail (bufsize>0, buf);
+	*buf = '\0';
+	g_return_val_if_fail (article_is_valid(a), buf);
 
 	return article_format_author_str (a->author_addr, a->author_real, buf, bufsize);
 }
@@ -944,10 +912,9 @@ article_get_short_author_str (const Article * a, char * buf, int len)
 	                                        buf, len);
 }
 
-static void
-article_set_author_from_header_impl (Article       * a,
-                                     const char    * header_from,
-                                     gboolean        init)
+void
+article_set_author (Article       * a,
+                    const char    * header_from)
 {
 	char addr_buf[512];
 	char real_buf[512];
@@ -990,22 +957,6 @@ article_set_author_from_header_impl (Article       * a,
 		const char * cpch = (const char*)default_incoming_name_real;
 		a->author_real = group_chunk_string (a->group, cpch, TRUE);
 	}
-
-	if (!init)
-		articles_set_dirty (&a, 1);
-}
-
-void
-article_init_author_from_header (Article         * a,
-                                 const char      * header_from)
-{
-	article_set_author_from_header_impl (a, header_from, TRUE);
-}
-void
-article_set_author_from_header (Article         * a,
-                                const char      * header_from)
-{
-	article_set_author_from_header_impl (a, header_from, FALSE);
 }
 
 void
@@ -1013,7 +964,7 @@ article_set_from_g_mime_message   (Article         * a,
                                    GMimeMessage    * msg)
 {
 	const char * cpch;
-	debug_enter ("article_set_from_raw_message");
+	debug_enter ("article_set_from_g_mime_message");
 
 	/* get the message */
 	g_return_if_fail (a != NULL);
@@ -1021,39 +972,31 @@ article_set_from_g_mime_message   (Article         * a,
 
 	g_mime_message_get_date (msg, &a->date, NULL);
 
-	cpch = g_mime_message_get_sender (msg);
-	if (cpch != NULL)
-		article_init_header (a, HEADER_FROM, cpch, DO_CHUNK_SHARE);
+	if (((cpch = g_mime_message_get_sender (msg))) != NULL)
+		article_set_author (a, cpch);
 
-	cpch = g_mime_message_get_reply_to (msg);
-	if (cpch != NULL)
-		article_init_header (a, HEADER_REPLY_TO, cpch, DO_CHUNK_SHARE);
+	if (((cpch = g_mime_message_get_subject (msg))) != NULL)
+		article_set_subject (a, cpch);
 
-	cpch = g_mime_message_get_subject (msg);
-	if (cpch != NULL)
-		article_init_header (a, HEADER_SUBJECT, cpch, DO_CHUNK_SHARE);
+	if (((cpch = g_mime_message_get_message_id (msg))) != NULL)
+		article_set_message_id (a, cpch);
 
-	cpch = g_mime_message_get_message_id (msg);
-	if (cpch != NULL)
-		article_init_header (a, HEADER_MESSAGE_ID, cpch, DO_CHUNK);
-
-	cpch = g_mime_message_get_header (msg, HEADER_REFERENCES);
-	if (cpch != NULL)
-		article_init_header (a, HEADER_REFERENCES, cpch, DO_CHUNK);
+	if (((cpch = g_mime_message_get_header (msg, HEADER_REFERENCES))) != NULL)
+		article_set_references (a, cpch);
 	
 	/* fallback if no message-id */
 	if (!is_nonempty_string(a->message_id)) {
 		char * id;
 		g_warning (_("Couldn't parse a Message-ID from the raw message!"));
 		id = gnksa_generate_message_id_from_email_addr (a->author_addr);
-		article_init_header (a, HEADER_MESSAGE_ID, id, DO_CHUNK);
+		article_set_message_id (a, id);
 		g_free (id);
-		article_init_header (a, HEADER_SUBJECT, _("Unparseable Subject"), DO_CHUNK_SHARE);
+		article_set_subject (a, _("Unparseable Subject"));
 	}
 
 
 	/* cleanup */
-	debug_exit ("article_set_from_raw_message");
+	debug_exit ("article_set_from_g_mime_message");
 }
 
 /****
@@ -1064,16 +1007,31 @@ gboolean
 article_is_valid (const Article * a)
 {
 	g_return_val_if_fail (a!=NULL, FALSE);
+
 	g_return_val_if_fail (a->group!=NULL, FALSE);
+	g_return_val_if_fail (a->group != DEAD_POINTER, FALSE);
 	g_return_val_if_fail (group_is_valid(a->group), FALSE);
 	g_return_val_if_fail (a->group->_articles_refcount>=0, FALSE);
 	g_return_val_if_fail (group_is_folder(a->group) || a->group->_articles_refcount>0, FALSE);
+
+	g_return_val_if_fail (a->message_id != DEAD_POINTER, FALSE);
 	g_return_val_if_fail (a->message_id!=NULL, FALSE);
 	g_return_val_if_fail (*a->message_id=='<', FALSE);
+
+	g_return_val_if_fail (a->subject != DEAD_POINTER, FALSE);
 	g_return_val_if_fail (is_nonempty_string(a->subject), FALSE);
-	g_return_val_if_fail (a->number!=0, FALSE);
+
+	g_return_val_if_fail (a->references != DEAD_POINTER, FALSE);
 	g_return_val_if_fail (!a->references || *a->references=='<', FALSE);
 
+	g_return_val_if_fail (a->number!=0, FALSE);
+
+	g_return_val_if_fail (a->threads != DEAD_POINTER, FALSE);
+	g_return_val_if_fail (a->parent != DEAD_POINTER, FALSE);
+	g_return_val_if_fail (a->xref != DEAD_POINTER, FALSE);
+	g_return_val_if_fail (a->author_addr != DEAD_POINTER, FALSE);
+	g_return_val_if_fail (a->author_real != DEAD_POINTER, FALSE);
+
 	return TRUE;
 }
 
@@ -1093,45 +1051,24 @@ articles_are_valid (const Article ** a, int qty)
 	return TRUE;
 }
 
-const char*
-article_get_message_id (const Article * a)
+gboolean
+articles_are_valid_in_group (const Article ** a, int qty)
 {
-	g_return_val_if_fail (article_is_valid(a), "");
-	return a->message_id;
-}
- 
-const char*
-article_get_subject (const Article * a)
-{
-	g_return_val_if_fail (article_is_valid(a), "");
-	return is_nonempty_string(a->subject) ? a->subject : "";
-}
+	int i;
 
+	/* sanity clause */
+	g_return_val_if_fail (articles_are_valid(a,qty), FALSE);
 
-gulong
-article_get_combined_linecount (const Article * a)
-{
-	gulong retval = 0;
-
-	g_return_val_if_fail (article_is_valid(a), FALSE);
-
-	retval = a->linecount;
-	if (a->part!=0 && a->parts!=0 && a->threads!=NULL) {
-		GSList * l;
-		int part = a->part + 1;
-		for (l=a->threads; l; l=l->next) {
-			Article * child = ARTICLE(l->data);
-			if (child->part == part) {
-				retval += child->linecount;
-				++part;
-			}
-		}
+	/* check each article */
+	if (qty > 0) {
+		Group * group = a[0]->group;
+		for (i=0; i<qty; ++i)
+			g_return_val_if_fail (a[i]->group == group, FALSE);
 	}
 
-	return retval;
+	return TRUE;
 }
 
-
 /***
 ****  EVENTS
 ***/
