@@ -352,19 +352,19 @@ decode_article (const decode_data * dd)
 
 		if (group!=NULL && group_ref_articles_if_loaded (group)) {
 			int qty = 0;
-			Article ** articles = g_alloca (sizeof(Article*) * dd->mid_qty);
+			Article ** articles = g_newa (Article*, dd->mid_qty);
 			for (i=0; i<dd->mid_qty; ++i) {
 				Article * a = group_get_article_by_message_id (group, dd->mids[i]->message_id);
 				if (a != NULL)
 					articles[qty++] = a;
 			}
-			articles_add_flag (articles, qty, (success?STATE_DECODED:STATE_DECODE_FAILED));
+			articles_set_decode_state (articles, qty, success ? DECODE_STATE_DECODED : DECODE_STATE_FAILED);
 			group_unref_articles (group, NULL);
 		}
 	}
 
 	if (success)
-		message_identifiers_mark_read (dd->mids, dd->mid_qty, TRUE, SERVER_GROUPS_SUBSCRIBED);
+		message_identifiers_mark_read ((const MessageIdentifier**)dd->mids, dd->mid_qty, TRUE, SERVER_GROUPS_SUBSCRIBED);
 
 	/* cleanup some more */
 	if (mm != NULL)
