@@ -131,7 +131,7 @@ read_group_data (Server        * server,
 	groups = g_ptr_array_new ();
 	if (version==1 || version==2 || version==3 || version==4 || 
 	    version==5 || version==6 || version==7 || version==8 ||
-	    version==9 || version==10 || version==11 || version==12)
+	    version==9 || version==10 || version==11 || version==12 || version==13)
 	{
 		long i;
 		long qty = get_next_token_long (march_idx, '\n', &march_idx);
@@ -207,7 +207,14 @@ read_group_data (Server        * server,
 				g->filter_show = get_next_token_ulong (march_idx, '\n', &march_idx);
 			}
 
-			g->filter_bits = (guint32) get_next_token_ulong (march_idx, '\n', &march_idx);
+			/* the filter bitfields changed between versions 12 and 13 ...
+			    reset the fields for file format versions < 13 */
+			if (version >= 13) {
+				g->filter_bits = (guint32) get_next_token_ulong (march_idx, '\n', &march_idx);
+			} else {
+				skip_next_token (march_idx, '\n', &march_idx);
+				g->filter_bits = ~STATE_FILTER_SCORE_IGNORED;
+			}
 
 			/* old sort style */
 			if (version >= 10)
@@ -412,7 +419,7 @@ write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * sta
 
 	/* Write the group information... */
 	pos = 0;
-	fprintf (idx_fp, "12\n"); /* file format version number */
+	fprintf (idx_fp, "13\n"); /* file format version number */
 	fprintf (idx_fp, "%u\n", groups->len); /* number of groups */
 	for (i=0; i!=groups->len; ++i)
 	{
@@ -476,6 +483,10 @@ write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * sta
 static void
 save_groups_to_file (Server * server, GPtrArray * groups, gchar * filemarker, StatusItem * status)
 {
+	char idx_fname[PATH_MAX];
+	char dat_fname[PATH_MAX];
+	char tmp_idx_fname[PATH_MAX];
+	char tmp_dat_fname[PATH_MAX];
 	debug_enter ("save_groups_to_file");
 
 	/* sanity clause */
@@ -483,29 +494,27 @@ save_groups_to_file (Server * server, GPtrArray * groups, gchar * filemarker, St
 	g_return_if_fail (groups!=NULL);
 	g_return_if_fail (is_nonempty_string(filemarker));
 
+	/* build the filenames */
+	g_snprintf (idx_fname, sizeof(idx_fname), "%s%c%s_%s.idx", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
+	g_snprintf (dat_fname, sizeof(dat_fname), "%s%c%s_%s.dat", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
+	g_snprintf (tmp_idx_fname, sizeof(tmp_idx_fname), "%s.tmp", idx_fname);
+	g_snprintf (tmp_dat_fname, sizeof(tmp_dat_fname), "%s.tmp", dat_fname);
+
 	if (groups->len == 0)
 	{
-		gchar path [PATH_MAX];
-		g_snprintf (path, PATH_MAX, "%s%c%s_%s.idx", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
-		remove (path);
-		g_snprintf (path, PATH_MAX, "%s%c%s_%s.dat", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
-		remove (path);
+		unlink (idx_fname);
+		unlink (dat_fname);
 	}
 	else
 	{
-		FILE * idx_fp = NULL;
-		FILE * dat_fp = NULL;
-		char * idx_path = pan_file_make_temp (&idx_fp);
-		char * dat_path = pan_file_make_temp (&dat_fp);
+		FILE * idx_fp = fopen (tmp_idx_fname, "w+");
+		FILE * dat_fp = fopen (tmp_dat_fname, "w+");
 		gboolean ok;
 
 		/* make sure that we were able to create the temp files */
-		if (idx_fp==NULL || dat_fp==NULL)
-		{
+		if (idx_fp==NULL || dat_fp==NULL) {
 			fclose (idx_fp);
 			fclose (dat_fp);
-			g_free (idx_path);
-			g_free (dat_path);
 			g_return_if_fail (0);
 		}
 
@@ -515,29 +524,15 @@ save_groups_to_file (Server * server, GPtrArray * groups, gchar * filemarker, St
 		fclose (idx_fp);
 		fclose (dat_fp);
 
-		/* rename the temp files (on success) or delete them (on error) */
-		if (!ok)
-		{
+		/* swap the temp files (on success) or delete them (on error) */
+		if (ok) {
+			pan_file_swap_datafile (tmp_idx_fname, idx_fname);
+			pan_file_swap_datafile (tmp_dat_fname, dat_fname);
+		} else {
 			log_add_va (LOG_ERROR, _("Error saving list of groups to \"%s\" - is the disk full?"), get_data_dir());
-			unlink (idx_path);
-			unlink (dat_path);
+			unlink (tmp_idx_fname);
+			unlink (tmp_dat_fname);
 		}
-		else
-		{
-			char path [PATH_MAX];
-
-			g_snprintf (path, PATH_MAX, "%s%c%s_%s.idx", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
-			pan_file_normalize_inplace (path);
-			pan_file_rename (idx_path, path);
-
-			g_snprintf (path, PATH_MAX, "%s%c%s_%s.dat", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
-			pan_file_normalize_inplace (path);
-			pan_file_rename (dat_path, path);
-		}
-
-		/* cleanup */
-		g_free (idx_path);
-		g_free (dat_path);
 	}
 
 	debug_exit ("save_groups_to_file");
@@ -595,3 +590,47 @@ file_grouplist_save (Server * server, ServerGroupsType group_type, StatusItem *
 
 	debug_exit ("file_grouplist_save");
 }
+
+/***
+****
+***/
+
+void
+file_grouplist_server_name_changed  (const char * old_name,
+                                     const char * new_name)
+{
+	char old_path [PATH_MAX];
+	char new_path [PATH_MAX];
+
+	/* sanity clause */
+	g_return_if_fail (is_nonempty_string (old_name));
+	g_return_if_fail (is_nonempty_string (new_name));
+
+	/* rename the files: sub idx */
+	g_snprintf (old_path, sizeof(old_path), "%s%c%s_sub.idx", get_data_dir(), G_DIR_SEPARATOR, old_name);
+	if (pan_file_exists (old_path)) {
+		g_snprintf (new_path, sizeof(new_path), "%s%c%s_sub.idx", get_data_dir(), G_DIR_SEPARATOR, new_name);
+		pan_file_rename (old_path, new_path);
+	}
+
+	/* rename the files: sub dat */
+	g_snprintf (old_path, sizeof(old_path), "%s%c%s_sub.dat", get_data_dir(), G_DIR_SEPARATOR, old_name);
+	if (pan_file_exists (old_path)) {
+		g_snprintf (new_path, sizeof(new_path), "%s%c%s_sub.dat", get_data_dir(), G_DIR_SEPARATOR, new_name);
+		pan_file_rename (old_path, new_path);
+	}
+
+	/* rename the files: unsub idx */
+	g_snprintf (old_path, sizeof(old_path), "%s%c%s_unsub.idx", get_data_dir(), G_DIR_SEPARATOR, old_name);
+	if (pan_file_exists (old_path)) {
+		g_snprintf (new_path, sizeof(new_path), "%s%c%s_unsub.idx", get_data_dir(), G_DIR_SEPARATOR, new_name);
+		pan_file_rename (old_path, new_path);
+	}
+
+	/* rename the files: unsub dat */
+	g_snprintf (old_path, sizeof(old_path), "%s%c%s_unsub.dat", get_data_dir(), G_DIR_SEPARATOR, old_name);
+	if (pan_file_exists (old_path)) {
+		g_snprintf (new_path, sizeof(new_path), "%s%c%s_unsub.dat", get_data_dir(), G_DIR_SEPARATOR, new_name);
+		pan_file_rename (old_path, new_path);
+	}
+}
