@@ -20,15 +20,16 @@
 #ifndef __FILE_GROUPLIST_H__
 #define __FILE_GROUPLIST_H__
 
-#include <glib.h>
-
 #include <pan/base/server.h>
 #include <pan/base/status-item.h>
 
-ServerGroupsType   file_grouplist_exists           (const Server * server);
+ServerGroupsType   file_grouplist_exists               (const Server * server);
 
-void               file_grouplist_load             (Server * server, ServerGroupsType, StatusItem * status);
+void               file_grouplist_load                 (Server * server, ServerGroupsType, StatusItem * status);
+
+void               file_grouplist_save                 (Server * server, ServerGroupsType, StatusItem * status);
+
+void               file_grouplist_server_name_changed  (const char * old_name, const char * new_name);
 
-void               file_grouplist_save             (Server * server, ServerGroupsType, StatusItem * status);
 
 #endif
