@@ -133,7 +133,7 @@ file_headers_load_group (Group * group, StatusItem * status)
 		const char * march = idx;
 		const glong version = get_next_token_int (march, '\n', &march);
 
-		if (version==1 || version==2 || version==3 || version==4 || version==5 || version==6 || version==7 || version==8 || version==9)
+		if (version==1 || version==2 || version==3 || version==4 || version==5 || version==6 || version==7 || version==8 || version==9 || version==10)
 		{
 			int i;
 			int purged_article_count = 0;
@@ -161,7 +161,7 @@ file_headers_load_group (Group * group, StatusItem * status)
 				{
 					l = get_next_token_long (march, '\n', &march);
 					if (0<=l && l<dat_len)
-						article_init_author_from_header (a, dat+l);
+						article_set_author (a, dat+l);
 				}
 				else
 				{
@@ -207,9 +207,9 @@ file_headers_load_group (Group * group, StatusItem * status)
 				if (version<6)
 					skip_next_token (march, '\n', &march);
 
-				/* state - removed in version 6, back in 7 */
-				if (version!=6)
-					a->state  = (guint16) get_next_token_int (march, '\n', &march);
+				/* state - removed in version 6, back in 7, removed in 10 */
+				if (version!=6 && version<10)
+					skip_next_token (march, '\n', &march);
 
 				a->date           = (time_t) get_next_token_ulong (march, '\n', &march);
 				a->number         = (gulong) get_next_token_ulong (march, '\n', &march);
@@ -425,7 +425,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 	}
 
 	/* Write DATBASE_VERSION */
-	fprintf (idx_fp, "9\n%ld\n", (long)articles->len);
+	fprintf (idx_fp, "10\n%ld\n", (long)articles->len);
 
 	/* Write the article information... */
 	pos = 0;
@@ -451,7 +451,6 @@ file_headers_save_group (Group * group, StatusItem * status)
 			"%d\n" "%d\n"
 			"%u\n"
 			"%lu\n"
-			"%d\n"
 			"%lu\n"
 			"%lu\n"
 			"%d\n",
@@ -459,7 +458,6 @@ file_headers_save_group (Group * group, StatusItem * status)
 			(int)a->part, (int)a->parts,
 			(unsigned int)a->linecount,
 			(unsigned long)a->byte_qty,
-			(int)a->state,
 			(unsigned long)a->date,
 			(unsigned long)a->number,
 			(int)(a->is_new != 0));
@@ -604,3 +602,30 @@ file_headers_destroy (const Group * group)
 
 	debug_exit ("file_headers_destroy");
 }
+
+/***
+****
+***/
+
+void
+file_headers_server_name_changed  (const char * old_name,
+                                   const char * new_name)
+{
+	char * old_path;
+
+	/* sanity clause */
+	g_return_if_fail (is_nonempty_string (old_name));
+	g_return_if_fail (is_nonempty_string (new_name));
+
+	/* rename the server's datafile directory */
+	old_path = g_build_filename (get_data_dir(), old_name, NULL);
+	if (g_file_test (old_path, G_FILE_TEST_IS_DIR))
+	{
+		char * new_path = g_build_filename (get_data_dir(), new_name, NULL);
+		pan_file_rename (old_path, new_path);
+		g_free (new_path);
+	}
+
+	/* cleanup */
+	g_free (old_path);
+}
