@@ -21,21 +21,20 @@
 #define __FILE_HEADERS_H__
 
 #include <stdio.h>
-#include <glib.h>
-
 #include <gmime/memchunk.h>
-
 #include <pan/base/group.h>
 #include <pan/base/status-item.h>
 
-void     file_headers_destroy          (const Group * group);
+void     file_headers_server_name_changed  (const char * old_name, const char * new_name);
 
-void     file_headers_load             (Group * group, StatusItem * status);
+void     file_headers_destroy              (const Group * group);
 
-void     file_headers_save             (Group * group, StatusItem * status);
+void     file_headers_load                 (Group * group, StatusItem * status);
 
-void     file_headers_save_noref       (Group * group, StatusItem * status);
+void     file_headers_save                 (Group * group, StatusItem * status);
 
-long     get_string_offset             (GHashTable *, MemChunk *, const gchar *, FILE *, glong *);
+void     file_headers_save_noref           (Group * group, StatusItem * status);
+
+long     get_string_offset                 (GHashTable *, MemChunk *, const char *, FILE *, long *);
 
 #endif
