@@ -20,7 +20,7 @@
 #ifndef __GNKSA_H__
 #define __GNKSA_H__
 
-#include <glib.h>
+#include <glib/gtypes.h>
 #include <time.h>
 
 enum
@@ -95,4 +95,17 @@ char*        gnksa_generate_message_id                   (const char   * domain_
 time_t       tzoffset_sec                                (time_t       * time);
 
 
+typedef enum
+{
+   SIG_NONE,
+   SIG_STANDARD,
+   SIG_NONSTANDARD
+}
+SigType;
+
+SigType   pan_is_signature_delimiter (const char       * line,
+                                      const int          line_len);
+
+gboolean  pan_remove_signature       (char             * body);
+
 #endif
