@@ -24,7 +24,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#include <glib.h>
+#include <glib/gthread.h>
 
 #include <gmime/gmime-stream-file.h>
 
@@ -73,7 +73,7 @@ group_constructor (Group                * g,
 	g->flags = (gint8)0;
 	g->old_sort_style = (gint8)0;
 	g->new_sort_style = (gint8)0;
-	g->filter_bits = ~((guint16)STATE_FILTER_IGNORED);
+	g->filter_bits = ~((guint16)STATE_FILTER_SCORE_IGNORED);
 	g->filter_show = FILTER_SHOW_MATCHES_AND_REFERENCES;
 	g->article_qty = (gint32)0;
 	g->article_read_qty = (gint32)0;
@@ -115,6 +115,37 @@ destroy_articles_ghfunc (gpointer key, gpointer val, gpointer user_data)
 	article_destructor (ARTICLE(val));
 }
 
+static void
+blow_article_memory (Group * g)
+{
+	g_return_if_fail (g!=NULL);
+	pan_warn_if_fail (g->_articles_refcount >= 0);
+
+	replace_gstr (&g->_one_big_chunk, NULL);
+
+	if (g->_articles != NULL) {
+		g_hash_table_foreach (g->_articles, destroy_articles_ghfunc, NULL);
+		g_hash_table_destroy (g->_articles);
+		g->_articles = NULL;
+	}
+
+	if (g->_deleted_articles != NULL) {
+		pan_g_ptr_array_foreach (g->_deleted_articles, (GFunc)article_destructor, NULL);
+		g_ptr_array_free (g->_deleted_articles, TRUE);
+		g->_deleted_articles = NULL;
+	}
+
+	if (g->chunk != NULL) {
+		g_string_chunk_free (g->chunk);
+		g->chunk = NULL;
+	}
+
+	if (g->_article_chunk != NULL) {
+		memchunk_destroy (g->_article_chunk);
+		g->_article_chunk = NULL;
+	}
+}
+
 void
 group_destructor (Group * o)
 {
@@ -139,26 +170,7 @@ group_destructor (Group * o)
 		pan_object_unref (PAN_OBJECT(g->_purged));
 
 	/* articles */
-	pan_warn_if_fail (g->_articles_refcount >= 0);
-	replace_gstr (&g->_one_big_chunk, NULL);
-	if (g->_articles != NULL) {
-		g_hash_table_foreach (g->_articles, destroy_articles_ghfunc, NULL);
-		g_hash_table_destroy (g->_articles);
-		g->_articles = NULL;
-	}
-	if (g->_deleted_articles != NULL) {
-		pan_g_ptr_array_foreach (g->_deleted_articles, (GFunc)article_destructor, NULL);
-		g_ptr_array_free (g->_deleted_articles, TRUE);
-		g->_deleted_articles = NULL;
-	}
-	if (g->chunk != NULL) {
-		g_string_chunk_free (g->chunk);
-		g->chunk = NULL;
-	}
-	if (g->_article_chunk != NULL) {
-		memchunk_destroy (g->_article_chunk);
-		g->_article_chunk = NULL;
-	}
+	blow_article_memory (g);
 	if (g->article_mutex != NULL) {
 		g_mutex_free (g->article_mutex);
 		g->article_mutex = NULL;
@@ -191,40 +203,51 @@ group_is_new (const Group * group)
 }
 
 void
-groups_set_subscribed (Group ** groups, gint qty, gboolean subscribed)
+groups_set_subscribed (Group ** groups, int qty, gboolean subscribed)
 {
-	gint i;
-	ServerGroupsType type;
+	Group ** changed;
 	Server * server = NULL;
+	const ServerGroupsType type = SERVER_GROUPS_SUBSCRIBED|SERVER_GROUPS_UNSUBSCRIBED;
+	int changed_qty = 0;
+	int i;
 	debug_enter ("groups_set_subscribed");
 
 	/* sanity clause */
 	g_return_if_fail (groups != NULL);
 	g_return_if_fail (qty > 0);
 
-	type = SERVER_GROUPS_SUBSCRIBED|SERVER_GROUPS_UNSUBSCRIBED;
-
+	/* change the subscription state of any group that needs it */
+	changed = g_new (Group*, qty);
 	for (i=0; i<qty; ++i)
 	{
 		Group * group = groups[i];
+		const gboolean is_subscribed = (group->flags & GROUP_SUBSCRIBED) != 0;
 
-		if (group->server != NULL)
+		if (is_subscribed != subscribed)
 		{
-			server = group->server;
-			server_set_group_type_dirty (server, type);
-		}
+			if (group->server != NULL)
+			{
+				server = group->server;
+				server_set_group_type_dirty (server, type);
+			}
 
-		if (subscribed)
-			group_set_flags (group, group->flags|GROUP_SUBSCRIBED);
-		else
-			group_set_flags (group, group->flags&~GROUP_SUBSCRIBED);
+			if (subscribed)
+				group_set_flags (group, group->flags | GROUP_SUBSCRIBED);
+			else
+				group_set_flags (group, group->flags & ~GROUP_SUBSCRIBED);
+
+			changed[changed_qty++] = group;
+		}
 	}
 
 	if (server != NULL)
 		server_save_grouplist_if_dirty (server, NULL);
 
-	fire_groups_changed (groups, qty);
+	if (changed_qty > 0)
+		fire_groups_changed (changed, changed_qty);
 
+	/* cleanup */
+	g_free (changed);
 	debug_exit ("groups_set_subscribed");
 }
 
@@ -241,8 +264,7 @@ group_set_dirty (Group * group)
 {
 	g_return_if_fail (group_is_valid(group));
 
-	if (group->server != NULL)
-		server_set_group_type_dirty (group->server, server_get_group_type (group));
+	server_set_group_type_dirty (group->server, server_get_group_type (group));
 }
 
 void
@@ -567,7 +589,7 @@ group_empty (Group * group, gboolean clear_counts)
 
 	/* remove the articles from memory */
 	a = group_get_article_array (group);
-	group_remove_articles (group, a);
+	group_remove_articles (group, (Article**)a->pdata, a->len);
 	g_ptr_array_free (a, TRUE);
 
 	/* reset the counters */
@@ -618,7 +640,12 @@ folder_add_message (Group            * folder,
 			article_set_from_g_mime_message (a, message);
 
 			if (article_is_valid (a))
-				group_add_article (folder, a);
+			{
+				GPtrArray * articles = g_ptr_array_new ();
+				g_ptr_array_add (articles, a);
+				group_add_articles (folder, articles, NULL, NULL, NULL);
+				g_ptr_array_free (articles, TRUE);
+			}
 
 			group_unref_articles (folder, NULL);
 		}
@@ -650,7 +677,7 @@ folder_remove_message  (Group         * folder,
 	else {
 		Article * a = group_get_article_by_message_id (folder, message_id);
 		if (a != NULL)
-			group_remove_article (folder, a);
+			group_remove_articles (folder, &a, 1);
 		group_unref_articles (folder, NULL);
 	}
 }
@@ -660,20 +687,6 @@ folder_remove_message  (Group         * folder,
 ****
 ***/
 
-void
-group_add_article (Group * group, Article * article)
-{
-	GPtrArray * a;
-	debug_enter ("group_add_article");
-
-	a = g_ptr_array_new ();
-	g_ptr_array_add (a, article);
-	group_add_articles (group, a, NULL, NULL, NULL);
-	g_ptr_array_free (a, TRUE);
-
-	debug_exit ("group_add_article");
-}
-
 static void
 group_add_articles_impl (Group             * group,
                          const GPtrArray   * articles,
@@ -693,16 +706,14 @@ group_add_articles_impl (Group             * group,
 	Newsrc * purged;
 	debug_enter ("group_add_articles");
 
-	/* sanity checks */
+	/* sanity clause */
 	g_return_if_fail (group_is_valid(group));
 	g_return_if_fail (group->_articles_refcount>=0);
 	g_return_if_fail (articles!=NULL);
 	if (!articles->len) return;
 	g_return_if_fail (articles->len>0);
-	g_return_if_fail (articles_are_valid ((const Article**)articles->pdata, articles->len));
-	for (i=0; i<articles->len; ++i)
-		g_return_if_fail (ARTICLE(g_ptr_array_index(articles,i))->group == group);
-
+	g_return_if_fail (articles_are_valid_in_group ((const Article**)articles->pdata, articles->len));
+	g_return_if_fail (ARTICLE(g_ptr_array_index(articles,0))->group == group);
 
 	/**
 	***  Get a handle on our current stats...
@@ -729,10 +740,10 @@ group_add_articles_impl (Group             * group,
 	for (i=0; i!=articles->len; ++i)
 	{
 		Article * a = ARTICLE(g_ptr_array_index(articles,i));
-		const gchar * message_id = article_get_message_id (a);
-		gboolean is_purged = newsrc_is_article_read(purged, a->number);
+		const char * message_id = article_get_message_id (a);
+		const gboolean is_purged = newsrc_is_article_read(purged, a->number);
 
-		if (!is_purged && g_hash_table_lookup(ours,message_id) == NULL)
+		if (!is_purged && g_hash_table_lookup(ours,message_id)==NULL)
 		{
 			a->group = group;
 			g_hash_table_insert (ours, (gpointer)message_id, a);
@@ -872,21 +883,7 @@ group_get_article_by_message_id (Group * group, const char * message_id)
 }
 
 void
-group_remove_article (Group * group, Article * article)
-{
-	GPtrArray * a;
-
-	g_return_if_fail (group_is_valid(group));
-	g_return_if_fail (article_is_valid(article));
-
-       	a = g_ptr_array_new ();
-	g_ptr_array_add (a, article);
-	group_remove_articles (group, a);
-	g_ptr_array_free (a, TRUE);
-}
-
-void
-group_remove_articles (Group * group, const GPtrArray * articles)
+group_remove_articles (Group * group, Article ** articles, int article_qty)
 {
 	guint i;
 	int removed_read;
@@ -895,17 +892,18 @@ group_remove_articles (Group * group, const GPtrArray * articles)
 	GHashTable * ours;
 	debug_enter ("group_remove_articles");
 
-	g_return_if_fail (group_is_valid(group));
-
 	/* sanity clause */
-	g_return_if_fail (group != NULL);
-	g_return_if_fail (articles != NULL);
-	if (!articles->len) return;
-	for (i=0; i!=articles->len; ++i)
-		g_return_if_fail (ARTICLE(g_ptr_array_index(articles,i))->group==group);
+	g_return_if_fail (group_is_valid (group));
+	g_return_if_fail (group->_articles_refcount>=0);
+	g_return_if_fail (articles!=NULL);
+	if (article_qty == 0) return;
+	g_return_if_fail (article_qty > 0);
+	g_return_if_fail (articles_are_valid_in_group ((const Article **)articles, article_qty));
+	g_return_if_fail (articles[0]->group == group);
 
 	/* let everyone know that articles are going to be removed */
-	tmp_remove = pan_g_ptr_array_dup ((GPtrArray*)articles);
+	tmp_remove = g_ptr_array_new ();
+	pan_g_ptr_array_assign (tmp_remove, (gpointer*)articles, article_qty);
 	sort_articles ((Article**)tmp_remove->pdata, tmp_remove->len, ARTICLE_SORT_MSG_ID, TRUE);
 	pan_callback_call (group_get_articles_removed_callback(), group, tmp_remove);
 
@@ -916,7 +914,7 @@ group_remove_articles (Group * group, const GPtrArray * articles)
 	g_mutex_lock (group_get_article_mutex (group));
 	ours = group_get_articles (group);
 	removed_read = 0;
-	removed = g_ptr_array_new ();
+	removed = g_ptr_array_sized_new (article_qty);
 	for (i=0; i!=tmp_remove->len; ++i)
 	{
 		Article * a = ARTICLE(g_ptr_array_index(tmp_remove,i));
@@ -955,12 +953,25 @@ group_remove_articles (Group * group, const GPtrArray * articles)
 			Article * a = ARTICLE(g_ptr_array_index(rethread_a, i));
 			g_hash_table_insert (rethread_hash, a, a);
 		}
+
 		/* remove the deleted articles from this rethread list */
 		for (i=0; i!=removed->len; ++i) {
 			Article * a = ARTICLE(g_ptr_array_index(removed, i));
 			g_hash_table_remove (rethread_hash, a);
 		}
 
+		/* isolate the deleted articles, in case any clients still
+		 * have handles to these deleted articles, because of some
+		 * workproc, etc. */
+		for (i=0; i!=removed->len; ++i) {
+			Article * a = ARTICLE(g_ptr_array_index(removed, i));
+			a->parent = NULL;
+			if (a->threads != NULL) {
+				g_slist_free (a->threads);
+				a->threads = NULL;
+			}
+		}
+
 		/* rethread the affected articles */
 		if (g_hash_table_size(rethread_hash) > 0) {
 			pan_hash_to_ptr_array (rethread_hash, rethread_a);
@@ -1024,7 +1035,7 @@ group_expire_articles_not_in_range (Group * g, gulong low, gulong high)
 			removeme->len,
 			group_get_name(g));
 
-		group_remove_articles (g, removeme);
+		group_remove_articles (g, (Article**)removeme->pdata, removeme->len);
 	}
 
 	/* cleanup */
@@ -1099,28 +1110,7 @@ group_unref_articles (Group * group, StatusItem * status)
 		g_mutex_lock (group_get_article_mutex (group));
 		file_headers_save_noref (group, status);
 		server_save_grouplist_if_dirty (group->server, status);
-
-		if (group->_articles!=NULL) {
-		       	g_hash_table_foreach (group->_articles, destroy_articles_ghfunc, NULL);
-			g_hash_table_destroy (group->_articles);
-			group->_articles = NULL;
-		}
-		if (group->_deleted_articles != NULL) {
-			pan_g_ptr_array_foreach (group->_deleted_articles, (GFunc)article_destructor, NULL);
-			g_ptr_array_free (group->_deleted_articles, TRUE);
-			group->_deleted_articles = NULL;
-		}
-		if (group->_article_chunk != NULL) {
-			memchunk_destroy (group->_article_chunk);
-			group->_article_chunk = NULL;
-		}
-
-		/* free the memory used by those articles */
-		replace_gstr (&group->_one_big_chunk, NULL);
-		if (group->chunk != NULL) {
-			g_string_chunk_free (group->chunk);
-			group->chunk = NULL;
-		}
+		blow_article_memory (group);
 		g_mutex_unlock (group_get_article_mutex (group));
 	}
 	--group->_articles_refcount;
@@ -1250,87 +1240,6 @@ group_get_purged (Group * group)
 	return group->_purged;
 }
 
-static gint
-compare_ppA_to_ppA_by_number (const void * va, const void * vb)
-{
-	const Article * a = *(const Article **)va;
-	const Article * b = *(const Article **)vb;
-	int retval;
-
-	if (a->number < b->number)
-		retval = -1;
-	else if (a->number > b->number)
-		retval = 1;
-	else
-		retval = 0;
-
-	return retval;
-}
-
-void
-group_remove_crossposts (Group     * g,
-		         gulong    * numbers,
-                         int         number_qty)
-{
-	/* sanity clause */
-	g_return_if_fail (group_is_valid(g));
-	g_return_if_fail (numbers!=NULL);
-	g_return_if_fail (number_qty>0);
-
-	/* if that group's articles are loaded, remove the corresponding Articles */
-	if (group_ref_articles_if_loaded (g))
-	{
-		int i;
-		GPtrArray * articles;
-		GPtrArray * remove;
-
-		/* get a list of articles sorted by number for fast lookup */
-		remove = g_ptr_array_sized_new (number_qty);
-		articles = group_get_article_array (g);
-		qsort (articles->pdata, articles->len, sizeof(Article*), compare_ppA_to_ppA_by_number);
-
-		/* for each number to remove, if we've got it, add it to the "remove" ptr array */
-		for (i=0; i<number_qty; ++i)
-		{
-			Article tmp;
-			gint index;
-			gboolean exact_match = FALSE;
-
-			tmp.number = numbers[i];
-			index = lower_bound (&tmp,
-			                     articles->pdata,
-			                     articles->len,
-			                     sizeof(Article*),
-			                     compare_ppA_to_ppA_by_number,
-			                     &exact_match);
-
-			if (exact_match) {
-				Article * a = ARTICLE(g_ptr_array_index(articles,index));
-				g_ptr_array_add (remove, a);
-			}
-		}
-
-		if (remove->len != 0)
-			group_remove_articles (g, remove);
-
-		g_ptr_array_free (remove, TRUE);
-		g_ptr_array_free (articles, TRUE);
-		group_unref_articles (g, NULL);
-	}
-
-	/* flag articles for deletion in case they get downloaded later */
-	if (1)
-	{
-		guint i;
-		Newsrc * n;
-		
-		n = group_get_purged (g);
-		for (i=0; i<number_qty; ++i) {
-			newsrc_mark_article (n, numbers[i], TRUE);
-		}
-	}
-}
-
 struct _Article*
 group_alloc_new_article (Group * group)
 {
