@@ -20,7 +20,9 @@
 #ifndef __GROUPS_H__
 #define __GROUPS_H__
 
-#include <glib.h>
+#include <glib/gtypes.h>
+#include <glib/garray.h>
+#include <glib/ghash.h>
 
 #include <gmime/memchunk.h>
 #include <gmime/gmime-message.h>
@@ -48,23 +50,30 @@ enum
 
 enum
 {
-        STATE_FILTER_UNREAD                     = (1<<0),
-        STATE_FILTER_DEPRECATED_DONT_USE        = (1<<1),
-        STATE_FILTER_WATCHED                    = (1<<2),
-        STATE_FILTER_IGNORED                    = (1<<3),
-        STATE_FILTER_NORMAL_RANK                = (1<<4),
-        STATE_FILTER_COMPLETE_BINARIES          = (1<<5),
-        STATE_FILTER_INCOMPLETE_BINARIES        = (1<<6),
-        STATE_FILTER_NONBINARIES                = (1<<7),
-        STATE_FILTER_SAVED                      = (1<<8),
-        STATE_FILTER_QUEUED                     = (1<<9),
-        STATE_FILTER_IDLE                       = (1<<10),
-	STATE_FILTER_NEW			= (1<<11),
-	STATE_FILTER_CACHED			= (1<<12),
-	STATE_FILTER_NOT_CACHED			= (1<<13),
-	STATE_FILTER_READ                       = (1<<14),
-	STATE_FILTER_MINE                       = (1<<15),
-	STATE_FILTER_NOT_MINE                   = (1<<16)
+	STATE_FILTER_NEW                   = (1<<0),
+        STATE_FILTER_UNREAD                = (1<<1),
+	STATE_FILTER_READ                  = (1<<2),
+        STATE_FILTER_COMPLETE_BINARIES     = (1<<3),
+        STATE_FILTER_INCOMPLETE_BINARIES   = (1<<4),
+        STATE_FILTER_NONBINARIES           = (1<<5),
+	STATE_FILTER_CACHED                = (1<<6),
+	STATE_FILTER_NOT_CACHED            = (1<<7),
+	STATE_FILTER_MINE                  = (1<<8),
+	STATE_FILTER_NOT_MINE              = (1<<9),
+
+        STATE_FILTER_SCORE_WATCHED         = (1<<10),
+        STATE_FILTER_SCORE_HIGH            = (1<<11),
+        STATE_FILTER_SCORE_MEDIUM          = (1<<12),
+        STATE_FILTER_SCORE_ZERO            = (1<<13),
+	STATE_FILTER_SCORE_LOW             = (1<<14),
+	STATE_FILTER_SCORE_IGNORED         = (1<<15),
+	STATE_FILTER_SCORE_MASK            = (STATE_FILTER_SCORE_WATCHED
+	                                     | STATE_FILTER_SCORE_HIGH
+	                                     | STATE_FILTER_SCORE_MEDIUM
+	                                     | STATE_FILTER_SCORE_ZERO
+	                                     | STATE_FILTER_SCORE_LOW
+	                                     | STATE_FILTER_SCORE_IGNORED),
+        STATE_FILTER_SAVED                 = (1<<16)
 };
 
 #define GROUP(x) ((Group*)(x))
@@ -227,9 +236,6 @@ void          folder_remove_message               (Group            * folder,
 
 struct _Article;
 
-void          group_add_article                   (Group            * group,
-                                                   struct _Article  * article);
-
 void          group_add_articles                  (Group            * group,
                                                    const GPtrArray  * articles,
                                                    StatusItem       * status_or_null,
@@ -244,15 +250,9 @@ void          group_add_articles_remove_unused    (Group          * group,
                                                    GPtrArray        * articles,
                                                    StatusItem       * status_or_null);
 
-void          group_remove_article                (Group            * group,
-                                                   struct _Article  * article);
-
 void          group_remove_articles               (Group            * group,
-                                                   const GPtrArray  * articles);
-
-void          group_remove_crossposts             (Group            * g,
-		                                   gulong           * numbers,
-					           int                number_qty);
+                                                   struct _Article ** articles,
+                                                   int                article_qty);
 
 gboolean      group_ref_articles_if_loaded        (Group            * group);
 
