@@ -34,6 +34,7 @@
 #include <pan/base/message-check.h>
 #include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
+#include <pan/base/text-massager.h>
 
 /***
 ****  PRIVATE UTILITIES
@@ -103,10 +104,11 @@ strip_attribution_and_signature (const char * body, GMimeMessage * message)
  * Check to see if the user is top-posting.
  */
 static void
-check_topposting (GPtrArray      * setme_errors,
-                  GoodnessLevel  * setme_goodness,
-                  const char     * body,
-		  GMimeMessage   * message)
+check_topposting (const TextMassager   * tm,
+                  GPtrArray            * setme_errors,
+                  GoodnessLevel        * setme_goodness,
+                  const char           * body,
+		  GMimeMessage         * message)
 {
 	char * freeme = NULL;
 	int line_len;
@@ -116,6 +118,7 @@ check_topposting (GPtrArray      * setme_errors,
 	gboolean quoted_found = FALSE;
 
 	/* sanity clause */
+	g_return_if_fail (tm!=NULL);
 	g_return_if_fail (setme_errors!=NULL);
 	g_return_if_fail (setme_goodness!=NULL);
 	g_return_if_fail (body!=NULL);
@@ -128,7 +131,7 @@ check_topposting (GPtrArray      * setme_errors,
 	/* walk through the body, looking for an original line after a quoted one */
 	march = body = freeme = strip_attribution_and_signature (body, message);
 	while ((get_next_token_run (march, '\n', &march, &line_begin, &line_len))) {
-		if (is_quote_char ((guchar)line_begin[0])) /* check for quoted */
+		if (text_massager_is_quote_char (tm, (guchar)line_begin[0])) /* check for quoted */
 			quoted_found = TRUE;
 		else if (quoted_found) /* check for non-quoted after quoted */
 			original_found_after_quoted = TRUE;
@@ -338,10 +341,11 @@ check_mostly_quoted (GPtrArray      * setme_errors,
  *
  */
 static void
-check_all_quoted (GPtrArray      * setme_errors,
-                  GoodnessLevel  * setme_goodness,
-	          const char     * body,
-		  GMimeMessage   * message)
+check_all_quoted (const TextMassager   * tm,
+                  GPtrArray            * setme_errors,
+                  GoodnessLevel        * setme_goodness,
+	          const char           * body,
+		  GMimeMessage         * message)
 {
 	int line_len;
 	char * freeme;
@@ -359,7 +363,7 @@ check_all_quoted (GPtrArray      * setme_errors,
 			++march;
 		if (march==end) /* blank line */
 			continue;
-		if (!is_quote_char ((guchar)*march)) { /* found new content */
+		if (!text_massager_is_quote_char (tm, (guchar)*march)) { /* found new content */
 			g_free (freeme);
 			return;
 		}
@@ -374,10 +378,11 @@ check_all_quoted (GPtrArray      * setme_errors,
 }
 
 static void
-check_body (GPtrArray       * setme_errors,
-	    GoodnessLevel   * setme_goodness,
-	    GMimeMessage    * message,
-	    const gchar     * body)
+check_body (const TextMassager    * tm,
+            GPtrArray             * setme_errors,
+	    GoodnessLevel         * setme_goodness,
+	    GMimeMessage          * message,
+	    const gchar           * body)
 {
 	gboolean posting = is_nonempty_string (g_mime_message_get_header (message, HEADER_NEWSGROUPS));
 
@@ -391,8 +396,8 @@ check_body (GPtrArray       * setme_errors,
 		check_wide_body (setme_errors, setme_goodness, body);
 		check_signature (setme_errors, setme_goodness, body);
 		check_mostly_quoted (setme_errors, setme_goodness, body);
-		check_all_quoted (setme_errors, setme_goodness, body, message);
-		check_topposting (setme_errors, setme_goodness, body, message);
+		check_all_quoted (tm, setme_errors, setme_goodness, body, message);
+		check_topposting (tm, setme_errors, setme_goodness, body, message);
 	}
 }
 
@@ -555,10 +560,10 @@ check_mail_addresses (GPtrArray      * setme_errors,
 
 
 void
-message_check (GMimeMessage     * message,
-               Server           * server,
-               GPtrArray        * appendme_errors,
-               GoodnessLevel    * setme_goodness)
+message_check (GMimeMessage        * message,
+               Server              * server,
+               GPtrArray           * appendme_errors,
+               GoodnessLevel       * setme_goodness)
 {
 	gchar * tmp;
 	int mail_qty;
@@ -568,6 +573,7 @@ message_check (GMimeMessage     * message,
 	gboolean is_html;
 	gboolean followup_to_set;
 	GoodnessLevel goodness = OKAY;
+	TextMassager * tm_default_values;
 
 	/* sanity check */
 	g_return_if_fail (GMIME_IS_MESSAGE(message));
@@ -578,9 +584,11 @@ message_check (GMimeMessage     * message,
 	check_subject (appendme_errors, &goodness, g_mime_message_get_subject(message));
 
 	/* check the body... */
+	tm_default_values = text_massager_new ();
 	tmp = g_mime_message_get_body (message, TRUE, &is_html);
-	check_body (appendme_errors, &goodness, message, tmp);
+	check_body (tm_default_values, appendme_errors, &goodness, message, tmp);
 	g_free (tmp);
+	text_massager_free (tm_default_values);
 
 	/* check the optional followup-to... */
 	followup_to_set = FALSE;
