@@ -93,9 +93,7 @@ newsrc_import (Server         * server,
 	g_return_if_fail (is_nonempty_string(filename_in));
 
 	/* make sure the file exists */
-	filename = strchr (filename_in, '~') == NULL
-		? g_strdup (filename_in)
-		: pan_substitute (filename_in, "~", g_get_home_dir());
+	filename = pan_file_normalize (filename_in, NULL);
 	filename_in = NULL;
 	g_return_if_fail (pan_file_exists (filename));
 
@@ -127,6 +125,8 @@ newsrc_import (Server         * server,
 				gboolean subscribed;
 				char * name;
 				Group * group;
+				gulong group_low = 0ul;
+				gulong group_high = 0ul;
 
 				/* maybe skip non-subscribed groups */
 				if (subscribed_only && *pch!=':')
@@ -147,7 +147,8 @@ newsrc_import (Server         * server,
 				subscribed = FALSE;
 				newsrc_import_line (n, &subscribed, line->str);
 				g_ptr_array_add (subscribed?sub_y:sub_n, group);
-				group_set_article_range (group, n->group_low, n->group_high);
+				newsrc_get_group_range (n, &group_low, &group_high);
+				group_set_article_range (group, group_low, group_high);
 
 				/* track how many groups imported */
 				++groups_imported_qty;
@@ -197,85 +198,68 @@ newsrc_export (Server          * server,
                const char      * filename_in,
                gboolean          subscribed_only)
 {
-	int i;
-	int groups_exported_qty;
-	int subbed_exported_qty;
-	ServerGroupsType type;
 	FILE * fp;
-	char * tmpfile;
+	char * filename_tmp;
 	char * filename;
-	GString * s;
-	GPtrArray * a;
 
 	/* sanity clause */
 	g_return_if_fail (server_is_valid (server));
 	g_return_if_fail (is_nonempty_string(filename_in));
-	filename = strchr (filename_in, '~') == NULL
-		? g_strdup (filename_in)
-		: pan_substitute (filename_in, "~", g_get_home_dir());
+	filename = pan_file_normalize (filename_in, NULL);
 	filename_in = NULL;
 
 	/* open a temp file to write to */
-	fp = NULL;
-	tmpfile = pan_file_make_temp (&fp);
+	filename_tmp = pan_file_make_temp (&fp);
 	if (fp == NULL)
 	{
-		log_add_va (LOG_ERROR,
-		            _("Could not create temporary file: %s"),
-		            g_strerror(errno));
-		g_free (filename);
-		return;
+		log_add_va (LOG_ERROR, _("Could not create temporary file: %s"), g_strerror(errno));
 	}
-
-	/* get the groups */
-	type = subscribed_only ? SERVER_GROUPS_SUBSCRIBED : SERVER_GROUPS_ALL;
-	a = server_get_groups (server, type);
-
-	/* walk through the groups */
-	s = g_string_sized_new (4096);
-	groups_exported_qty = 0;
-	subbed_exported_qty = 0;
-	for (i=0; i<a->len; ++i)
+	else
 	{
-		Group * g;
-		Newsrc * n;
+		int i;
+		int groups_exported_qty = 0;
+		int subbed_exported_qty = 0;
+		gboolean ok;
+		GString * s = g_string_sized_new (4096);
+		ServerGroupsType type = subscribed_only ? SERVER_GROUPS_SUBSCRIBED : SERVER_GROUPS_ALL;
+		GPtrArray * a = server_get_groups (server, type);
 
-		/* get the group's newsrc */
-		g = GROUP(g_ptr_array_index(a, i));
-		n = group_has_newsrc(g) ? group_get_newsrc(g) : NULL;
+		/* walk through the groups */
+		for (i=0; i<a->len; ++i)
+		{
+			Group * g;
+			Newsrc * n;
 
-		/* write its .newsrc line */
-		g_string_truncate (s, 0);
-		newsrc_export_line (n, g->name,
-		                    group_is_subscribed(g), s);
-		fprintf (fp, "%s\n", s->str);
+			/* get the group's newsrc */
+			g = GROUP(g_ptr_array_index(a, i));
+			n = group_has_newsrc(g) ? group_get_newsrc(g) : NULL;
 
-		/* keep track of how many we've exported */
-		++groups_exported_qty;
-		if (group_is_subscribed(g))
-			++subbed_exported_qty;
+			/* write its .newsrc line */
+			g_string_truncate (s, 0);
+			newsrc_export_line (n, g->name,
+					    group_is_subscribed(g), s);
+			fprintf (fp, "%s\n", s->str);
+
+			/* keep track of how many we've exported */
+			++groups_exported_qty;
+			if (group_is_subscribed(g))
+				++subbed_exported_qty;
+		}
+		ok = !ferror(fp);
+		fclose (fp);
+
+		if (ok && pan_file_swap_datafile (filename_tmp, filename))
+			log_add_va (LOG_INFO, _("Exported %d groups (%d subscribed) to \"%s\""),
+					      groups_exported_qty,
+					      subbed_exported_qty,
+					      filename);
+
+		/* cleanup */
+		g_ptr_array_free (a, TRUE);
+		g_string_free (s, TRUE);
 	}
-	fclose (fp);
-
-	/* backup the old .newsrc file */
-	if (1) {
-		char bak[PATH_MAX];
-		g_snprintf (bak, sizeof(bak), "%s.bak", filename);
-		pan_file_rename (filename, bak);
-	}
-
-	/* move the temp file to newsrc */
-	pan_file_rename (tmpfile, filename);
-
-	/* log */
-	log_add_va (LOG_INFO, _("Exported %d groups (%d subscribed) to \"%s\""),
-	                      groups_exported_qty,
-	                      subbed_exported_qty,
-	                      filename);
 
 	/* cleanup */
 	g_free (filename);
-	g_free (tmpfile);
-	g_ptr_array_free (a, TRUE);
-	g_string_free (s, TRUE);
+	g_free (filename_tmp);
 }
