@@ -19,7 +19,7 @@
 
 #ifndef __NEWSRC_PORT_H__
 
-#include <glib.h>
+#include <glib/gtypes.h>
 #include <pan/base/server.h>
 
 void newsrc_import (Server          * server,
