@@ -34,6 +34,7 @@
 #include <pan/base/debug.h>
 #include <pan/base/newsrc.h>
 #include <pan/base/pan-glib-extensions.h>
+#include <pan/base/pan-object.h>
 
 /*********************
 **********************  Defines / Enumerated types
@@ -54,6 +55,18 @@ typedef struct
 }
 Range;
 
+struct _Newsrc
+{
+	/* parent class */
+	PanObject parent;
+
+	/* private fields */
+	gulong group_low;	/* the low article number in the group */
+	gulong group_high;	/* the high article number in the group */
+	GArray * read;	/* sorted array of ranges of read article #s */
+	char * read_str;	/* the .newsrc string of read article #s */
+};
+
 /*********************
 **********************  Private Function Prototypes
 *********************/
@@ -393,7 +406,20 @@ newsrc_mark_article (Newsrc * n, gulong number, gboolean read)
 *************  PROTECTED
 ************/
 
-void
+static void
+newsrc_destructor (PanObject * obj)
+{
+	Newsrc * n = NEWSRC (obj);
+
+	debug1 (DEBUG_PAN_OBJECT, "newsrc_dtor: %p", n);
+	g_free (n->read_str);
+	if (n->read != NULL)
+		g_array_free (n->read, TRUE);
+
+	pan_object_destructor (obj);
+}
+
+static void
 newsrc_constructor (Newsrc * n,
                     const char * read_str,
                     gulong low,
@@ -411,18 +437,6 @@ newsrc_constructor (Newsrc * n,
 	newsrc_init (n, read_str, low, high);
 }
 
-void
-newsrc_destructor (PanObject * obj)
-{
-	Newsrc * n = NEWSRC (obj);
-
-	debug1 (DEBUG_PAN_OBJECT, "newsrc_dtor: %p", n);
-	g_free (n->read_str);
-	if (n->read != NULL)
-		g_array_free (n->read, TRUE);
-
-	pan_object_destructor (obj);
-}
 
 /************
 *************  PUBLIC
@@ -459,6 +473,19 @@ newsrc_new (const char * read_str, gulong low, gulong high)
 ******
 *****/
 
+void
+newsrc_get_group_range  (Newsrc             * newsrc,
+                         gulong             * group_low,
+                         gulong             * group_high)
+{
+	g_return_if_fail (newsrc!=NULL);
+
+	if (group_low != NULL)
+		*group_low = newsrc->group_low;
+	if (group_high != NULL)
+		*group_high = newsrc->group_high;
+}
+
 void
 newsrc_set_group_range (Newsrc * n, gulong low, gulong high)
 {
