@@ -353,50 +353,27 @@ write_node (GNode * node, gpointer data)
 static void
 pan_config_save (void)
 {
-	GString * output;
-	char      fname[PATH_MAX];
-	char    * tmp_path;
-	FILE    * fp;
-	gboolean  ok;
+	if (config_root != NULL)
+	{
+		char filename[PATH_MAX];
+		GString * output = g_string_sized_new (4096);
 
-	if (config_root == NULL)
-		return;
+		/* build the datafile */
+		g_string_assign (output, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
+		                         "<!DOCTYPE config SYSTEM \"config.dtd\">\n"
+		                         "<config>");
+		last_depth = 0;
+		g_node_traverse (config_root, G_PRE_ORDER, G_TRAVERSE_ALL, -1, write_node, output);
+		close_previous_level (output, last_depth, 2);
+		g_string_append (output, "\n</config>");
 
-	output = g_string_sized_new (4096);
+		/* save the datafile */
+		get_config_filename (filename, sizeof(filename));
+		pan_file_write_datafile (filename, output->str, output->len);
 
-	g_string_assign (output, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
-		"<!DOCTYPE config SYSTEM \"config.dtd\">\n"
-		"<config>");
-
-	last_depth = 0;
-	g_node_traverse (config_root, G_PRE_ORDER, G_TRAVERSE_ALL, -1, 
-	                 write_node, output);
-	close_previous_level (output, last_depth, 2);
-
-	g_string_append (output, "\n</config>");
-
-	get_config_filename (fname, sizeof(fname));
-	tmp_path = pan_file_make_temp (&fp);
-	ok = fp != NULL;
-
-	if (ok) {
-		const size_t to_write = strlen (output->str);
-		const size_t written  = fwrite (output->str, sizeof(char), to_write, fp);
-		fclose (fp);
-		ok = to_write == written;
+		/* cleanup */
+		g_string_free (output, TRUE);
 	}
-
-	if (ok) {
-		ok = pan_file_rename (tmp_path, fname);
-	}
-
-	if (!ok) {
-		log_add_va (LOG_ERROR, _("Can't write to \"%s\": %s"),
-		            fname, g_strerror (errno));
-	}
-
-	g_free (tmp_path);
-	g_string_free (output, TRUE);
 }
 
 void
@@ -500,7 +477,7 @@ pan_config_load_ini (void)
 
 	/* read the ini file */
 	log_add (LOG_INFO, "can't find config.xml file; trying to import for ~/.gnome/Pan");
-	filename = g_build_filename (g_get_home_dir(), ".gnome", "Pan", NULL);
+	filename = g_build_filename (pan_get_home_dir(), ".gnome", "Pan", NULL);
 	g_file_get_contents (filename, &file, NULL, NULL);
 
 	/* march through the file */
