@@ -28,17 +28,45 @@
 #include <glib.h>
 
 #include <gmime/gmime-charset.h>
+#include <gmime/gmime-filter-charset.h>
+#include <gmime/gmime-stream-filter.h>
+#include <gmime/gmime-stream-mem.h>
 #include <gmime/gmime-utils.h>
 
 #include <iconv.h>
 #include <gmime/gmime-iconv.h>
 #include <gmime/gmime-iconv-utils.h>
 
+#include <pan/base/base-prefs.h>
 #include <pan/base/log.h>
 #include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 
 
+/***
+****
+****  DIRECTORIES
+****
+***/
+
+const char*
+pan_get_home_dir (void)
+{
+	static const char * home = NULL;
+
+ 	if (home == NULL)
+	{
+		home = g_get_home_dir ();
+
+#ifdef G_OS_WIN32
+		if (home == NULL)
+			home = g_get_current_dir ();
+#endif
+	}
+
+	return home;
+}
+
 /***
 ****
 ****  STRINGS
@@ -780,6 +808,51 @@ pan_utf8ize (const char    * str,
 	return retval;
 }
 
+static char*
+g_mime_charset_strndup (const char     * to_charset,
+                        const char     * from_charset,
+                        const char     * text,
+                        int              text_len)
+{
+	char * retval;
+	gssize out_len;
+	GMimeStream * in_stream;
+	GMimeStream * in_stream_filter;
+	GMimeFilter * charset_filter;
+	GMimeStream * out_stream;
+	GByteArray * byte_array;
+
+	/* set up an input stream with the desired charset filter */
+	in_stream = g_mime_stream_mem_new_with_buffer (text, text_len);
+	charset_filter = g_mime_filter_charset_new (from_charset, to_charset);
+	in_stream_filter = g_mime_stream_filter_new_with_stream (in_stream);
+	g_mime_stream_filter_add (GMIME_STREAM_FILTER(in_stream_filter), charset_filter);
+
+	/* set up an output stream attached to a byte array */
+	byte_array = g_byte_array_new ();
+	out_stream = g_mime_stream_mem_new ();
+	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM(out_stream), byte_array);
+
+	/* write the input stream to the output stream */
+	out_len = g_mime_stream_write_to_stream (in_stream_filter, out_stream);
+
+	/* if the write was successful, zero-terminate the string and return it. */
+	if (out_len < 0)
+		retval = NULL;
+	else {
+		g_byte_array_append (byte_array, "", 1);
+		retval = byte_array->data;
+		g_byte_array_free (byte_array, FALSE);
+	}
+
+	/* cleanup */
+	g_object_unref (G_OBJECT(out_stream));
+	g_object_unref (G_OBJECT(in_stream_filter));
+	g_object_unref (G_OBJECT(in_stream));
+
+	return retval;
+}
+
 char*
 pan_header_to_utf8 (const char   * header,
                     int            header_len,
@@ -802,14 +875,14 @@ pan_header_to_utf8 (const char   * header,
 	}
 
 	/* naughty boys don't properly encode their 8bit string */
-	else if (g_mime_utils_text_is_8bit (header, header_len)) {
+	else if (g_mime_utils_text_is_8bit (header, header_len) &&
+		 !g_utf8_validate (header, header_len, NULL)) {
 		if (is_nonempty_string(charset)) {
-			iconv_t cd = g_mime_iconv_open ("UTF-8", charset);
-			retval = g_mime_iconv_strndup (cd, header, header_len);
-			g_mime_iconv_close (cd);
+			retval = g_mime_charset_strndup ("UTF-8", charset, header, header_len);
 		}
 		if (retval == NULL) {
-			retval = g_mime_iconv_locale_to_utf8_length (header, header_len);
+			const char * local_charset = get_charset_from_locale ();
+			retval = g_mime_charset_strndup ("UTF-8", local_charset, header, header_len);
 		}
 	}
 
@@ -839,19 +912,13 @@ pan_body_to_utf8 (GMimePart     * part,
 	/* use the part's content-type charset, if it's got one */
 	if (retval == NULL) {
 		const char * charset = g_mime_object_get_content_type_parameter (GMIME_OBJECT (part), "charset");
-		if (is_nonempty_string (charset)) {
-			iconv_t cd = g_mime_iconv_open ("UTF-8", charset);
-			retval = g_mime_iconv_strndup (cd, content, content_len);
-			g_mime_iconv_close (cd);
-		}
+		if (is_nonempty_string (charset))
+			retval = g_mime_charset_strndup ("UTF-8", charset, content, content_len);
 	}
 
 	/* try the group's default charset */
-	if (retval==NULL && is_nonempty_string (default_charset)) {
-		iconv_t cd = g_mime_iconv_open ("UTF-8", default_charset);
-		retval = g_mime_iconv_strndup (cd, content, content_len);
-		g_mime_iconv_close (cd);
-	}
+	if (retval==NULL && is_nonempty_string (default_charset))
+		retval = g_mime_charset_strndup ("UTF-8", default_charset, content, content_len);
 
 	/* otherwise just use the text and hope for the best */
 	if (retval == NULL) {
