@@ -21,7 +21,11 @@
 #define __PAN_GLIB_EXTENSIONS_H__
 
 #include <stddef.h>
-#include <glib.h>
+#include <glib/galloca.h>
+#include <glib/ghash.h>
+#include <glib/garray.h>
+#include <glib/gstring.h>
+#include <glib/gmessages.h>
 #include <gmime/gmime-part.h>
 #include <time.h>
 
@@ -130,6 +134,8 @@ char* pan_str_escape   (const char * encode_for_xml);
 
 char* pan_str_unescape (const char * decode_xml);
 
+const char* pan_get_home_dir (void);
+
 
 /**
  * Wrapper around localtime_r for platforms that don't have it.
@@ -173,7 +179,6 @@ int    lower_bound   (const void   * key,
                       gboolean     * exact_match );
 
 
-
 typedef enum
 {
 	RUN_SUCCESS = 0,
