@@ -134,7 +134,7 @@ server_get_group_type (const Group * group)
        	type = 0;
 	if (group_is_subscribed (group))
 		type |= SERVER_GROUPS_SUBSCRIBED;
-	if (!group_is_subscribed (group))
+	else
 		type |= SERVER_GROUPS_UNSUBSCRIBED;
 
 	return type;
@@ -162,7 +162,7 @@ server_get_groups_type (const Group ** groups, gint qty)
 void
 server_ensure_groups_loaded (Server * server, ServerGroupsType type)
 {
-	gboolean load_needed = FALSE;
+	gboolean load_needed;
 	debug_enter ("server_ensure_groups_loaded");
 
 	/* sanity clause */
@@ -192,10 +192,11 @@ server_ensure_groups_loaded (Server * server, ServerGroupsType type)
 GPtrArray*
 server_get_groups (Server * server, ServerGroupsType type)
 {
-	gint i;
+	int i;
 	GPtrArray * a = g_ptr_array_new ();
 	debug_enter ("server_get_groups");
 
+	/* sanity clause */
 	g_return_val_if_fail (server_is_valid(server), a);
 	g_return_val_if_fail (type!=0, a);
 
@@ -238,9 +239,11 @@ server_set_group_type_dirty (Server * server, ServerGroupsType type)
 void
 server_save_grouplist_if_dirty (Server * server, StatusItem * status)
 {
+	/* sanity clause */
 	g_return_if_fail (server_is_valid(server));
 	g_return_if_fail (server_saving_enabled);
 
+	/* save anything that's dirty */
 	if (server->_groups_dirty)
 	{
 		file_grouplist_save (server, server->_groups_dirty, status);
@@ -318,7 +321,7 @@ server_add_groups_impl (Server       * server,
                         GPtrArray    * fillme_not_used,
                         gboolean       initializing)
 {
-	gint i;
+	int i;
 	GHashTable * old;
 	GPtrArray * added;
 	debug_enter ("server_add_groups_impl");
@@ -327,10 +330,14 @@ server_add_groups_impl (Server       * server,
 	g_return_if_fail (server_is_valid (server));
 	g_return_if_fail (groups != NULL);
 	g_return_if_fail (group_qty >= 1);
+	for (i=0; i<group_qty; ++i) {
+		g_return_if_fail (groups[i] != NULL);
+		g_return_if_fail (groups[i]->server == server);
+	}
 
 	/* ensure the groups sets are loaded before adding to them */
 	if (1) {
-		ServerGroupsType type = 0;
+		ServerGroupsType type;
 		type = server_get_groups_type ((const Group **)groups, group_qty);
 		server_ensure_groups_loaded (server, type);
 	}
@@ -343,7 +350,7 @@ server_add_groups_impl (Server       * server,
 	}
 
 	/* try to add the new groups */
-	added = g_ptr_array_new ();
+	added = g_ptr_array_sized_new (group_qty);
 	for (i=0; i<group_qty; ++i)
 	{
 		Group * new_group = groups[i];
@@ -351,13 +358,6 @@ server_add_groups_impl (Server       * server,
 
 		if (!has_group)
 		{
-			/* update the group's server field */
-			if (new_group->server!=server) {
-				if (new_group->server!=NULL)
-					server_remove_groups (new_group->server, &new_group, 1);
-				new_group->server = server;
-			}
-
 			/* add the group */
 			g_hash_table_insert (old, (gpointer)new_group->name, new_group);
 			g_ptr_array_add (added, new_group);
@@ -372,8 +372,6 @@ server_add_groups_impl (Server       * server,
 	/* if any change was made, update the states */
 	if (added->len != 0)
 	{
-		ServerGroupsType type;
-
 		/* update the groups array */
 		pan_hash_to_ptr_array  (old, server->_groups);
 		g_ptr_array_sort_with_data (server->_groups, compare_ppgroup_ppgroup_name, NULL);
@@ -381,7 +379,7 @@ server_add_groups_impl (Server       * server,
 		if (!initializing)
 		{
 			/* mark the appropriate sets as dirty */
-			type = server_get_groups_type ((const Group**)added->pdata, added->len);
+			ServerGroupsType type = server_get_groups_type ((const Group**)added->pdata, added->len);
 			server_set_group_type_dirty (server, type);
 
 			/* let listeners know which were added */
@@ -538,13 +536,38 @@ server_is_valid (const Server * server)
 	return TRUE;
 }
 
-const gchar * server_get_name (const Server * server)
+const char*
+server_get_name (const Server * server)
 {
 	g_return_val_if_fail (server_is_valid (server), "");
 
 	return server->name;
 }
 
+void
+server_set_name (Server * server, const char * new_name)
+{
+	char * my_new_name;
+
+	/* sanity clause */
+	g_return_if_fail (server != NULL);
+	g_return_if_fail (is_nonempty_string (new_name));
+
+	/* create a stripped copy of new_name */
+	my_new_name = g_strdup (new_name);
+	g_strstrip (my_new_name);
+	g_return_if_fail (is_nonempty_string (my_new_name));
+
+	/* if name is changing -- not just being initialized -- tell file_headers */
+	if (server->name!=NULL && pan_strcmp (server->name, my_new_name)) {
+		file_headers_server_name_changed (server->name, my_new_name);
+		file_grouplist_server_name_changed (server->name, my_new_name);
+	}
+
+	/* update the name */
+	replace_gstr (&server->name, my_new_name);
+}
+
 void
 server_destroy_groups (Server * server, struct _Group ** groups, gint qty)
 {
