@@ -21,7 +21,8 @@
 #define __SERVER_H__
 
 #include <time.h>
-#include <glib.h>
+#include <glib/gtypes.h>
+#include <glib/garray.h>
 
 #include <gmime/memchunk.h>
 
@@ -86,7 +87,9 @@ Server*       server_new (void);
 
 gboolean      server_is_valid            (const Server*);
 
-const gchar * server_get_name            (const Server*);
+const char *  server_get_name            (const Server*);
+
+void          server_set_name            (Server * server, const char * name);
 
 
 /**
@@ -117,8 +120,6 @@ void       server_add_groups             (Server            * server,
                                           GPtrArray         * fillme_used,
                                           GPtrArray         * fillme_not_used);
 
-gboolean    server_add_group             (Server * server, struct _Group * group);
-
 void        server_remove_groups          (Server * server, struct _Group ** groups, gint qty);
 
 void        server_destroy_groups         (Server * server, struct _Group ** groups, gint qty);
