@@ -92,7 +92,7 @@ server_fetch (const char *name)
 
 			/* load the server from pan_config */
 			server->port = pan_config_get_int ("Port=119");
-			server->name = g_strdup(name);
+			server_set_name (server, name);
 			server->use_newsrc = pan_config_get_bool ("Use_Newsrc=0");
 			server->newsrc_filename = pan_config_get_string ("Newsrc_Filename");
 			server->need_auth = pan_config_get_int ("Auth_Required=0");
@@ -346,7 +346,7 @@ serverlist_save (void)
 		const char * str_begin = NULL;
 		const char * walk = p;
 		while (get_next_token_run (walk, ',', &walk, &str_begin, &str_len)) {
-			char * line = g_alloca (str_len+1);
+			char * line = g_newa (char, str_len+1);
 			memcpy (line, str_begin, str_len);
 			line[str_len] = '\0';
 
@@ -355,7 +355,6 @@ serverlist_save (void)
 		}
 		g_free (p);
 	}
-	pan_config_sync ();
 
 	/* In with the new */
 	for (l=serverlist; l!=NULL; l=l->next)
