@@ -21,7 +21,7 @@
 #define __SERVER_MODULE_H__
 
 #include <time.h>
-#include <glib.h>
+#include <glib/garray.h>
 
 #include <pan/base/pan-callback.h>
 #include <pan/base/pan-object.h>
