@@ -1000,7 +1000,7 @@ pan_g_mime_message_mark_read (GMimeMessage * message)
 
 	if (is_nonempty_string (xref))
 	{
-		char * pch;
+		GString * token = g_string_new (NULL);
 		Server * server = serverlist_get_active_server ();
 		const char * message_id = g_mime_message_get_message_id (message);
 
@@ -1008,10 +1008,10 @@ pan_g_mime_message_mark_read (GMimeMessage * message)
 		skip_next_token (xref, ' ', &xref);
 
 		/* walk through the xrefs group:number group2:number2*/
-		while ((pch = get_next_token_str (xref, ' ', &xref)))
+		while ((get_next_token_g_str (xref, ' ', &xref, token)))
 		{
 			/* parse out the group and number values */
-			char * delim = strchr (pch, ':');
+			char * delim = strchr (token->str, ':');
 			if (delim != NULL)
 			{
 				const char * group_name;
@@ -1020,7 +1020,7 @@ pan_g_mime_message_mark_read (GMimeMessage * message)
 				*delim = '\0';
 
 				/* if the group is subscribed, mark the article as read */
-				group_name = pch;
+				group_name = token->str;
 				group = server_get_named_group_in_type (server, group_name, SERVER_GROUPS_SUBSCRIBED);
 				if (group != NULL)
 				{
@@ -1037,9 +1037,9 @@ pan_g_mime_message_mark_read (GMimeMessage * message)
 
 				}
 			}
-
-			g_free (pch);
 		}
+
+		g_string_free (token, TRUE);
 	}
 
 	debug_exit ("pan_g_mime_message_mark_read");
@@ -1065,18 +1065,52 @@ pan_g_mime_message_get_group (GMimeMessage * message)
 		if (is_nonempty_string (newsgroups))
 		{
 			const char * march = newsgroups;
+			GString * group_name = g_string_new (NULL);
 			for (;;)
 			{
-				char * group_name = get_next_token_str (march, ',', &march);
-				if (group_name == NULL)
+				if (!get_next_token_g_str (march, ',', &march, group_name))
 					break;
-				retval = server_get_named_group (server, group_name);
-				g_free (group_name);
+				retval = server_get_named_group (server, group_name->str);
 				if (retval != NULL)
 					break;
 			}
+			g_string_free (group_name, TRUE);
 		}
 	}
 
 	return retval;
 }
+
+/**
+ * Retrieve the charset from a mime message
+ */
+
+static void
+get_charset_partfunc (GMimeObject * obj, gpointer charset_gpointer)
+{
+	GMimePart * part;
+	const GMimeContentType * type;
+	char ** charset;
+
+	if (!GMIME_IS_PART (obj))
+		return;
+
+	part = GMIME_PART (obj);
+	type = g_mime_object_get_content_type (GMIME_OBJECT (part));
+	charset = (char **) charset_gpointer;
+	if (g_mime_content_type_is_type (type, "text", "*")) {
+		*charset = (char *) g_mime_object_get_content_type_parameter (GMIME_OBJECT (part), "charset");
+		/*odebug1 ("found charset in article: %s", *charset);*/
+	}
+}
+
+const char *
+pan_g_mime_message_get_charset (GMimeMessage * message)
+{
+	char * retval = NULL;
+	g_return_val_if_fail (message!=NULL, NULL);
+
+	g_mime_message_foreach_part (message, get_charset_partfunc, &retval);
+
+	return retval;
+}
