@@ -93,32 +93,29 @@ static char *
 get_credits_translators (void)
 {
 	const char * str =
-		"\n\tam - Ge'ez Frontier Foundation"
-		"\n\tcs - Michal Bukovjan, Miloslav Trmac"
-		"\n\tde - Benjamin Greiner, Christian Meyer, Karsten Weiss"
-		"\n\tes - Pablo del Campo"
+		"\n\tcs - Miloslav Trmac"
+		"\n\tde - Benjamin Greiner"
+		"\n\tes - Pablo Gonzalo del Campo"
 		"\n\tet - Tivo Leedjrv"
-		"\n\tfr - Gildas Guillemot, Christophe Merlet, Laurent Richard"
-		"\n\tno - Kjartan Maraas"
-		"\n\tsk - Marcel Telka"
-		"\n\tsv - Christian Rose"
+		"\n\tfr - Christophe Merlet"
+		"\n\tja - KAMAGASAKO Masatoshi"
 		"\n\tlv - Artis Trops"
+		"\n\tno - Kjartan Maraas"
+		"\n\tnl - Vincent van Adrighem"
 		"\n\tpl - GNOME PL Team"
 		"\n\tpt - Duarte Loreto"
-		"\n\tnl - Vincent van Adrighem, Reinout van Schouwen"
-		"\n\tuk - Yuri Syrota"
-		"\n\tja - Takayuki KUSANO";
+		"\n\tpt_BR - Evandro Fernandes Giovanini"
+		"\n\tsk - Marcel Telka"
+		"\n\tsv - Christian Rose"
+		"\n\ttr - Arman Aksoy (Armish)";
 	return g_convert (str, strlen(str), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 }
 
 static char *
 get_credits_code (void)
 {
-	const char * str =
-		"\n\tRobert Boucher"
-		"\n\tClinton Bunch"
-		"\n\tHarav Shivanan"
-		"\n\tNam SungHyun";
+	const char * str = 
+		"\n\t";
 	return g_convert (str, strlen(str), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 }
 
@@ -126,25 +123,47 @@ static char *
 get_credits_users (void)
 {
 	const char * str = 
-		"\n\tRobert Boucher"
-		"\n\tMichaela Susan Buesin"
-		"\n\tBobby D. Bryant (#102208)"
-		"\n\tJean-Marc Desperrier (#104479)"
-		"\n\tNicolas Girard (#104155)"
-		"\n\tJakob Breivik Grimstveit (#105879)"
-		"\n\tMike Hearn (#103563)"
-		"\n\tHolger Hoffsttte (#102881)"
-		"\n\tMark Kelly"
-		"\n\tMaurizio (#101013)"
-		"\n\tBas Mevissen (#103908, #103907)"
-		"\n\tMr. Nice (#107025)"
-		"\n\tMatt Pinkham (#103579)"
-		"\n\tMartin Schrode (#88869)"
-		"\n\tJerry Segers (#102586, #102577)"
-		"\n\tHarav Shivanan"
-		"\n\tVille Skytt (#102158)"
-		"\n\tLuigi Trovato"
-		"\n\tAdam Warner (#102469)"
-		"\n\tMichael Waters";
+		"\n\tBlueJAMC (#108769, #108991)"
+		"\n\tBruce Bowler (#107484)"
+		"\n\tBobby D. Bryant (#108592, #108593)"
+		"\n\tDave C (#111432)"
+		"\n\tChris Charlebois (#109274)"
+		"\n\tMichael A. Chase (#111307)"
+		"\n\tPetr Chloupek (#99247)"
+		"\n\tStephen Cofer"
+		"\n\tSimon Cooke (#107328)"
+		"\n\tDuncan (#110794)"
+		"\n\tJim Dutton (#104716)"
+		"\n\tErik Funkenbusch (#107324)"
+		"\n\tEvandro Fernandes Giovanini (#107900)"
+		"\n\tLin Ford (#109271, #109273)"
+		"\n\tCarl Hudkins (#107891, #107894)"
+		"\n\tPaul Hudson (#109279)"
+		"\n\tDave Hill (#110937, #110938)"
+		"\n\tSamuli Karkkainen (#110319)"
+		"\n\tPatrick Lamaiziere (#108567, #109004, #109006)"
+		"\n\tTivo Leedjrv (#110478, #110491, #110967)"
+		"\n\tChristian Lohmaier (#90067)"
+		"\n\tEugenia Loli-Queru (#109652)"
+		"\n\tPav Lucistnik (#111145)"
+		"\n\tMahalo (#104479, #108057, #108489)"
+		"\n\tGeorge McAllister (#109282)"
+		"\n\tDonald L. McDaniel (#107302, #107519)"
+		"\n\tJ. B. Moreno (#108126)"
+		"\n\tBrian Morrison (#110790)"
+		"\n\tChris Petersen"
+		"\n\tGabriel Phoenix (#100444)"
+		"\n\tJustin Piszcz (#107919)"
+		"\n\tNathan Robertson (#110151)"
+		"\n\tThorsten Ruffle-Brandt (#108171)"
+		"\n\tJeffrey Stedfast"
+		"\n\tTorstein Sunde (#107869, #107901)"
+		"\n\tSal Tepedino (#98452)"
+		"\n\tKetil Thorsen (#107238, #108575)"
+		"\n\tTRB (#108220)"
+		"\n\tJohn van der Kamp (#100651)"
+		"\n\tFrederik Vanrenterghem (#103872)"
+		"\n\tOlav Vitters (#110041)"
+		"\n\tMichael Waters (#107113)" ;
 	return g_convert (str, strlen(str), "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 }
