@@ -360,7 +360,7 @@ druid_user_page (Druid * druid)
 
 		/* signature file */
 		w = druid->signature_entry = pan_file_entry_new (_("Select Signature File"));
-		pch = g_build_filename (g_get_home_dir(), ".signature", NULL);
+		pch = g_build_filename (pan_get_home_dir(), ".signature", NULL);
 		if (pan_file_exists (pch))
 			pan_file_entry_set (w, pch);
 		else
