@@ -42,9 +42,7 @@
 #include <pan/filters/filter-linecount.h>
 #include <pan/filters/filter-new.h>
 #include <pan/filters/filter-phrase.h>
-#include <pan/filters/filter-saved.h>
 #include <pan/filters/filter-score.h>
-#include <pan/filters/filter-thread.h>
 
 #include <pan/filter-edit-ui.h>
 #include <pan/util.h>
@@ -79,13 +77,15 @@ typedef struct
 	GtkWidget * age_rb;
 	GtkWidget * age_sb;
 	GtkWidget * score_rb;
-	GtkWidget * score_sb;
+#if 0
+	GtkWidget * score_combo;
+#else
+	GtkWidget * score_menu;
+#endif
 	GtkWidget * thread_rb;
 	GtkWidget * thread_menu;
 	GtkWidget * attach_rb;
 	GtkWidget * attach_menu;
-	GtkWidget * saved_rb;
-	GtkWidget * saved_menu;
 
 	GtkWidget * article_is_rb;
 	GtkWidget * article_is_menu;
@@ -360,7 +360,7 @@ get_new_filter (FilterEditDialog * d)
 		if (is_nonempty_string(key))
 		{
 			f = filter_phrase_new ();
-			filter_phrase_set (FILTER_PHRASE(f), match_type, key_type, key);
+			filter_phrase_set (FILTER_PHRASE(f), match_type, key_type, key, FALSE);
 			if (negate)
 				filter_negate (f);
 		}
@@ -383,17 +383,29 @@ get_new_filter (FilterEditDialog * d)
 	}
 	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->score_rb)))
 	{
-		f = filter_score_new ();
-		FILTER_SCORE(f)->minimum_score = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(d->score_sb));
-	}
-	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->thread_rb)))
-	{
-		GtkWidget * active;
-		FilterThreadState state;
- 		active = gtk_menu_get_active (GTK_MENU(d->thread_menu));
- 		state = GPOINTER_TO_INT(gtk_object_get_user_data (GTK_OBJECT(active)));
-		f = filter_thread_new ();
-		filter_thread_set_state (FILTER_THREAD(f), state);
+		GtkWidget * active = gtk_menu_get_active (GTK_MENU(d->score_menu));
+		const FilterScoreMode menu_mode = GPOINTER_TO_INT (gtk_object_get_user_data (GTK_OBJECT (active)));
+		FilterScoreMode mode = 0;
+
+		switch (menu_mode) /* note fall-through to build "greater than or equal" bitmask */
+		{
+			case SCORE_IGNORED:
+				mode |= SCORE_IGNORED;
+			case SCORE_LOW:
+				mode |= SCORE_LOW;
+			case SCORE_ZERO:
+				mode |= SCORE_ZERO;
+			case SCORE_MEDIUM:
+				mode |= SCORE_MEDIUM;
+			case SCORE_HIGH:
+				mode |= SCORE_HIGH;
+			case SCORE_WATCHED:
+				mode |= SCORE_WATCHED;
+			default:
+				break;
+		}
+
+		f = filter_score_new (mode);
 	}
 	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->attach_rb)))
 	{
@@ -402,13 +414,6 @@ get_new_filter (FilterEditDialog * d)
 		f = filter_binary_new ();
 		filter_binary_set_state (FILTER_BINARY(f), GPOINTER_TO_INT(state));
 	}
-	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->saved_rb)))
-	{
-		GtkWidget * active = gtk_menu_get_active (GTK_MENU(d->saved_menu));
-		gpointer state = gtk_object_get_user_data (GTK_OBJECT(active));
-		f = filter_saved_new ();
-		filter_saved_set_state (FILTER_SAVED(f), GPOINTER_TO_INT(state));
-	}
 	else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->article_is_rb)))
 	{
 		GtkWidget * active = gtk_menu_get_active (GTK_MENU(d->article_is_menu));
@@ -759,66 +764,40 @@ filter_edit_dialog_new (GtkWindow * window, const FilterTop * top)
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 
-#define add_menu_item(str, callback_int) \
+#define add_menu_item(str, callback_int, radiobutton) \
 	tmp = gtk_menu_item_new_with_label (str); \
 	gtk_object_set_user_data (GTK_OBJECT(tmp), GINT_TO_POINTER(callback_int)); \
-	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(d->phrase_rb)); \
+	g_signal_connect_swapped (GTK_OBJECT(tmp), "activate", G_CALLBACK(tb_activate), GTK_OBJECT(radiobutton)); \
 	gtk_widget_show (tmp); \
 	gtk_menu_append (GTK_MENU(w2), tmp); \
 
 	w = gtk_option_menu_new ();
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 	w2 = d->phrase_key_menu = gtk_menu_new ();
-	add_menu_item (_("Subject"), PHRASE_KEY_SUBJECT)
-	add_menu_item (_("Author"), PHRASE_KEY_AUTHOR)
-	add_menu_item (_("Message-ID"), PHRASE_KEY_MESSAGE_ID)
+	add_menu_item (_("Subject"), PHRASE_KEY_SUBJECT, d->phrase_rb)
+	add_menu_item (_("Author"), PHRASE_KEY_AUTHOR, d->phrase_rb)
+	add_menu_item (_("Message-ID"), PHRASE_KEY_MESSAGE_ID, d->phrase_rb)
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
 
 	w = gtk_option_menu_new ();
 	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
 	w2 = d->phrase_match_menu = gtk_menu_new ();
-	add_menu_item (_("contains"), PHRASE_MATCH_CONTAINS)
-	add_menu_item (_("does not contain"), -PHRASE_MATCH_CONTAINS)
-	add_menu_item (_("is"), PHRASE_MATCH_IS)
-	add_menu_item (_("is not"), -PHRASE_MATCH_IS)
-	add_menu_item (_("starts with"), PHRASE_MATCH_STARTS_WITH)
-	add_menu_item (_("does not start with"), -PHRASE_MATCH_STARTS_WITH)
-	add_menu_item (_("ends with"), PHRASE_MATCH_ENDS_WITH)
-	add_menu_item (_("does not end with"), -PHRASE_MATCH_ENDS_WITH)
-	add_menu_item (_("matches regular expression"), PHRASE_MATCH_REGEX)
-	add_menu_item (_("does not match regular expression"), -PHRASE_MATCH_REGEX)
+	add_menu_item (_("contains"), PHRASE_MATCH_CONTAINS, d->phrase_rb)
+	add_menu_item (_("does not contain"), -PHRASE_MATCH_CONTAINS, d->phrase_rb)
+	add_menu_item (_("is"), PHRASE_MATCH_IS, d->phrase_rb)
+	add_menu_item (_("is not"), -PHRASE_MATCH_IS, d->phrase_rb)
+	add_menu_item (_("starts with"), PHRASE_MATCH_STARTS_WITH, d->phrase_rb)
+	add_menu_item (_("does not start with"), -PHRASE_MATCH_STARTS_WITH, d->phrase_rb)
+	add_menu_item (_("ends with"), PHRASE_MATCH_ENDS_WITH, d->phrase_rb)
+	add_menu_item (_("does not end with"), -PHRASE_MATCH_ENDS_WITH, d->phrase_rb)
+	add_menu_item (_("matches regular expression"), PHRASE_MATCH_REGEX, d->phrase_rb)
+	add_menu_item (_("does not match regular expression"), -PHRASE_MATCH_REGEX, d->phrase_rb)
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
 
 	w = d->phrase_key_entry = gtk_entry_new ();
 	g_signal_connect_swapped (GTK_OBJECT(w), "focus_in_event", G_CALLBACK(tb_activate), GTK_OBJECT(d->phrase_rb));
 	gtk_box_pack_start (GTK_BOX(hbox), w, TRUE, TRUE, 0);
 
-	/* saved */
-	++row;
-	w = d->saved_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article has been"));
-	gtk_table_attach (GTK_TABLE(table), w, col, col+1, row, row+1, GTK_FILL, 0, 0, 0);
-	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
-	w = gtk_option_menu_new ();
-	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_FILL, 0, 0, 0);
-	w2 = d->saved_menu = gtk_menu_new ();
-	add_menu_item (_("Saved"), FILTER_SAVED_SAVED)
-	add_menu_item (_("Queued for Saving"), FILTER_SAVED_QUEUED)
-	add_menu_item (_("Neither Saved nor Queued"), FILTER_SAVED_NOT_SAVED)
-	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
-
-	/* thread */
-	++row;
-	w = d->thread_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article _thread is"));
-	gtk_table_attach (GTK_TABLE(table), w, col, col+1, row, row+1, GTK_FILL, 0, 0, 0);
-	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
-	w = gtk_option_menu_new ();
-	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_FILL, 0, 0, 0);
-	w2 = d->thread_menu = gtk_menu_new ();
-	add_menu_item (_("Watched"), FILTER_THREAD_WATCHED)
-	add_menu_item (_("Ignored"), FILTER_THREAD_IGNORED)
-	add_menu_item (_("Neither Watched nor Ignored"), FILTER_THREAD_NOTHING)
-	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
-
 	/* attachment */
 	++row;
 	w = d->attach_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article h_as"));
@@ -827,9 +806,9 @@ filter_edit_dialog_new (GtkWindow * window, const FilterTop * top)
 	w = gtk_option_menu_new ();
 	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_FILL, 0, 0, 0);
 	w2 = d->attach_menu = gtk_menu_new ();
-	add_menu_item (_("Attachments"), FILTER_BINARY_COMPLETE)
-	add_menu_item (_("Incomplete Attachments"), FILTER_BINARY_INCOMPLETE)
-	add_menu_item (_("No Attachments"), FILTER_BINARY_NONBINARY)
+	add_menu_item (_("Attachments"), FILTER_BINARY_COMPLETE, d->attach_rb)
+	add_menu_item (_("Incomplete Attachments"), FILTER_BINARY_INCOMPLETE, d->attach_rb)
+	add_menu_item (_("No Attachments"), FILTER_BINARY_NONBINARY, d->attach_rb)
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
 
 	/* read, cached, new, posted-by-me */
@@ -840,11 +819,11 @@ filter_edit_dialog_new (GtkWindow * window, const FilterTop * top)
 	w = gtk_option_menu_new ();
 	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_FILL, 0, 0, 0);
 	w2 = d->article_is_menu = gtk_menu_new ();
-	add_menu_item (_("New"), ARTICLE_IS_NEW)
-	add_menu_item (_("Unread"), ARTICLE_IS_UNREAD)
-	add_menu_item (_("Read"), ARTICLE_IS_READ)
-	add_menu_item (_("Cached"), ARTICLE_IS_CACHED)
-	add_menu_item (_("Posted by Me"), ARTICLE_IS_MINE)
+	add_menu_item (_("New"), ARTICLE_IS_NEW, d->article_is_rb)
+	add_menu_item (_("Unread"), ARTICLE_IS_UNREAD, d->article_is_rb)
+	add_menu_item (_("Read"), ARTICLE_IS_READ, d->article_is_rb)
+	add_menu_item (_("Cached"), ARTICLE_IS_CACHED, d->article_is_rb)
+	add_menu_item (_("Posted by Me"), ARTICLE_IS_MINE, d->article_is_rb)
 	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
 
 	/* lines */
@@ -882,10 +861,16 @@ filter_edit_dialog_new (GtkWindow * window, const FilterTop * top)
 	w = d->score_rb = gtk_radio_button_new_with_mnemonic (rb_group, _("Article _score is at least"));
 	gtk_table_attach (GTK_TABLE(table), w, col, col+1, row, row+1, GTK_FILL, 0, 0, 0);
 	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(w));
-	adj = gtk_adjustment_new (0, -32768, 32767, 1, 1, 1);
-	w = d->score_sb = gtk_spin_button_new (GTK_ADJUSTMENT(adj), 1, 0);
-	g_signal_connect_swapped (GTK_OBJECT(w), "focus_in_event", G_CALLBACK(tb_activate), GTK_OBJECT(d->score_rb));
-	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
+	w = gtk_option_menu_new ();
+	gtk_table_attach (GTK_TABLE(table), w, col+1, col+2, row, row+1, GTK_FILL, 0, 0, 0);
+	w2 = d->score_menu = gtk_menu_new ();
+	add_menu_item (_("Watched"), SCORE_WATCHED, d->score_rb)
+	add_menu_item (_("High"),    SCORE_HIGH,    d->score_rb)
+	add_menu_item (_("Medium"),  SCORE_MEDIUM,  d->score_rb)
+	add_menu_item (_("Zero"),    SCORE_ZERO,    d->score_rb)
+	add_menu_item (_("Low"),     SCORE_LOW,     d->score_rb)
+	add_menu_item (_("Ignored"), SCORE_IGNORED, d->score_rb)
+	gtk_option_menu_set_menu (GTK_OPTION_MENU(w), w2);
 
 	/* prepare the ui state */
 	update_clist (d);
