@@ -33,20 +33,18 @@
 #include <pan/filters/filter-mine.h>
 #include <pan/filters/filter-new.h>
 #include <pan/filters/filter-phrase.h>
-#include <pan/filters/filter-saved.h>
-#include <pan/filters/filter-thread.h>
 #include <pan/filters/filter-top.h>
+#include <pan/filters/filter-score.h>
 #include <pan/filters/filter-manager.h>
 
 #include <pan/filter-mediator.h>
-#include <pan/gui.h>
 #include <pan/util.h>
 
 
 #define DEFAULT_FILTER_SHOW FILTER_SHOW_MATCHES_AND_REFERENCES
 
 static FilterCurrentMediatorCtor _buttons;
-static gulong _filter_bits = ~STATE_FILTER_IGNORED;
+static gulong _filter_bits = ~STATE_FILTER_SCORE_LOW;
 static FilterShow _filter_show = DEFAULT_FILTER_SHOW;
 static void fire_user_changed_filter (void);
 
@@ -128,32 +126,47 @@ refresh (void)
 		gtk_toggle_button_set_active (tb, model);
 
 	/**
-	***  Watched
+	***  Score
 	**/
 
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_watched_ckm);
-	model = _filter_bits & STATE_FILTER_WATCHED ? 1 : 0;
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_watched_ckm);
+	model = _filter_bits & STATE_FILTER_SCORE_WATCHED ? 1 : 0;
+	view = ckm->active ? 1 : 0;
+	if (model != view )
+		gtk_check_menu_item_set_active (ckm, model);
+
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_high_ckm);
+	model = _filter_bits & STATE_FILTER_SCORE_HIGH ? 1 : 0;
 	view = ckm->active ? 1 : 0;
 	if (model != view)
 		gtk_check_menu_item_set_active (ckm, model);
 
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_ignored_ckm);
-	model = _filter_bits & STATE_FILTER_IGNORED ? 1 : 0;
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_medium_ckm);
+	model = _filter_bits & STATE_FILTER_SCORE_MEDIUM ? 1 : 0;
 	view = ckm->active ? 1 : 0;
 	if (model != view)
 		gtk_check_menu_item_set_active (ckm, model);
 
-	ckm = GTK_CHECK_MENU_ITEM(_buttons._match_normal_ckm);
-	model = _filter_bits & STATE_FILTER_NORMAL_RANK ? 1 : 0;
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_zero_ckm);
+	model = _filter_bits & STATE_FILTER_SCORE_ZERO ? 1 : 0;
 	view = ckm->active ? 1 : 0;
 	if (model != view)
 		gtk_check_menu_item_set_active (ckm, model);
 
-	tb = GTK_TOGGLE_BUTTON(_buttons._show_only_watched_threads_tb);
-	model = (_filter_bits & STATE_FILTER_WATCHED)
-		&& !(_filter_bits & STATE_FILTER_IGNORED)
-		&& !(_filter_bits & STATE_FILTER_NORMAL_RANK)
-		&& (_filter_show == FILTER_SHOW_MATCHES_AND_REPLIES);
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_low_ckm);
+	model = _filter_bits & STATE_FILTER_SCORE_LOW ? 1 : 0;
+	view = ckm->active ? 1 : 0;
+	if (model != view )
+		gtk_check_menu_item_set_active (ckm, model);
+
+	ckm = GTK_CHECK_MENU_ITEM (_buttons._match_score_ignored_ckm);
+	model = _filter_bits & STATE_FILTER_SCORE_IGNORED ? 1 : 0;
+	view = ckm->active ? 1 : 0;
+	if (model != view )
+		gtk_check_menu_item_set_active (ckm, model);
+
+	tb = GTK_TOGGLE_BUTTON(_buttons._match_only_watched_tb);
+	model = ((_filter_bits & STATE_FILTER_SCORE_MASK) == STATE_FILTER_SCORE_WATCHED);
 	view = gtk_toggle_button_get_active (tb);
 	if (model != view)
 		gtk_toggle_button_set_active (tb, model);
@@ -336,27 +349,6 @@ show_only_complete_cb (GtkToggleButton * tb, gpointer user_data)
 	set_value (value);
 }
 
-static void
-show_only_watched_cb (GtkToggleButton * tb, gpointer user_data)
-{
-	gulong value = _filter_bits;
-	FilterShow show = _filter_show;
-	const gboolean active = gtk_toggle_button_get_active (tb);
-
-	if (!active) {
-		value |= STATE_FILTER_WATCHED | STATE_FILTER_NORMAL_RANK;
-		show = DEFAULT_FILTER_SHOW;
-	}
-	else {
-		value |= STATE_FILTER_WATCHED;
-		value &= ~STATE_FILTER_IGNORED;
-		value &= ~STATE_FILTER_NORMAL_RANK;
-		show = FILTER_SHOW_MATCHES_AND_REPLIES;
-	}
-
-	set (value, show);
-}
-
 static void
 show_only_cached_cb (GtkToggleButton * tb, gpointer user_data)
 {
@@ -365,10 +357,22 @@ show_only_cached_cb (GtkToggleButton * tb, gpointer user_data)
 
 	if (!active)
 		value |= STATE_FILTER_CACHED | STATE_FILTER_NOT_CACHED;
-	else {
-		value |= STATE_FILTER_WATCHED;
+	else
 		value &= ~STATE_FILTER_NOT_CACHED;
-	}
+
+	set_value (value);
+}
+
+static void
+show_only_watched_cb (GtkToggleButton * tb, gpointer user_data)
+{
+	const gboolean active = gtk_toggle_button_get_active (tb);
+	gulong value = _filter_bits & ~STATE_FILTER_SCORE_MASK;
+
+	if (active)
+		value |= STATE_FILTER_SCORE_WATCHED;
+	else
+		value |= (STATE_FILTER_SCORE_MASK & ~STATE_FILTER_SCORE_IGNORED);
 
 	set_value (value);
 }
@@ -496,6 +500,7 @@ rebuild_custom_filter_menu (void)
 	filter_manager_get_filters (a);
 	rebuild_custom_filter_menu_with_filters (a);
 	pan_g_ptr_array_foreach (a, (GFunc)pan_object_unref, NULL);
+	g_ptr_array_free (a, TRUE);
 }
 
 static int
@@ -547,17 +552,6 @@ filter_mediator_init (FilterCurrentMediatorCtor * ctor)
 	g_signal_connect (G_OBJECT(_buttons._match_only_complete_tb), "toggled",
 	                  G_CALLBACK(show_only_complete_cb), NULL);
 
-	/* Watched */
-
-	g_signal_connect (G_OBJECT(_buttons._match_watched_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_WATCHED));
-	g_signal_connect (G_OBJECT(_buttons._match_ignored_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_IGNORED));
-	g_signal_connect (G_OBJECT(_buttons._match_normal_ckm), "toggled",
-	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_NORMAL_RANK));
-	g_signal_connect (G_OBJECT(_buttons._show_only_watched_threads_tb), "toggled",
-	                  G_CALLBACK(show_only_watched_cb), NULL);
-
 	/* Cached */
 
 	g_signal_connect (G_OBJECT(_buttons._match_cached_ckm), "toggled",
@@ -576,6 +570,23 @@ filter_mediator_init (FilterCurrentMediatorCtor * ctor)
 	g_signal_connect (G_OBJECT(_buttons._show_only_my_threads_tb), "toggled",
 	                  G_CALLBACK(show_only_mine_cb), NULL);
 
+	/* Score */
+
+	g_signal_connect (G_OBJECT(_buttons._match_score_watched_ckm), "toggled",
+	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_WATCHED));
+	g_signal_connect (G_OBJECT(_buttons._match_score_high_ckm), "toggled",
+	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_HIGH));
+	g_signal_connect (G_OBJECT(_buttons._match_score_medium_ckm), "toggled",
+	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_MEDIUM));
+	g_signal_connect (G_OBJECT(_buttons._match_score_zero_ckm), "toggled",
+	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_ZERO));
+	g_signal_connect (G_OBJECT(_buttons._match_score_low_ckm), "toggled",
+	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_LOW));
+	g_signal_connect (G_OBJECT(_buttons._match_score_ignored_ckm), "toggled",
+	                  G_CALLBACK(menu_button_toggled), GSIZE_TO_POINTER(STATE_FILTER_SCORE_IGNORED));
+	g_signal_connect (G_OBJECT(_buttons._match_only_watched_tb), "toggled",
+	                  G_CALLBACK(show_only_watched_cb), NULL);
+
 	/* show */
 
 	g_signal_connect (G_OBJECT(_buttons._show_articles_rmi), "toggled",
@@ -639,17 +650,18 @@ filter_mediator_get_filter  (gulong        bits,
 	const gboolean show_bin = (l & STATE_FILTER_COMPLETE_BINARIES) != 0;
 	const gboolean show_inc_bin = (l & STATE_FILTER_INCOMPLETE_BINARIES) != 0;
 	const gboolean show_nonbinary = (l & STATE_FILTER_NONBINARIES) != 0;
-	const gboolean show_watched = (l & STATE_FILTER_WATCHED) != 0;
-	const gboolean show_ignored = (l & STATE_FILTER_IGNORED) != 0;
-	const gboolean show_normal = (l & STATE_FILTER_NORMAL_RANK) != 0;
-	const gboolean show_saved = (l & STATE_FILTER_SAVED) != 0;
-	const gboolean show_idle = (l & STATE_FILTER_IDLE) != 0;
-	const gboolean show_queued = (l & STATE_FILTER_QUEUED) != 0;
 	const gboolean show_cached = (l & STATE_FILTER_CACHED) != 0;
 	const gboolean show_non_cached = (l & STATE_FILTER_NOT_CACHED) != 0;
 	const gboolean show_by_me = (l & STATE_FILTER_MINE) != 0;
 	const gboolean show_by_others = (l & STATE_FILTER_NOT_MINE) != 0;
 
+	const gboolean show_score_watched = (l & STATE_FILTER_SCORE_WATCHED) != 0;
+	const gboolean show_score_high    = (l & STATE_FILTER_SCORE_HIGH) != 0;
+	const gboolean show_score_medium  = (l & STATE_FILTER_SCORE_MEDIUM) != 0;
+	const gboolean show_score_zero    = (l & STATE_FILTER_SCORE_ZERO) != 0;
+	const gboolean show_score_low     = (l & STATE_FILTER_SCORE_LOW) != 0;
+	const gboolean show_score_ignored = (l & STATE_FILTER_SCORE_IGNORED) != 0;
+
 	/* top */
 	top = FILTER_AGGREGATE(filter_aggregate_new ());
 	filter_aggregate_set_type (top, AGGREGATE_TYPE_AND);
@@ -708,60 +720,6 @@ filter_mediator_get_filter  (gulong        bits,
 		pan_object_unref (PAN_OBJECT(filter));
 	}
 
-	/* thread state */
-	n = 0;
-	if (show_watched) ++n;
-	if (show_ignored) ++n;
-	if (show_normal) ++n;
-	if (n==1 || n==2) {
-		Filter * filter = filter_thread_new ();
-		if (n==1) {
-			if (show_watched)
-				filter_thread_set_state (FILTER_THREAD(filter), FILTER_THREAD_WATCHED);
-			else if (show_ignored)
-				filter_thread_set_state (FILTER_THREAD(filter), FILTER_THREAD_IGNORED);
-			else if (show_normal)
-				filter_thread_set_state (FILTER_THREAD(filter), FILTER_THREAD_NOTHING);
-		} else {
-			if (!show_watched)
-				filter_thread_set_state (FILTER_THREAD(filter), FILTER_THREAD_WATCHED);
-			else if (!show_ignored)
-				filter_thread_set_state (FILTER_THREAD(filter), FILTER_THREAD_IGNORED);
-			else if (!show_normal)
-				filter_thread_set_state (FILTER_THREAD(filter), FILTER_THREAD_NOTHING);
-			filter_negate (filter);
-		}
-		filter_aggregate_add (top, &filter, 1);
-		pan_object_unref (PAN_OBJECT(filter));
-	}
-
-	/* saved/queued/idle */
-	n = 0;
-	if (show_saved) ++n;
-	if (show_queued) ++n;
-	if (show_idle) ++n;
-	if (n==1 || n==2) {
-		Filter * filter = filter_saved_new ();
-		if (n==1) {
-			if (show_saved)
-				filter_saved_set_state (FILTER_SAVED(filter), FILTER_SAVED_SAVED);
-			else if (show_queued)
-				filter_saved_set_state (FILTER_SAVED(filter), FILTER_SAVED_QUEUED);
-			else if (show_idle)
-				filter_saved_set_state (FILTER_SAVED(filter), FILTER_SAVED_NOT_SAVED);
-		} else {
-			if (!show_saved)
-				filter_saved_set_state (FILTER_SAVED(filter), FILTER_SAVED_SAVED);
-			else if (!show_queued)
-				filter_saved_set_state (FILTER_SAVED(filter), FILTER_SAVED_QUEUED);
-			else if (!show_idle)
-				filter_saved_set_state (FILTER_SAVED(filter), FILTER_SAVED_NOT_SAVED);
-			filter_negate (filter);
-		}
-		filter_aggregate_add (top, &filter, 1);
-		pan_object_unref (PAN_OBJECT(filter));
-	}
-
 	/* cached */
 	n = 0;
 	if (show_cached) ++n;
@@ -787,6 +745,23 @@ filter_mediator_get_filter  (gulong        bits,
 		pan_object_unref (PAN_OBJECT(filter));
 	}
 
+	/* score */
+	{
+		FilterScoreMode mode = 0;
+		if (show_score_watched)  mode |= SCORE_WATCHED;
+		if (show_score_high)     mode |= SCORE_HIGH;
+		if (show_score_medium)   mode |= SCORE_MEDIUM;
+		if (show_score_zero)     mode |= SCORE_ZERO;
+		if (show_score_low)      mode |= SCORE_LOW;
+		if (show_score_ignored)  mode |= SCORE_IGNORED;
+		if (mode != 0)
+		{
+			Filter * filter = filter_score_new (mode);
+			filter_aggregate_add (top, &filter, 1);
+			pan_object_unref (PAN_OBJECT(filter));
+		}
+	}
+
 	/* custom filters */
 	for (i=1; i<_custom_filter_entry_qty; ++i) {
 		GtkWidget * w = GTK_WIDGET (_custom_filter_entries[i].extra_data);
