@@ -36,9 +36,13 @@ typedef struct
 	GtkWidget * _match_nonbinaries_ckm;
 	GtkWidget * _match_only_complete_tb;
 
-	GtkWidget * _match_watched_ckm;
-	GtkWidget * _match_ignored_ckm;
-	GtkWidget * _match_normal_ckm;
+	GtkWidget * _match_score_watched_ckm;
+	GtkWidget * _match_score_high_ckm;
+	GtkWidget * _match_score_medium_ckm;
+	GtkWidget * _match_score_zero_ckm;
+	GtkWidget * _match_score_low_ckm;
+	GtkWidget * _match_score_ignored_ckm;
+	GtkWidget * _match_only_watched_tb;
 
 	GtkWidget * _match_mine_ckm;
 	GtkWidget * _match_others_ckm;
