@@ -41,7 +41,7 @@ flagset_has_message (const char * message_id)
 	return g_hash_table_lookup (_id_to_mid, message_id) != NULL;
 }
 
-void
+static void
 flagset_add_messages (MessageIdentifier  ** mids,
                       int                   qty)
 {
@@ -70,7 +70,7 @@ flagset_add_articles (const Article   ** articles,
 	g_return_if_fail (articles_are_valid(articles, qty));
 
 	/* convert to message-identifiers */
-	mids = g_alloca (sizeof(MessageIdentifier*) * qty);
+	mids = g_newa (MessageIdentifier*, qty);
 	for (i=0; i<qty; ++i)
 		mids[i] = message_identifier_new_from_article (articles[i]);
 
@@ -82,7 +82,7 @@ flagset_add_articles (const Article   ** articles,
 		g_object_unref (mids[i]);
 }
 
-void
+static void
 flagset_remove_messages (MessageIdentifier  ** mids,
                          int                   qty)
 {
@@ -109,7 +109,7 @@ flagset_remove_articles (const Article   ** articles,
 	g_return_if_fail (articles_are_valid(articles, qty));
 
 	/* convert to message-identifiers */
-	mids = g_alloca (sizeof(MessageIdentifier*) * qty);
+	mids = g_newa (MessageIdentifier*, qty);
 	for (i=0; i<qty; ++i)
 		mids[i] = message_identifier_new_from_article (articles[i]);
 
