@@ -20,23 +20,17 @@
 #ifndef FLAGSET_H
 #define FLAGSET_H
 
-#include <glib.h>
-#include <pan/base/pan-callback.h>
+#include <glib/gtypes.h>
+#include <pan/base/article.h>
 #include <pan/base/message-identifier.h>
 
 void          flagset_init            (void);
 
 gboolean      flagset_has_message     (const char          * message_id);
 
-void          flagset_add_messages    (MessageIdentifier  ** mids,
-                                       int                   qty);
-
 void          flagset_add_articles    (const Article      ** articles,
                                        int                   qty);
 
-void          flagset_remove_messages (MessageIdentifier  ** mids,
-                                       int                   qty);
-
 void          flagset_remove_articles (const Article      ** articles,
                                        int                   qty);
 
