@@ -1163,10 +1163,10 @@ void
 grouplist_update_font (void)
 {
 	pan_lock ();
-	if (!use_custom_fonts)
+	if (!group_pane_custom_font_enabled)
 		gtk_widget_modify_font (Pan.group_tree, gtk_widget_get_default_style()->font_desc);
 	else {
-		PangoFontDescription * pfd = pango_font_description_from_string (group_pane_font);
+		PangoFontDescription * pfd = pango_font_description_from_string (group_pane_custom_font);
 		gtk_widget_modify_font (Pan.group_tree, pfd);
 		pango_font_description_free (pfd);
 	}
