@@ -59,7 +59,7 @@ gui_notebook_construct (void)
 		gtk_notebook_append_page (GTK_NOTEBOOK (notebook), text_box, gtk_label_new (_("Body")));
 		gtk_notebook_set_tab_border (GTK_NOTEBOOK (notebook), 2);
 		gui_restore_column_widths (Pan.group_tree, "group");
-		gui_restore_column_widths (Pan.article_ctree, "thread_pane");
+		gui_restore_column_widths (Pan.article_ctree, "header_pane");
 		gtk_box_pack_start (GTK_BOX(Pan.workarea), notebook, TRUE, TRUE, 0);
 		gtk_widget_show_all (GTK_WIDGET(Pan.workarea));
 
