@@ -47,6 +47,9 @@
 #include <pan/identities/identity-manager.h>
 #include <pan/identities/identity-ui.h>
 
+#include <pan/filters/score.h>
+
+#include <pan/action.h>
 #include <pan/article-actions.h>
 #include <pan/article-find.h>
 #include <pan/articlelist.h>
@@ -66,6 +69,8 @@
 #include <pan/prefs.h>
 #include <pan/print.h>
 #include <pan/save-ui.h>
+#include <pan/score-add-ui.h>
+#include <pan/score-view-ui.h>
 #include <pan/server-menu-ui.h>
 #include <pan/server-ui.h>
 #include <pan/sockets.h>
@@ -76,12 +81,11 @@
 #include <pan/task-post.h>
 #include <pan/util.h>
 #include <pan/queue.h>
+
 #include <pan/dialogs/dialogs.h>
 
 #include <pan/xpm/pan-pixbufs.h>
 
-static void delete_last_task_cb (gpointer, int, GtkWidget*);
-
 GtkTooltips * ttips = NULL;
 GdkColormap * cmap = NULL;
 
@@ -89,31 +93,6 @@ GdkColormap * cmap = NULL;
 ***  Actions
 **/
 
-typedef enum
-{
-	ACTION_COMPOSE_NEW,
-	ACTION_COMPOSE_FOLLOWUP,
-	ACTION_GET_NEW_HEADERS_FOR_SUBSCRIBED,
-	ACTION_GET_NEW_HEADERS_FOR_SELECTED,
-	ACTION_GET_BODIES_FOR_SELECTED,
-	ACTION_READ_MORE,
-	ACTION_READ_LESS,
-	ACTION_READ_NEXT_UNREAD_ARTICLE,
-	ACTION_READ_NEXT_UNREAD_THREAD,
-	ACTION_READ_NEXT_UNREAD_GROUP,
-	ACTION_SAVE,
-	ACTION_SAVE_AS,
-	ACTION_SHOW_ONLY_NEW,
-	ACTION_SHOW_ONLY_MY_THREADS,
-	ACTION_SHOW_ONLY_CACHED,
-	ACTION_SHOW_ONLY_ATTACHMENTS,
-	ACTION_SHOW_ONLY_WATCHED_THREADS,
-	ACTION_NET_ONLINE,
-	ACTION_NET_CANCEL_LAST_TASK,
-	ACTION_QTY
-}
-ToolbarAction;
-
 GtkWidget * _toolbar_buttons[ACTION_QTY];
 
 /**
@@ -157,85 +136,18 @@ static ViewStruct views[VIEW_QTY];
 ***  MENUS
 **/
 
-#define MAIN_MENU_ENTRIES_QTY 156
+#define MAIN_MENU_ENTRIES_QTY 163
 static GtkItemFactoryEntry main_menu_entries[MAIN_MENU_ENTRIES_QTY];
 GtkItemFactory * _main_menu_factory;
 
 
 
 static int current_layout = -1;
-static gboolean pan_maximized = FALSE;
 
 GtkWidget * groups_vbox;
 GtkWidget * articlelist_ctree;
 GtkWidget * text_box;
 
-
-/****
-*****
-*****   SECTION:  GUI QUEUE
-*****
-****/
-
-static GStaticMutex gui_queue_mutex = G_STATIC_MUTEX_INIT;
-
-static GSList * gui_queue_slist = NULL;
-
-typedef struct
-{
-	GSourceFunc func;
-	gpointer user_data;
-}
-GuiQueueItem;
-
-static int
-gui_queue_timer_cb (gpointer user_data)
-{
-	GSList * list;
-
-	/* get the work list */
-	g_static_mutex_lock (&gui_queue_mutex);
-	list = gui_queue_slist;
-	gui_queue_slist = NULL;
-	g_static_mutex_unlock (&gui_queue_mutex);
-
-	if (list != NULL)
-	{
-		GSList * l;
-
-		/* gui_queue_slist is LIFO for efficiency of adding nodes;
-		 * reverse the nodes to make list FIFO */
-		list = g_slist_reverse (list);
-
-		/* run the work functions */
-		for (l=list; l!=NULL; l=l->next)
-		{
-			GuiQueueItem * queue_item = (GuiQueueItem*) l->data;
-			(*queue_item->func)(queue_item->user_data);
-			g_free (queue_item);
-		}
-		g_slist_free (list);
-	}
-
-	return 1;
-}
-
-void
-gui_queue_add (GSourceFunc func, gpointer user_data)
-{
-	GuiQueueItem * item;
-
-	/* create the new queue item */	
-	item = g_new (GuiQueueItem, 1);
-	item->func = func;
-	item->user_data = user_data;
-
-	/* add it to the queue */
-	g_static_mutex_lock (&gui_queue_mutex);
-	gui_queue_slist = g_slist_prepend (gui_queue_slist, item);
-	g_static_mutex_unlock (&gui_queue_mutex);
-}
-
 /****
 *****
 *****   SECTION:  GUI UPKEEP
@@ -436,14 +348,15 @@ update_menus_idle (gpointer data)
 	menu_set_sensitive (gif, "/Articles/Download", have_article);
 	menu_set_sensitive (gif, "/Articles/Watch Thread", have_article);
 	menu_set_sensitive (gif, "/Articles/Ignore Thread", have_article);
-	menu_set_sensitive (gif, "/Articles/Add to Bozo or Spam Filter...", have_article);
+	menu_set_sensitive (gif, "/Articles/Plonk Author", have_article);
+	menu_set_sensitive (gif, "/Articles/View Article's Scores", have_article);
 	menu_set_sensitive (gif, "/Articles/Cancel...", have_article);
 	menu_set_sensitive (gif, "/Articles/Supersede...", have_article);
 	menu_set_sensitive (gif, "/Articles/Delete",  have_article);
 
 	/* update file menu */
-	menu_set_sensitive (gif, "/File/Save Article As...", have_article);
-	menu_set_sensitive (gif, "/File/Save Article Attachments", have_article);
+	menu_set_sensitive (gif, "/File/Save Attachments", have_article);
+	menu_set_sensitive (gif, "/File/Save Attachments As...", have_article);
 	menu_set_sensitive (gif, "/File/Cancel Last Task", !queue_is_empty());
 	menu_set_sensitive (gif, "/File/Print Article...",  have_article);
 	menu_set_sensitive (gif, "/File/Delete Folder...", have_glist_folder && strncmp(glist_group->name, "pan.", 4));
@@ -692,10 +605,6 @@ status_item_active_changed_cb (gpointer call_obj, gpointer call_arg, gpointer us
 
 	pan_object_ref (PAN_OBJECT(item));
 
-	/**
-	***  Push the GUI work to the main thread
-	**/
-
 	gui_queue_add (is_active ? gui_add_status_item_idle : gui_remove_status_item_idle, item);
 
 
@@ -739,7 +648,7 @@ text_fill_body_changed_cb (gpointer call_obj, gpointer call_arg, gpointer user_d
 void
 gui_layout_clear_workarea (void)
 {
-	/* clear out the workarea */
+	/* remove the three panes from the workarea */
 	gtk_widget_hide_all (GTK_WIDGET(Pan.workarea));
 	if (groups_vbox->parent != NULL)
 		gtk_container_remove (GTK_CONTAINER(groups_vbox->parent), groups_vbox);
@@ -748,7 +657,7 @@ gui_layout_clear_workarea (void)
 	if (text_box->parent != NULL)
 		gtk_container_remove (GTK_CONTAINER(text_box->parent), text_box);
 
-	/* make a new UI */
+	/* clear the workarea */
 	if (1) {
 		GList * children = gtk_container_children (GTK_CONTAINER(Pan.workarea));
 		GList * l;
@@ -1003,96 +912,6 @@ gui_key_press_cb (GtkWidget      * widget,
 ****/
 
 static unsigned int gui_refresh_timer_id = 0;
-static unsigned int gui_queue_timer_id = 0;
-
-void
-gui_save_column_widths (GtkWidget    * clist,
-                        const char   * type)
-{
-	if (GTK_IS_CLIST (clist))
-	{
-		int i;
-		int cols = GTK_CLIST (clist)->columns;
-		GtkCList * list = GTK_CLIST (clist);
-		char buf[1024];
-		
-		for (i=0; i<cols; i++)
-		{
-			g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
-			pan_config_set_int (buf, (int)list->column[i].width);
-		}
-
-
-		pan_config_sync ();
-	}
-}
-
-void
-gui_save_column_widths_tree_view (GtkWidget    * tree_view,
-				  const char   * type)
-{
-	GList * columns;
-	char buf[1024];	
-	int i;
-	int length;
-
-	debug_enter ("gui_save_column_widths_tree_view");
-
-	columns = gtk_tree_view_get_columns(GTK_TREE_VIEW(tree_view));
-	length = g_list_length(columns);
-
-	for (i = 0; i < length; i++) {
-		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
-		pan_config_set_int (buf, (int)gtk_tree_view_column_get_width(gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), i)));
-	}
-
-	pan_config_sync ();
-
-	/* cleanup */
-	g_list_free(columns);
-
-	debug_exit ("gui_save_column_widths_tree_view");	
-}
-
-
- 
-void
-gui_save_window_size (GtkWidget     * widget,
-                      const char    * key)
-{
-	char buf[1024];
-	int x, y, w, h;
-
-	/* sanity clause */
-	g_return_if_fail (GTK_IS_WIDGET(widget));
-	g_return_if_fail (widget->window!=NULL);
-	g_return_if_fail (is_nonempty_string(key));
-
-	if ((widget != Pan.window) || !pan_maximized)
-	{
-		gtk_window_get_position (GTK_WINDOW(widget), &x, &y);
-		gtk_window_get_size (GTK_WINDOW(widget), &w, &h);
-
-		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_x", key);
-		pan_config_set_int (buf, x);
-		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_y", key);
-		pan_config_set_int (buf, y);
-		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_width", key);
-		pan_config_set_int (buf, w);
-		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_height", key);
-		pan_config_set_int (buf, h);
-	}
-
-	if (widget == Pan.window)
-	{
-		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_max", key);
-		pan_config_set_bool (buf, pan_maximized);
-	}
-
-	pan_config_sync ();
-
-
-}
 
 static void
 get_accel_filename (char * buf, int buf_max)
@@ -1136,27 +955,22 @@ gui_shutdown (void)
 
 	g_source_remove (gui_refresh_timer_id);
 	gui_refresh_timer_id = 0;
-	g_source_remove (gui_queue_timer_id);
-	gui_queue_timer_id = 0;
+	gui_queue_shutdown ();
 
 	/**
 	***  Save settings
 	**/
 
-	pan_config_push_prefix ("/Pan/Display/");
-	pan_config_set_bool ("show_group_pane", show_group_pane);
-	pan_config_set_bool ("header_pane_is_threaded", header_pane_is_threaded);
-	pan_config_set_bool ("show_header_pane", show_header_pane);
-	pan_config_set_bool ("show_body_pane", show_body_pane);
-	pan_config_set_bool ("collapse_group_names", collapse_group_names);
-	pan_config_set_bool ("do_wrap", text_get_wrap());
-	pan_config_set_bool ("use_fixed_font", text_use_fixed_font);
-	pan_config_set_bool ("Show_All_Headers_In_Body", text_get_show_all_headers());
-	pan_config_set_bool ("Mute_Quoted_text", text_get_mute_quoted());
-
-	pan_config_pop_prefix ();
-
-	pan_config_set_int ("/Pan/State/viewmode", Pan.viewmode);
+	pan_config_set_bool (KEY_GROUP_PANE_ENABLED, show_group_pane);
+	pan_config_set_bool (KEY_HEADER_PANE_THREADING_ENABLED, header_pane_is_threaded);
+	pan_config_set_bool (KEY_HEADER_PANE_ENABLED, show_header_pane);
+	pan_config_set_bool (KEY_BODY_PANE_ENABLED, show_body_pane);
+	pan_config_set_bool (KEY_GROUP_PANE_COLLAPSE_GROUP_NAMES, collapse_group_names);
+	pan_config_set_bool (KEY_BODY_PANE_WRAP_ENABLED, text_get_wrap());
+	pan_config_set_bool (KEY_BODY_PANE_MONOSPACE_FONT_ENABLED, body_pane_monospace_font_enabled);
+	pan_config_set_bool (KEY_BODY_PANE_SHOW_ALL_HEADERS, text_get_show_all_headers());
+	pan_config_set_bool (KEY_BODY_PANE_MUTE_QUOTED_TEXT, text_get_mute_quoted());
+	pan_config_set_int  (KEY_PANE_MODE, Pan.viewmode);
 
 	grouplist_shutdown_module ();
 
@@ -1201,9 +1015,8 @@ gui_shutdown (void)
 	}
 
 	gui_save_column_widths (Pan.group_tree, "group");
-	gui_save_column_widths (Pan.article_ctree, "thread_pane");
+	gui_save_column_widths (Pan.article_ctree, "header_pane");
 	gui_save_window_size (Pan.window, "main_window");
-	pan_config_sync();
 
 	/**
 	*** Save accelerators
@@ -1231,136 +1044,6 @@ gui_shutdown (void)
 *****
 ****/
 
-void
-pan_widget_set_font (GtkWidget * w, const char * font_name)
-{
-	PangoFontDescription * font_desc;
-
-	g_return_if_fail (GTK_IS_WIDGET(w));
-	g_return_if_fail (is_nonempty_string(font_name));
-
-	font_desc = pango_font_description_from_string (font_name);
-	gtk_widget_modify_font (w, font_desc);
-	pango_font_description_free (font_desc);
-}
-
-void
-gui_restore_column_widths (GtkWidget     * clist,
-                           const char   * type)
-{
-	if (GTK_IS_CLIST (clist))
-	{
-		int i;
-		GtkCList * list = GTK_CLIST (clist);
-		const int cols = list->columns;
-		int * widths = g_newa (int, cols);
-		char buf[1024];
-		debug_enter ("gui_restore_column_widths");
-
-		/* get width from config... */
-		for (i=0; i!=cols; ++i) {
-			g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
-			widths[i] = pan_config_get_int (buf);
-		}
-
-		/* set ui.. */
-		pan_lock();
-		for (i=0; i!=cols; ++i)
-			if (widths[i] != 0)
-				gtk_clist_set_column_width (list, i, widths[i]);
-		pan_unlock();
-
-		/* cleanup */
-		debug_exit ("gui_restore_column_widths");
-	}
-}
-
-void
-gui_restore_column_widths_tree_view (GtkWidget     * tree_view,
-				     const char   * type)
-{
-	GList * columns;
-	char buf[1024];	
-	int * widths;
-	int i;
-	int length;
-		
-	debug_enter ("gui_restore_column_widths_tree_view");
-	
-	columns = gtk_tree_view_get_columns(GTK_TREE_VIEW(tree_view));
-	length = g_list_length(columns);
-	widths = g_newa (int, length);
-	
-	/* get width from config... */
-	for (i = 0; i != length; ++i) {
-		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
-		widths[i] = pan_config_get_int (buf);
-	}
-
-	/* set ui.. */
-	pan_lock();
-	for (i = 0; i != length; ++i) {
-		if (widths[i] != 0)
-			gtk_tree_view_column_set_fixed_width(gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), i), widths[i]);
-	}
-	
-	pan_unlock();
-	
-	/* cleanup */
-	g_list_free(columns);
-	
-	debug_exit ("gui_restore_column_widths_tree_view");
-}
-
-
-
-gboolean
-gui_restore_window_size (GtkWidget     * widget,
-                         const char    * key)
-{
-	int x, y, w, h;
-	char buf[1024];
-	int screen_w = gdk_screen_width ();
-	int screen_h = gdk_screen_height ();
-	const int fuzz = 10;
-	gboolean retval = FALSE;
-
-	/* sanity clause */
-	g_return_val_if_fail (GTK_IS_WIDGET(widget), FALSE);
-	g_return_val_if_fail (is_nonempty_string(key), FALSE);
-
-	/* get the dimensions */
-	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_x", key);
-	x = pan_config_get_int (buf);
-	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_y", key);
-	y = pan_config_get_int (buf);
-	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_width", key);
-	w = pan_config_get_int (buf);
-	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_height", key);
-	h = pan_config_get_int (buf);
-
-	/* move & resize */
-	if ((0<=x && x<screen_w-fuzz) && (0<=y && y<screen_h-fuzz))
-		gtk_window_move (GTK_WINDOW(widget), x, y);
-	if (w>0 && h>0) {
-		gtk_window_set_default_size (GTK_WINDOW(widget), w, h);
-		retval = TRUE;
-	}
-
-	if (widget == Pan.window)
-	{
-		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_max", key);
-		pan_maximized = pan_config_get_bool (buf);
-
-		if (pan_maximized)
-			gtk_window_maximize (GTK_WINDOW(widget));
-		else
-			gtk_window_unmaximize (GTK_WINDOW(widget));
-	}
-
-	return retval;
-}
-
 static void
 queue_button_clicked (GtkButton *button,
                       gpointer user_data)
@@ -1443,21 +1126,6 @@ window_delete_event_cb (GtkWidget *widget,
 	return FALSE; /* causes "destroy" signal to be fired */
 }
 
-static int
-window_state_event_cb (GtkWidget *widget,
-			GdkEvent *event,
-			gpointer  data)
-{
-	if( (event->type) == (GDK_WINDOW_STATE))
-	{
-		if((((GdkEventWindowState*)event)->new_window_state) & GDK_WINDOW_STATE_MAXIMIZED)
-			pan_maximized = TRUE;	
-		else
-			pan_maximized = FALSE;	
-	}
-	return FALSE;
-}
-
 static char*
 menu_translate(const char* path, gpointer data)
 {
@@ -1579,74 +1247,14 @@ gui_add_network_button_to_toolbar (GtkWidget * toolbar)
 static void
 toolbar_cb (GtkWidget * button, gpointer user_data)
 {
-	ToolbarAction action = (ToolbarAction) user_data;
-
-	switch (action)
-	{
-		case ACTION_COMPOSE_NEW:
-			message_post_window ();
-			break;
-
-		case ACTION_COMPOSE_FOLLOWUP:
-			message_followup_window ();
-			break;
-
-		case ACTION_GET_NEW_HEADERS_FOR_SUBSCRIBED:
-			group_action_subscribed_download_new ();
-			break;
-
-		case ACTION_GET_NEW_HEADERS_FOR_SELECTED:
-			group_action_selected_download_new ();
-			break;
-
-		case ACTION_GET_BODIES_FOR_SELECTED:
-			group_action_selected_download_new_and_bodies ();
-			break;
-
-		case ACTION_READ_MORE:
-			text_read_more ();
-			break;
-
-		case ACTION_READ_LESS:
-			text_read_less ();
-			break;
-
-		case ACTION_READ_NEXT_UNREAD_ARTICLE:
-			articlelist_read_next_unread ();
-			break;
-
-		case ACTION_READ_NEXT_UNREAD_THREAD:
-			articlelist_read_next_unread_thread ();
-			break;
-
-		case ACTION_READ_NEXT_UNREAD_GROUP:
-			grouplist_activate_next_unread_group ();
-			break;
-
-		case ACTION_SAVE:
-			article_action_selected_save_attachments ();
-			break;
-
-		case ACTION_SAVE_AS:
-			article_action_selected_save_as ();
-			break;
-
-		case ACTION_NET_ONLINE:
-			break;
-
-		case ACTION_NET_CANCEL_LAST_TASK:
-			delete_last_task_cb (NULL, 0, NULL);
-			break;
-
-		default:
-			pan_warn_if_reached ();
-			break;
-	}
+	const PanAction action = (PanAction) user_data;
+	pan_action_do (action);
 }
 static void
-menu_cb (gpointer user_data, int action, GtkWidget * w)
+menu_cb (gpointer user_data, int iaction, GtkWidget * w)
 {
-	toolbar_cb (w, GINT_TO_POINTER(action));
+	const PanAction action = (PanAction) iaction;
+	pan_action_do (action);
 }
 
 
@@ -1730,10 +1338,10 @@ create_toolbar (void)
 		create_image (icon_read_group),
 		G_CALLBACK(toolbar_cb), GINT_TO_POINTER(ACTION_READ_NEXT_UNREAD_GROUP));
 	_toolbar_buttons[ACTION_SAVE] = gtk_toolbar_insert_stock (
-		toolbar, GTK_STOCK_SAVE, _("Save Article Attachments"), NULL,
+		toolbar, GTK_STOCK_SAVE, _("Save Attachments"), NULL,
 		G_CALLBACK(toolbar_cb), GINT_TO_POINTER(ACTION_SAVE), -1);
 	_toolbar_buttons[ACTION_SAVE_AS] = gtk_toolbar_insert_stock (
-		toolbar, GTK_STOCK_SAVE_AS, _("Save Article As..."), NULL,
+		toolbar, GTK_STOCK_SAVE_AS, _("Save Attachments As..."), NULL,
 		G_CALLBACK(toolbar_cb), GINT_TO_POINTER(ACTION_SAVE_AS), -1);
 	gtk_toolbar_append_space (toolbar);
 
@@ -1747,17 +1355,17 @@ create_toolbar (void)
 	_toolbar_buttons[ACTION_SHOW_ONLY_NEW] = gtk_toolbar_append_element (
 		toolbar, GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, _("New"), _("Show Only New Articles"), NULL,
 		create_image (icon_filter_only_new), NULL, NULL);
-	_toolbar_buttons[ACTION_SHOW_ONLY_MY_THREADS] = gtk_toolbar_append_element (
-		toolbar, GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, _("Mine"), _("Show Only Replies to My Articles"), NULL,
-		create_image (icon_filter_only_me), NULL, NULL);
 	_toolbar_buttons[ACTION_SHOW_ONLY_CACHED] = gtk_toolbar_append_element (
 		toolbar, GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, _("Cached"), _("Show Only Cached Articles"), NULL,
 		create_image (icon_filter_only_cached), NULL, NULL);
 	_toolbar_buttons[ACTION_SHOW_ONLY_ATTACHMENTS] = gtk_toolbar_append_element (
 		toolbar, GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, _("Files"), _("Show Only Complete Attachments"), NULL,
 		create_image (icon_filter_only_attachments), NULL, NULL);
-	_toolbar_buttons[ACTION_SHOW_ONLY_WATCHED_THREADS] = gtk_toolbar_append_element (
-		toolbar, GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, _("Watched"), _("Show Only Watched Threads"), NULL,
+	_toolbar_buttons[ACTION_SHOW_ONLY_MY_THREADS] = gtk_toolbar_append_element (
+		toolbar, GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, _("Mine"), _("Show Only Replies to My Articles"), NULL,
+		create_image (icon_filter_only_me), NULL, NULL);
+	_toolbar_buttons[ACTION_SHOW_ONLY_WATCHED] = gtk_toolbar_append_element (
+		toolbar, GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL, _("Score"), _("Show Only Watched Articles"), NULL,
 		create_image (icon_filter_only_watched), NULL, NULL);
 	gtk_toolbar_append_space (toolbar);
 
@@ -1803,8 +1411,6 @@ gui_construct (const char * geometry)
 	                  G_CALLBACK (window_delete_event_cb), NULL);
 	g_signal_connect (GTK_OBJECT(Pan.window), "destroy",
 	                  G_CALLBACK (pan_shutdown), NULL);
-	g_signal_connect (GTK_OBJECT(Pan.window), "window_state_event",
-	                  G_CALLBACK (window_state_event_cb), NULL);
 
 	/* set the window icon */
 	if (1) {
@@ -1873,7 +1479,7 @@ gui_construct (const char * geometry)
 	gtk_widget_ref (text_box);
 
 	/* this also updates menu sensitivity */
-	gui_layout_set_mode (pan_config_get_int ("/Pan/State/viewmode=1"));
+	gui_layout_set_mode (pan_config_get_int (KEY_PANE_MODE "=1"));
 
 	pan_callback_add (article_get_articles_changed_callback(), articles_changed_cb, NULL);
 	pan_callback_add (articlelist_get_group_changed_callback(), update_menus_cb, NULL);
@@ -1894,53 +1500,57 @@ gui_construct (const char * geometry)
 	menu_set_checked (_main_menu_factory, "/View/Abbreviate Names in Group Pane", collapse_group_names);
 	menu_set_checked (_main_menu_factory, "/View/Wrap Article Body", text_get_wrap());
 	menu_set_checked (_main_menu_factory, "/View/Show All Headers in Body Pane", text_get_show_all_headers());
-	menu_set_checked (_main_menu_factory, "/View/Use Monospace Font in Body Pane", text_use_fixed_font);
+	menu_set_checked (_main_menu_factory, "/View/Use Monospace Font in Body Pane", body_pane_monospace_font_enabled);
 	menu_set_checked (_main_menu_factory, "/View/Thread Header Pane", header_pane_is_threaded);
 	menu_set_checked (_main_menu_factory, "/View/Show Group Pane", show_group_pane);
 	menu_set_checked (_main_menu_factory, "/View/Show Header Pane", show_header_pane);
 	menu_set_checked (_main_menu_factory, "/View/Show Body Pane", show_body_pane);
 
-	text_set_show_all_headers (pan_config_get_bool("/Pan/Display/Show_All_Headers_In_Body=false"));
-	text_set_mute_quoted (pan_config_get_bool("/Pan/Display/Mute_Quoted_Text=false"));
+	text_set_show_all_headers (pan_config_get_bool(KEY_BODY_PANE_SHOW_ALL_HEADERS "=FALSE"));
+	text_set_mute_quoted (pan_config_get_bool(KEY_BODY_PANE_MUTE_QUOTED_TEXT "=FALSE"));
 	gui_refresh_timer_id = pan_timeout_add (3000, gui_refresh_timer_cb, NULL); 
-	gui_queue_timer_id = pan_timeout_add (100, gui_queue_timer_cb, NULL);
+	gui_queue_init ();
 	gtk_widget_show_all (Pan.window);
 	server_menu_update ();
 
 	if (1)
 	{
+#define get_menu_widget(KEY) gtk_item_factory_get_item (_main_menu_factory, KEY)
 		FilterCurrentMediatorCtor ctor;
 
-		ctor._match_new_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match New Articles");
-		ctor._match_unread_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Unread Articles");
-		ctor._match_read_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Read Articles");
-		ctor._match_only_new_tb = _toolbar_buttons[ACTION_SHOW_ONLY_NEW];
+		ctor._match_new_ckm                  = get_menu_widget ( "/Filter/Match New Articles");
+		ctor._match_unread_ckm               = get_menu_widget ( "/Filter/Match Unread Articles");
+		ctor._match_read_ckm                 = get_menu_widget ( "/Filter/Match Read Articles");
+		ctor._match_only_new_tb              = _toolbar_buttons[ACTION_SHOW_ONLY_NEW];
 
-		ctor._match_complete_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Articles with Complete Attachments");
-		ctor._match_incomplete_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Articles with Incomplete Attachments");
-		ctor._match_nonbinaries_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Articles with No Attachments");
-		ctor._match_only_complete_tb = _toolbar_buttons[ACTION_SHOW_ONLY_ATTACHMENTS];
+		ctor._match_complete_ckm             = get_menu_widget ( "/Filter/Match Articles with Complete Attachments");
+		ctor._match_incomplete_ckm           = get_menu_widget ( "/Filter/Match Articles with Incomplete Attachments");
+		ctor._match_nonbinaries_ckm          = get_menu_widget ( "/Filter/Match Articles with No Attachments");
+		ctor._match_only_complete_tb         = _toolbar_buttons[ACTION_SHOW_ONLY_ATTACHMENTS];
 
-		ctor._match_watched_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Watched Threads");
-		ctor._match_ignored_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Ignored Threads");
-		ctor._match_normal_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Threads Neither Watched nor Ignored");
+		ctor._match_cached_ckm               = get_menu_widget ( "/Filter/Match Articles which are Cached");
+		ctor._match_noncached_ckm            = get_menu_widget ( "/Filter/Match Articles which are not Cached");
+		ctor._match_only_cached_tb           = _toolbar_buttons[ACTION_SHOW_ONLY_CACHED];
 
-		ctor._match_cached_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Articles which are Cached");
-		ctor._match_noncached_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Articles which are not Cached");
-		ctor._match_only_cached_tb = _toolbar_buttons[ACTION_SHOW_ONLY_CACHED];
+		ctor._match_mine_ckm                 = get_menu_widget ( "/Filter/Match Articles Written by Me");
+		ctor._match_others_ckm               = get_menu_widget ( "/Filter/Match Articles Written by Other People");
+		ctor._show_only_my_threads_tb        = _toolbar_buttons[ACTION_SHOW_ONLY_MY_THREADS];
 
-		ctor._match_mine_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Articles Written by Me");
-		ctor._match_others_ckm = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Match Articles Written by Other People");
+	        ctor._match_score_watched_ckm        = get_menu_widget ("/Filter/Match Articles with a Score of 9999 or higher");
+	        ctor._match_score_high_ckm           = get_menu_widget ("/Filter/Match Articles with a Score between 5000 and 9998");
+	        ctor._match_score_medium_ckm         = get_menu_widget ("/Filter/Match Articles with a Score between 1 and 4999");
+	        ctor._match_score_zero_ckm           = get_menu_widget ("/Filter/Match Articles with a Score of 0");
+	        ctor._match_score_low_ckm            = get_menu_widget ("/Filter/Match Articles with a Score between -9998 and -1");
+	        ctor._match_score_ignored_ckm        = get_menu_widget ("/Filter/Match Articles with a Score of -9999 or lower");
+		ctor._match_only_watched_tb          = _toolbar_buttons[ACTION_SHOW_ONLY_WATCHED];
 
-		ctor._show_articles_rmi = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Show Matching Articles");
-		ctor._show_articles_and_replies_rmi = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Show Matching Articles and Replies");
-		ctor._show_articles_and_refs_rmi = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Show Matching Articles and References");
-		ctor._show_threads_rmi = gtk_item_factory_get_item (_main_menu_factory, "/Filter/Show Threads with Matching Articles");
-
-		ctor._show_only_watched_threads_tb = _toolbar_buttons[ACTION_SHOW_ONLY_WATCHED_THREADS];
-		ctor._show_only_my_threads_tb = _toolbar_buttons[ACTION_SHOW_ONLY_MY_THREADS];
+		ctor._show_articles_rmi              = get_menu_widget ("/Filter/Show Matching Articles");
+		ctor._show_articles_and_replies_rmi  = get_menu_widget ("/Filter/Show Matching Articles and Replies");
+		ctor._show_articles_and_refs_rmi     = get_menu_widget ("/Filter/Show Matching Articles and References");
+		ctor._show_threads_rmi               = get_menu_widget ("/Filter/Show Threads with Matching Articles");
 
 		filter_mediator_init (&ctor);
+#undef get_menu_widget
 	}
 
 	debug_exit ("gui_construct");
@@ -1980,16 +1590,16 @@ pan_manual_url (void)
 static void
 send_feedback_cb (GtkWidget * w)
 {
-	char * from;
+	char from[512];
 	GMimeMessage * message;
 	Identity * id;
 	debug_enter ("send_feedback_cb");
 
 	/* who is sending this? */
-	from = NULL;
+	*from = '\0';
        	id = identity_manager_get_default (ID_MAIL_DEFAULT);
 	if (id != NULL)
-		from = g_strdup_printf ("\"%s\" <%s>", id->author_real, id->author_addr);
+		article_format_author_str (id->author_addr, id->author_real, from, sizeof(from));
 
 	message = g_mime_message_new (TRUE);
 	g_mime_message_set_subject (message, "Pan " VERSION " Feedback");
@@ -1998,7 +1608,6 @@ send_feedback_cb (GtkWidget * w)
 	message_edit_window (message);
 
 	/* cleanup */
-	g_free (from);
 	debug_exit ("send_feedback_cb");
 }
 
@@ -2053,7 +1662,7 @@ use_fixed_font_cb (gpointer user_data, int action, GtkWidget * w)
 {
 	GtkCheckMenuItem * item = GTK_CHECK_MENU_ITEM(w);
 	g_return_if_fail (item!=NULL);
-	text_use_fixed_font = item->active;
+	body_pane_monospace_font_enabled = item->active;
 	text_set_font ();
 }
 
@@ -2120,23 +1729,6 @@ zoom_cb (gpointer user_data, int action, GtkWidget * w)
 	debug_exit ("zoom_cb");
 }
 
-static void
-delete_last_task_cb (gpointer user_data, int action, GtkWidget * w)
-{
-	GPtrArray * tasks;
-
-	/* remove the last task, if one exists */
-	tasks = queue_get_tasks ();
-	if (tasks->len) {
-		Task * task = TASK(g_ptr_array_index (tasks, tasks->len-1));
-		queue_remove_tasks (g_slist_append(NULL,task));
-	}
-
-	/* cleanup */
-	pan_g_ptr_array_foreach (tasks, (GFunc)pan_object_unref, NULL);
-	g_ptr_array_free (tasks, TRUE);
-}
-
 static void
 work_online_cb (gpointer user_data, int action, GtkWidget * w)
 {
@@ -2156,11 +1748,11 @@ widget_destroyed_cb (GtkObject * o, gpointer data)
 	*((GtkWidget**)data) = NULL;
 }
 
-static GtkWidget * identity_dialog = NULL;
-
 static void
 identity_cb (void)
 {
+	static GtkWidget * identity_dialog = NULL;
+
 	if (identity_dialog == NULL)
 	{
 		identity_dialog = identity_dialog_new (GTK_WINDOW(Pan.window));
@@ -2172,11 +1764,11 @@ identity_cb (void)
 	gtk_widget_show_all (identity_dialog);
 }
 
-static GtkWidget * server_dialog = NULL;
-
 static void
 server_custom_cb (void)
 {
+	static GtkWidget * server_dialog = NULL;
+
 	if (server_dialog == NULL)
 	{
 		server_dialog = server_dialog_new (GTK_WINDOW(Pan.window));
@@ -2188,11 +1780,11 @@ server_custom_cb (void)
 	gtk_widget_show_all (server_dialog);
 }
 
-static GtkWidget * filter_dialog = NULL;
-
 static void
 filter_custom_cb (void)
 {
+	static GtkWidget * filter_dialog = NULL;
+
 	if (filter_dialog == NULL)
 	{
 		filter_dialog = filter_dialog_new (GTK_WINDOW(Pan.window));
@@ -2204,12 +1796,11 @@ filter_custom_cb (void)
 	gtk_widget_show_all (filter_dialog);
 }
 
-
-static GtkWidget * rule_dialog = NULL;
-
 static void
 rules_cb (void)
 {
+	static GtkWidget * rule_dialog = NULL;
+
 	if (rule_dialog == NULL)
 	{
 		rule_dialog = rule_dialog_new (GTK_WINDOW(Pan.window));
@@ -2231,8 +1822,8 @@ rules_cb (void)
 static GtkItemFactoryEntry main_menu_entries[MAIN_MENU_ENTRIES_QTY] =
 {
 	/*   */ {N_("/_File"), NULL, NULL, 0, "<Branch>"},
-	/* s */ {N_("/_File/_Save Article As..."), "<control>S", menu_cb, ACTION_SAVE_AS, "<StockItem>", GTK_STOCK_SAVE_AS},
-	/* t */ {N_("/_File/Save Article A_ttachments"), "<shift>S", menu_cb, ACTION_SAVE, "<StockItem>", GTK_STOCK_SAVE},
+	/* t */ {N_("/_File/Save A_ttachments"), "<shift>S", menu_cb, ACTION_SAVE, "<StockItem>", GTK_STOCK_SAVE},
+	/* s */ {N_("/_File/_Save Attachments As..."), "<control>S", menu_cb, ACTION_SAVE_AS, "<StockItem>", GTK_STOCK_SAVE_AS},
 	/*   */ {N_("/_File/---"), NULL, NULL, 0, "<Separator>"},
 	/* l */ {N_("/_File/Work On_line"), "L", work_online_cb, 0, "<ToggleItem>"},
 	/* c */ {N_("/_File/_Cancel Last Task"), "<control>Delete", menu_cb, ACTION_NET_CANCEL_LAST_TASK, "<ImageItem>", icon_cancel},
@@ -2243,7 +1834,11 @@ static GtkItemFactoryEntry main_menu_entries[MAIN_MENU_ENTRIES_QTY] =
 	/* f */ {N_("/_File/Delete _Folder..."), NULL, group_action_selected_destroy, 0, "<StockItem>", GTK_STOCK_DELETE},
 	/* e */ {N_("/_File/_Edit Article in Folder"), NULL, article_action_edit_selected, 0, NULL},
 	/*   */ {N_("/_File/---"), NULL, NULL, 0, "<Separator>"},
+#ifdef G_OS_WIN32
+	/* x */ {N_("/_File/E_xit"), "<alt>F4", exit_cb, 0, "<StockItem>", GTK_STOCK_QUIT},
+#else
 	/* q */ {N_("/_File/_Quit"), NULL, exit_cb, 0, "<StockItem>", GTK_STOCK_QUIT},
+#endif
 
 	/*   */ {N_("/_Edit"), NULL, NULL, 0, "<Branch>"},
 	/*   */ {N_("/_Edit/Select all"), "<control>A", gui_select_all, 0, NULL},
@@ -2306,15 +1901,19 @@ static GtkItemFactoryEntry main_menu_entries[MAIN_MENU_ENTRIES_QTY] =
 	/* m */ {N_("/Filte_r/Match Articles Written by _Me"), NULL, NULL, 0, "<ToggleItem>"},
 	/* o */ {N_("/Filte_r/Match Articles Written by _Other People"), NULL, NULL, 0, "<ToggleItem>"},
 	/*   */ {N_("/Filte_r/---"), NULL, NULL, 0, "<Separator>"},
-	/* w */ {N_("/Filte_r/Match _Watched Threads"), NULL, NULL, 0, "<ToggleItem>"},
-	/* i */ {N_("/Filte_r/Match _Ignored Threads"), NULL, NULL, 0, "<ToggleItem>"},
-	/* e */ {N_("/Filte_r/Match Threads N_either Watched nor Ignored"), NULL, NULL, 0, "<ToggleItem>"},
+	/*   */ {N_("/Filte_r/Match Articles with a Score of 9999 or higher"), NULL, NULL, 0, "<ToggleItem>"},
+	/*   */ {N_("/Filte_r/Match Articles with a Score between 5000 and 9998"), NULL, NULL, 0, "<ToggleItem>"},
+	/*   */ {N_("/Filte_r/Match Articles with a Score between 1 and 4999"), NULL, NULL, 0, "<ToggleItem>"},
+	/*   */ {N_("/Filte_r/Match Articles with a Score of 0"), NULL, NULL, 0, "<ToggleItem>"},
+	/*   */ {N_("/Filte_r/Match Articles with a Score between -9998 and -1"), NULL, NULL, 0, "<ToggleItem>"},
+	/*   */ {N_("/Filte_r/Match Articles with a Score of -9999 or lower"), NULL, NULL, 0, "<ToggleItem>"},
 
 	/*   */ {N_("/_Go"), NULL, NULL, 0, "<Branch>"},
-	/* s */ {N_("/_Go/Read More"), "space", menu_cb, ACTION_READ_MORE, "<ImageItem>", icon_read_more},
-	/* s */ {N_("/_Go/Read Back"), "BackSpace", menu_cb, ACTION_READ_LESS, "<ImageItem>", icon_read_less},
+	/*   */ {N_("/_Go/Read More"), "space", menu_cb, ACTION_READ_MORE, "<ImageItem>", icon_read_more},
+	/*   */ {N_("/_Go/Read Back"), "BackSpace", menu_cb, ACTION_READ_LESS, "<ImageItem>", icon_read_less},
 	/*   */ {N_("/_Go/---"), NULL, NULL, 0, "<Separator>"},
 	/* n */ {N_("/_Go/_Next Unread Article"), "N", menu_cb, ACTION_READ_NEXT_UNREAD_ARTICLE, "<ImageItem>", icon_read_new_article},
+	/* s */ {N_("/_Go/Next High-_Score Article"), "<shift>N", menu_cb, ACTION_READ_NEXT_SCORE_ARTICLE, 0, NULL},
 	/* x */ {N_("/_Go/Ne_xt Article"), "<control>N", articlelist_read_next, 0, NULL},
 	/* t */ {N_("/_Go/Next Unread _Thread"), "T", menu_cb, ACTION_READ_NEXT_UNREAD_THREAD, "<ImageItem>", icon_read_new_thread},
 	/* d */ {N_("/_Go/Next Threa_d"), "<shift>T", articlelist_read_next_thread, 0, NULL},
@@ -2359,14 +1958,18 @@ static GtkItemFactoryEntry main_menu_entries[MAIN_MENU_ENTRIES_QTY] =
 	/*   */ {N_("/_Articles/---"), NULL, NULL, 0, "<Separator>"},
 	/* l */ {N_("/_Articles/Down_load"), NULL, articlelist_selected_download_nolock, 0, NULL},
 	/*   */ {N_("/_Articles/---"), NULL, NULL, 0, "<Separator>"},
-	/* w */ {N_("/_Articles/_Watch Thread"), "<control><shift>W", articlelist_selected_thread_watch_nolock, 0, "<ImageItem>", icon_watched},
-	/* i */ {N_("/_Articles/_Ignore Thread"), "<control><shift>I", articlelist_selected_thread_ignore_nolock, 0, "<StockItem>", GTK_STOCK_CLEAR},
-	/* z */ {N_("/_Articles/Add to Bo_zo or Spam Filter..."), "<control><shift>Z", articlelist_spamize_nolock, 0, "<ImageItem>", icon_bozo},
+	/* w */ {N_("/_Articles/_Watch Thread"), "<shift>W", menu_cb, ACTION_SCORE_WATCH_THREAD, "<ImageItem>", icon_watched},
+	/* i */ {N_("/_Articles/_Ignore Thread"), "<shift>I", menu_cb, ACTION_SCORE_IGNORE_THREAD, "<StockItem>", GTK_STOCK_CLEAR},
+	/* p    note to translators: "plonk" is a decade-old Usenet word defined as
+	 *      the sound an articles' author makes when he falls to the bottom of the users' killfile.
+	 *      so "plonk author"means "don't show this author's articles anymore." */
+	        {N_("/_Articles/_Plonk Author"), "<shift>P", menu_cb, ACTION_SCORE_PLONK, NULL},
+	/* v */ {N_("/_Articles/_View Article's Scores"), "<control><shift>C", menu_cb, ACTION_SCORE_VIEW, "<ImageItem>", icon_score},
 	/*   */ {N_("/_Articles/---"), NULL, NULL, 0, "<Separator>"},
 	/* c */ {N_("/_Articles/_Cancel..."), NULL, articlelist_selected_cancel, 0, "<StockItem>", GTK_STOCK_CANCEL},
-	/* s */ {N_("/_Articles/_Supersede..."), NULL, articlelist_selected_supersede, 0, "<StockItem>", GTK_STOCK_CONVERT},
+	/*   */ {N_("/_Articles/Supersed_e..."), NULL, articlelist_selected_supersede, 0, "<StockItem>", GTK_STOCK_CONVERT},
 	/*   */ {N_("/_Articles/---"), NULL, NULL, 0, "<Separator>"},
-	/* d */ {N_("/_Articles/_Delete"), "Delete", article_action_delete_selected_articles, 0, "<StockItem>", GTK_STOCK_DELETE},
+	/* d */ {N_("/_Articles/_Delete"), "Delete", menu_cb, ACTION_ARTICLE_DELETE, "<StockItem>", GTK_STOCK_DELETE},
 
 	/*   */ {N_("/_Post"), NULL, NULL, 0, "<Branch>"},
 	/* p */ {N_("/_Post/_Post to Newsgroup"), "P", menu_cb, ACTION_COMPOSE_NEW, "<ImageItem>", icon_compose_post},
@@ -2383,6 +1986,8 @@ static GtkItemFactoryEntry main_menu_entries[MAIN_MENU_ENTRIES_QTY] =
 	/*   */ {N_("/_Tools/---"), NULL, NULL, 0, "<Separator>"},
 	/* f */ {N_("/_Tools/Custom _Filters..."), NULL, filter_custom_cb, 0, NULL},
 	/* r */ {N_("/_Tools/_Rules..."), NULL, rules_cb, 0, NULL},
+	/* c */ {N_("/_Tools/_Create Score..."), "S", menu_cb, ACTION_ADD_TO_SCOREFILE, "<ImageItem>", icon_score},
+	/* e */ {N_("/_Tools/_Edit Scorefile..."), NULL, menu_cb, ACTION_EDIT_SCOREFILE, "<ImageItem>", icon_score},
 	/*   */ {N_("/_Tools/---"), NULL, NULL, 0, "<Separator>"},
 	/* t */ {N_("/_Tools/_Task Manager..."), NULL, task_manager_spawn, 0, NULL},
 	/*   */ {N_("/_Tools/---"), NULL, NULL, 0, "<Separator>"},
