@@ -22,11 +22,6 @@
 
 #include <gtk/gtkwidget.h>
 
-#include <pan/base/article.h>
-#include <pan/base/group.h>
-#include <pan/base/status-item.h>
-#include <pan/filters/filter.h>
-
 extern GtkWidget * groups_vbox;
 extern GtkWidget * articlelist_ctree;
 extern GtkWidget * text_box;
@@ -42,33 +37,15 @@ enum {
 	BODY_PANE
 };
 
-void pan_widget_set_font (GtkWidget* w, const char* font_name);
+void gui_construct (const char *geometry);
 
 void gui_layout_clear_workarea (void);
-void gui_layout_set_mode (int type);
-void gui_layout_refresh (void);
-
-
-void gui_construct (const gchar *geometry);
-
-void gui_page_set (int page);
-
-void gui_restore_column_widths    (GtkWidget * clist, const gchar * key);
-void gui_restore_column_widths_tree_view    (GtkWidget * tree_view, const gchar * key);
-
-void gui_save_column_widths       (GtkWidget * clist, const gchar * key);
-void gui_save_column_widths_tree_view       (GtkWidget * tree_view, const gchar * key);
-
-gboolean gui_restore_window_size      (GtkWidget * window, const gchar * key);
-
-void gui_save_window_size         (GtkWidget * window, const gchar * key);
+void gui_layout_refresh        (void);
+void gui_layout_set_mode       (int type);
+void gui_page_set              (int page);
 
 GtkWidget* gui_add_network_button_to_toolbar  (GtkWidget * toolbar);
 
-void gui_queue_add (GSourceFunc func, gpointer user_data);
-
 GdkColormap* cmap;
 
-void gui_edit_current_filter (void);
-
 #endif /* __GUI_H__ */
