@@ -36,6 +36,7 @@
 #include <pan/identities/identity-edit-ui.h>
 
 #include <pan/pan-file-entry.h>
+#include <pan/pan-charset-picker.h>
 
 #include <pan/util.h>
 
@@ -67,6 +68,7 @@ typedef struct
 	GtkWidget * signature_entry;
 	GtkWidget * attribution_entry;
 	GtkWidget * msg_id_fqdn_entry;
+	GtkWidget * posting_charset_entry;
 
 	GtkWidget * custom_headers_text;
 
@@ -276,6 +278,18 @@ create_identity_tab (IdentityEditDialog *d)
 		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
 		++row;
 
+		l = gtk_label_new_with_mnemonic (_("C_haracter Set:"));
+		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
+		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
+
+		w = d->posting_charset_entry = pan_charset_picker_new ("");
+		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w, 
+			_("The default charset set for articles "
+			  "posted with this profile"), "");
+		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
+		gtk_table_attach_defaults (GTK_TABLE(t), w, 3, 4, row, row+1);
+		++row;
+
 	return t;
 }
 
@@ -342,6 +356,12 @@ populate_identity_tab (IdentityEditDialog * d,
 	pan_file_entry_set (d->signature_entry, 
 		b && is_nonempty_string (id->signature) ?
 		id->signature : "");
+
+	/* posting charset */
+	w = d->posting_charset_entry;
+	pan_charset_picker_set_charset (w,
+		b && is_nonempty_string (id->posting_charset) ? 
+		id->posting_charset : get_charset_from_locale ());
 }
 
 static void
@@ -413,6 +433,11 @@ identity_from_identity_tab (IdentityEditDialog * d,
 	cpch = pan_file_entry_get (d->signature_entry);
 	replace_gstr (&id->signature, g_strdup(cpch));
 	g_strstrip (id->signature);
+
+	/* posting charset */
+	cpch = pan_charset_picker_get_charset (d->posting_charset_entry);
+	replace_gstr (&id->posting_charset, g_strdup (cpch));
+	g_strstrip (id->posting_charset);
 }
 
 /*****
