@@ -112,7 +112,7 @@ construct_identity_from_prefs (void)
 	return id;
 }
 
-static void
+void
 identity_manager_save (void)
 {
 	if (_identities)
@@ -175,7 +175,7 @@ identity_manager_count (void)
 }
 
 Identity *
-identity_manager_get_identity (const gchar * name)
+identity_manager_get_identity_nocopy (const gchar * name)
 {
 	Identity * id = NULL;
 	guint      i;
@@ -186,19 +186,30 @@ identity_manager_get_identity (const gchar * name)
 
         for (i=0; i<_identities->len; i++)
         {
-                Identity *test = g_ptr_array_index (_identities, i);
+                id = g_ptr_array_index (_identities, i);
 
-                if (!pan_strcmp(test->name, name))
-		{
-                        id = identity_dup (test);
+                if (!pan_strcmp(id->name, name))
 			break;
-		}
         }
 
 	debug_exit ("identity_manager_get_identity");
 	return id;
 }
 
+Identity *
+identity_manager_get_identity (const gchar * name)
+{
+	Identity *id = NULL;
+	debug_enter ("identity_manager_get_identity");
+
+	id = identity_manager_get_identity_nocopy (name);
+	if (id != NULL)
+		id = identity_dup (id);
+
+	debug_exit ("identity_manager_get_identity");
+	return id;
+}
+
 Identity *
 identity_manager_get_identity_by_author (const gchar * author_real,
                                          const gchar * author_addr)
