@@ -28,15 +28,25 @@ enum {
 	ID_MAIL_DEFAULT
 };
 
-void identity_manager_shutdown_module (void);
+void       identity_manager_shutdown_module (void);
+
+void       identity_manager_save (void);
 
 guint      identity_manager_count (void);
 
+
 Identity * identity_manager_get_identity (const gchar * name);
 
 Identity * identity_manager_get_identity_by_author (const gchar * author_real,
                                                     const gchar * author_addr);
 
+/*
+ * This is ugly, I know. Looking at this now, having the 
+ * identity_manager make copies each identity was not the 
+ * best approach. Should be replaced with a refcount mechanism
+ */
+Identity * identity_manager_get_identity_nocopy (const gchar * name);
+
 Identity * identity_manager_get_default (int type);
 
 void       identity_manager_set_default (const gchar * id_name, int type);
