@@ -27,6 +27,7 @@
 
 #include <glib.h>
 #include <gtk/gtk.h>
+#include <gmime/gmime.h>
 
 #ifdef HAVE_LIBGTKSPELL
 #include <gtkspell/gtkspell.h>
@@ -43,8 +44,8 @@
 #include <pan/base/serverlist.h>
 #include <pan/base/util-file.h>
 #include <pan/base/util-mime.h>
-#include <pan/base/util-wrap.h>
 #include <pan/base/pan-callback.h>
+#include <pan/base/text-massager.h>
 
 #include <pan/identities/identity-manager.h>
 
@@ -52,7 +53,6 @@
 #include <pan/articlelist.h>
 #include <pan/globals.h>
 #include <pan/grouplist.h>
-#include <pan/gui.h>
 #include <pan/gui-headers.h>
 #include <pan/message-check-ui.h>
 #include <pan/message-window.h>
@@ -113,6 +113,7 @@ typedef struct
 	GtkWidget * encoding_yenc_tb;
 
 	GMimeMessage * message;
+	TextMassager * text_massager;
 	ComposeType type;
 	Server * server;
 
@@ -167,6 +168,7 @@ window_destroy_cb (GtkWidget * window, Compose * compose)
 
 	if (compose->message != NULL)
 		g_object_unref (compose->message);
+	text_massager_free (compose->text_massager);
 	g_free (compose->identity_name);
 	g_free (compose->last_attribution_text);
 	g_free (compose);
@@ -192,6 +194,8 @@ text_inserted_cb (GtkTextBuffer   * text_buffer,
 		  int               text_len,
 		  gpointer          user_data)
 {
+	Compose * compose = (Compose*) user_data;
+	const int wrap_column = text_massager_get_wrap_column (compose->text_massager);
 	static gboolean dampen_feedback = FALSE;
 	debug_enter ("text_inserted_cb");
 
@@ -260,11 +264,11 @@ text_inserted_cb (GtkTextBuffer   * text_buffer,
 }
 
 static void
-experimental_wrap_handler (GtkTextBuffer * buffer, GtkToggleButton * tb)
+experimental_wrap_handler (Compose * compose, GtkTextBuffer * buffer, GtkToggleButton * tb)
 {
 	pan_lock ();
 	gtk_toggle_button_set_active (tb, experimental_toggle_active);
-	g_signal_connect (buffer, "insert-text", G_CALLBACK(text_inserted_cb), NULL);
+	g_signal_connect (buffer, "insert-text", G_CALLBACK(text_inserted_cb), compose);
 	g_signal_connect (tb, "toggled", G_CALLBACK(wrap_tb_toggled_cb), NULL);
 	pan_unlock ();
 }
@@ -510,7 +514,7 @@ rot13_cb (gpointer user_data, int action, GtkWidget * w)
 	if (gtk_text_buffer_get_selection_bounds (buffer, &start, &end))
 	{
 		char * str = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-		pan_text_rot13_inplace (str);
+		text_massager_rot13_inplace (compose->text_massager, str);
 		gtk_text_buffer_delete (buffer, &start, &end);
 		gtk_text_buffer_insert (buffer, &start, str, strlen(str));
 		g_free (str);
@@ -526,7 +530,7 @@ static void
 wrap_cb (gpointer user_data, int action, GtkWidget * w)
 {
 	Compose * compose = (Compose*) user_data;
-	GtkTextBuffer * buffer = compose->body_buffer;
+	GtkTextBuffer * buffer;
 	char * body;
 	char * new_body;
 	gboolean b;
@@ -535,14 +539,15 @@ wrap_cb (gpointer user_data, int action, GtkWidget * w)
 	debug_enter ("wrap_cb");
 
 	/* get the current body */
+	buffer = compose->body_buffer;
 	gtk_text_buffer_get_bounds (buffer, &start, &end);
 	body = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-	new_body = pan_text_fill (body, wrap_column);
+	new_body = text_massager_fill (compose->text_massager, body);
 
 	/* turn off our own wrapping while we fill the body pane */
 	b = experimental_toggle_active;
 	experimental_toggle_active = FALSE;
-	update_body_pane (compose->body_buffer, new_body, FALSE);
+	update_body_pane (compose->text_massager, compose->body_buffer, new_body, FALSE);
 	experimental_toggle_active = b;
 
 	/* cleanup */
@@ -914,27 +919,6 @@ attachment_page (Compose *mw)
 ****
 ***/
 
-static void
-get_charset_partfunc(GMimeObject * obj, gpointer charset_gpointer)
-{
-	GMimePart * part;
-	const GMimeContentType * type;
-	char ** charset;
-
-	if (!GMIME_IS_PART (obj))
-		return;
-
-	part = GMIME_PART (obj);
-	type = g_mime_object_get_content_type (GMIME_OBJECT (part));
-	charset = (char **) charset_gpointer;
-
-	if (g_mime_content_type_is_type (type, "text", "*")) {
-		*charset = (char *) g_mime_object_get_content_type_parameter (GMIME_OBJECT (part), "charset");
-
-		/*odebug1 ("found charset in article: %s", *charset);*/
-	}
-}
-
 static const char *
 determine_charset_for_message (const Compose *mw)
 {
@@ -942,10 +926,29 @@ determine_charset_for_message (const Compose *mw)
   
 	/* does the article specify a charset ? */
 	if (mw && mw->message)
-		g_mime_message_foreach_part (mw->message, get_charset_partfunc, &charset);
+		charset = pan_g_mime_message_get_charset (mw->message);
 
-	/* fallback: use the group's charset */
-	if (charset == NULL || !pan_charset_picker_has (charset)) {
+	/* is it one we support ? */
+	if (!pan_charset_picker_has (charset)) {
+		charset = NULL;
+	}
+
+	/* otherwise, use the profile's charset */
+	if (charset == NULL) {
+		Identity * id = is_nonempty_string (mw->identity_name)
+			? identity_manager_get_identity (mw->identity_name)
+			: NULL;
+
+		if (id != NULL)
+		{
+			charset = id->posting_charset;
+			pan_object_unref (PAN_OBJECT(id));
+		}
+	}
+
+	/* fallback #1: use group's charset */
+	/* fallback #2: use user's locale to guess the charset */
+	if (charset == NULL) {
 		const Group * group = articlelist_get_group ();
 
 		charset = group ? group_get_default_charset (group) :
@@ -1043,7 +1046,7 @@ extra_headers_page (Compose * mw)
 	gtk_table_attach (GTK_TABLE(table), w,
 	                  0, 1, row, row+1,
 	                  GTK_FILL, GTK_FILL, GUI_PAD, 0);
-	mw->charset_om = pan_charset_picker_new (determine_charset_for_message(mw));
+	mw->charset_om = pan_charset_picker_new (determine_charset_for_message (mw));
 	gtk_table_attach (GTK_TABLE(table), mw->charset_om,
 	                  1, 2, row, row+1,
 	                  GTK_FILL|GTK_EXPAND, GTK_FILL, GUI_PAD, 0); 
@@ -1228,6 +1231,37 @@ set_gtk_entry_if_blank (GtkEntry * entry, const char * string)
  * Generates an attribution line
  */
 
+static char * 
+get_author_utf8 (GMimeMessage * message)
+{
+	const char * charset = NULL;
+	const char * author = NULL;
+	char * author_utf8 = NULL;
+
+	if (message == NULL)
+		return g_strdup ("");
+
+	author = g_mime_message_get_sender (message);
+
+	charset = pan_g_mime_message_get_charset (message);
+
+	if (charset == NULL)
+	{
+		Group * group = articlelist_get_group ();
+		charset = group ? group_get_default_charset (group) : get_charset_from_locale ();
+	}
+
+	author_utf8 = pan_header_to_utf8 (author, -1, charset);
+
+	if (author_utf8 == NULL)
+	{
+		author_utf8 = g_strdup (author);
+		log_add_va (LOG_ERROR, _("Could not convert \"%s\" to UTF-8. Article may not display correctly."), author);
+	}
+
+	return author_utf8;
+}
+
 static char*
 message_gen_attribution_string (GMimeMessage    * message,
                                 const char      * attribution_line)
@@ -1243,43 +1277,40 @@ message_gen_attribution_string (GMimeMessage    * message,
 		pan_g_string_replace (gstr, "%i", cpch);
 	}
 
-	if (strstr (attribution_line, "%d")) { /* substitutions: date */ /* this wierd format is to not screw the string freeze */
-		if (!message) pan_g_string_replace (gstr, "%d", "");
-		else {  char * pch = g_mime_message_get_date_string (message);
+	/* substitutions: date */ 
+	if (strstr (attribution_line, "%d"))
+	{
+		if (!message) 
+			pan_g_string_replace (gstr, "%d", "");
+		else
+		{
+			char * pch = g_mime_message_get_date_string (message);
 			pan_g_string_replace (gstr, "%d", pch);
-			g_free (pch); }
+			g_free (pch);
+		}
 	}
 
-	/* substitutions: author */
+	/* substitutions: full author */
 	if (strstr (attribution_line, "%a")) {
-		const char * cpch = g_mime_message_get_header (message, HEADER_FROM);
-		char * author_utf8 = NULL;
-		const Group * group = articlelist_get_group ();
-		const char * charset = group ? group_get_default_charset (group) : get_charset_from_locale ();
-
-		author_utf8 = pan_header_to_utf8 (cpch, -1, charset);
-		if (author_utf8 != NULL) {
-			pan_g_string_replace (gstr, "%a", author_utf8);
-			g_free (author_utf8);
-		}
-		else {
-			pan_g_string_replace (gstr, "%a", cpch);
-			log_add_va (LOG_ERROR, _("Could not convert \"%s\" to UTF-8. Article may not display correctly."), cpch);
-		}
+		char * author_utf8 = get_author_utf8 (message);
+		pan_g_string_replace (gstr, "%a", author_utf8);
+		g_free (author_utf8);
 	}
 
-	/* attribution */
+	/* substitutions: author name */
 	if (strstr (attribution_line, "%n") != NULL) {
 		char buf[512] = { '\0' };
-		const char * sender = g_mime_message_get_sender (message);
-		InternetAddressList * list = internet_address_parse_string (sender);
-		if (list != NULL) {
+		char * author_utf8 = get_author_utf8 (message);
+		InternetAddressList * list = internet_address_parse_string (author_utf8);
+		if (list != NULL)
+		{
 			InternetAddress * ia = list->address;
 			if (ia!=NULL && ia->type==INTERNET_ADDRESS_NAME)
 				article_format_short_author_str (ia->value.addr, ia->name, buf, sizeof(buf));
 			internet_address_list_destroy (list);
 		}
 		pan_g_string_replace (gstr, "%n", buf);
+		g_free (author_utf8);
 	}
 
 	return g_string_free (gstr, FALSE);
@@ -1309,7 +1340,7 @@ remove_attribution_line (Compose * compose)
 		index = pan_strstr (text, compose->last_attribution_text);
 		if (index) {
 			index += 2+strlen (compose->last_attribution_text);
-			update_body_pane (compose->body_buffer, index, FALSE);
+			update_body_pane (compose->text_massager, compose->body_buffer, index, FALSE);
 		}
 
 		replace_gstr (&compose->last_attribution_text, NULL);
@@ -1344,17 +1375,13 @@ prepend_attribution_line (Compose      * compose,
 		pch = message_gen_attribution_string (compose->message, attribution_line);
 
 		if (is_nonempty_string (pch))
-		{
-			char * wrapped = pan_text_fill (pch, wrap_column);
-			g_free (pch);
-			pch = wrapped;
-		}
+			replace_gstr (&pch, text_massager_fill (compose->text_massager, pch));
 
 		if (is_nonempty_string (pch))
 		{
 			char * out = g_strdup_printf ("%s\n\n%s", pch, text);
 
-			update_body_pane (compose->body_buffer, out, FALSE);
+			update_body_pane (compose->text_massager, compose->body_buffer, out, FALSE);
 
 			replace_gstr (&compose->last_attribution_text, pch);
 
@@ -1379,71 +1406,68 @@ load_signature (const char * sig_file)
 	char ** argv = NULL;
 
 	/* process file name */
-	pch = g_strdup (sig_file);
-	g_strstrip (pch);
-	if (is_nonempty_string (pch))
+	pch = pan_file_normalize (sig_file, NULL);
+	g_return_val_if_fail (pch!=NULL, NULL);
+
+	/* bear in mind that we need to give care to the order of the steps we use
+	 * to test & poke the sig file.  Older versions of Pan parsed to the shell
+	 * and used argv[0] as the filename, which fails on Windows if there are
+	 * spaces in the filename.  Some test cases:
+	 * 1. "fortune -s -o" (to keep the sig length polite and content rude)
+	 * 2. "c:\program files\pan\signature.txt" */
+
+	if (g_file_test (pch, G_FILE_TEST_EXISTS))
 	{
-		/* expand the home directory character */
-		if (*pch=='~') 
-			replace_gstr (&pch, g_build_filename (g_get_home_dir(), pch+1, NULL));
-
-		/* if not an absolute path, assume in home directory */
-		if (!g_path_is_absolute (pch))
-			replace_gstr (&pch, g_build_filename (g_get_home_dir(), pch, NULL));
-
-		if (!g_shell_parse_argv (pch, &argc, &argv, &err))
+		if (g_file_test (pch, G_FILE_TEST_IS_EXECUTABLE))
 		{
-			log_add_va (LOG_ERROR, _("Couldn't parse signature command \"%s\""), pch);
-			log_add (LOG_ERROR, err->message);
+			/* file is executable */
+			argc = 1;
+			argv = g_new (char*, 2);
+			argv[0] = g_strdup (pch);
+			argv[1] = NULL; /* this is for g_strfreev() */
+		}
+	}
+	else if (!g_shell_parse_argv (pch, &argc, &argv, &err))
+	{
+		log_add_va (LOG_ERROR, _("Couldn't parse signature command \"%s\""), pch);
+		log_add (LOG_ERROR, err->message);
+		g_error_free (err);
+		err = NULL;
+	}
+
+	/* try to execute the file... */
+	if (argc>0 && argv!=NULL && argv[0]!=NULL
+	    && g_file_test (argv[0], G_FILE_TEST_IS_EXECUTABLE))
+	{
+		char * spawn_stdout = NULL;
+		char * spawn_stderr = NULL;
+		int exit_status = 0;
+
+		if (g_spawn_sync (NULL, argv, NULL, 0, NULL, NULL, &spawn_stdout, &spawn_stderr, &exit_status, NULL))
+			sig = g_strdup (spawn_stdout);
+		if (is_nonempty_string (spawn_stderr))
+			log_add_va (LOG_ERROR, "%s", spawn_stderr);
+
+		g_free (spawn_stderr);
+		g_free (spawn_stdout);
+	}
+
+	/* if the file wasn't executable, or we couldn't execute it, load the file. */
+	if (sig == NULL)
+	{
+		if (!g_file_get_contents (pch, &sig, NULL, &err))
+		{
+			log_add_va (LOG_ERROR, _("Couldn't read signature file \"%s\": %s"), pch, err->message);
 			g_error_free (err);
 			err = NULL;
 		}
-		else if (!argv || !argv[0])
-		{
-			log_add_va (LOG_ERROR, _("Couldn't parse signature command \"%s\""), pch);
-		}
 	}
 
-	/* get signature */
-	if (argv && is_nonempty_string(argv[0]))
-	{
-		if (g_file_test (argv[0], G_FILE_TEST_IS_EXECUTABLE))
-		{
-			int exit_status = 0;
-			char * standard_output = NULL;
-			char * standard_error = NULL;
-
-			if (!g_spawn_sync (NULL, argv, NULL, 0, NULL, NULL, &standard_output, &standard_error, &exit_status, &err))
-			{
-				log_add_va (LOG_ERROR, _("Couldn't run signature program \"%s\": %s"), pch, err->message);
-				g_error_free (err);
-				err = NULL;
-			}
-			else
-			{
-				sig = g_strdup (is_nonempty_string (standard_output) ? standard_output : standard_error);
-				replace_gstr (&standard_output, NULL);
-				replace_gstr (&standard_error, NULL);
-			}
-		}
-
-		if (!sig)
-		{
-			GError * err = NULL;
-			if (!g_file_get_contents (argv[0], &sig, NULL, &err))
-			{
-				log_add_va (LOG_ERROR, _("Couldn't read signature file \"%s\": %s"), pch, err->message);
-				g_error_free (err);
-			}
-
-		}
-	}
-
-	/* Convert signature to UTF-8. Since the signature is a local file, */
-	/* we assume the contents is in the user's locale's charset.        */
-	/* If we can't convert, clear the signature. Otherwise, we'd add an */
-	/* charset-encoded sig (say 'iso-8859-1') to the body (in UTF-8),   */
-	/* which could result in a blank message in the composer window.    */
+	/* Convert signature to UTF-8. Since the signature is a local file,
+	 * we assume the contents is in the user's locale's charset.
+	 * If we can't convert, clear the signature. Otherwise, we'd add an
+	 * charset-encoded sig (say 'iso-8859-1') to the body (in UTF-8),
+	 * which could result in a blank message in the composer window. */
 	if (sig!=NULL)
 	{
 		const char * sig_utf8 = NULL;
@@ -1468,7 +1492,8 @@ load_signature (const char * sig_file)
 }
 
 static void
-remove_signature (GtkTextBuffer * body_buffer)
+remove_signature (const TextMassager    * text_massager,
+                  GtkTextBuffer         * body_buffer)
 {
 	GtkTextIter start;
 	GtkTextIter end;
@@ -1484,14 +1509,16 @@ remove_signature (GtkTextBuffer * body_buffer)
 
 	if (pan_remove_signature (text))
 	{
-		update_body_pane (body_buffer, text, FALSE);
+		update_body_pane (text_massager, body_buffer, text, FALSE);
 	}
 
 	g_free (text);
 }
 
 static void
-append_signature (GtkTextBuffer * body_buffer, const char *sig_file)
+append_signature (const TextMassager    * text_massager,
+                  GtkTextBuffer         * body_buffer,
+                  const char            * sig_file)
 {
 	char * sig = NULL;
 
@@ -1501,7 +1528,7 @@ append_signature (GtkTextBuffer * body_buffer, const char *sig_file)
 	g_return_if_fail (sig_file!=NULL);
 
 	/* since we're applying a signature, remove any signature anyway */
-	remove_signature (body_buffer);
+	remove_signature (text_massager, body_buffer);
 
         /* load the signature */	
 	sig = load_signature (sig_file);
@@ -1528,7 +1555,7 @@ append_signature (GtkTextBuffer * body_buffer, const char *sig_file)
 		out = g_strdup_printf ("%s%s%s", text, 
 			delimiter_exists ? "" : "\n-- \n", sig);
 
-		update_body_pane (body_buffer, out, FALSE);
+		update_body_pane (text_massager, body_buffer, out, FALSE);
 
 		g_free (out);
 		g_free (sig);
@@ -1657,41 +1684,44 @@ add_custom_headers (GtkWidget * header_view, const Identity * id)
 ***/
 
 static void
-revoke_identity (Compose * mw, const Identity * id)
+revoke_identity (Compose * compose, const Identity * id)
 {
 	/* sanity clause */
-	g_return_if_fail (mw!=NULL);
+	g_return_if_fail (compose!=NULL);
 	g_return_if_fail (id!=NULL);
 
-	clear_gtk_entry_if_same (GTK_ENTRY(mw->organization), id->organization);
+	clear_gtk_entry_if_same (GTK_ENTRY(compose->organization), id->organization);
 
-	clear_gtk_entry_if_same (GTK_ENTRY(mw->reply_to), id->reply_to);
+	clear_gtk_entry_if_same (GTK_ENTRY(compose->reply_to), id->reply_to);
 
-	remove_attribution_line (mw);
+	remove_attribution_line (compose);
 
 	if (is_nonempty_string (id->signature))
-		remove_signature (mw->body_buffer);
+		remove_signature (compose->text_massager, compose->body_buffer);
 
-	remove_custom_headers (mw->custom_headers_text, id);
+	remove_custom_headers (compose->custom_headers_text, id);
 }
 
 static void
-apply_identity (Compose * mw, const Identity * id)
+apply_identity (Compose * compose, const Identity * id)
 {
-	g_return_if_fail (mw!=NULL);
+	g_return_if_fail (compose!=NULL);
 	g_return_if_fail (id!=NULL);
 
-	set_gtk_entry_if_blank (GTK_ENTRY(mw->organization), id->organization);
+	set_gtk_entry_if_blank (GTK_ENTRY(compose->organization), id->organization);
 
-	set_gtk_entry_if_blank (GTK_ENTRY(mw->reply_to), id->reply_to);
+	set_gtk_entry_if_blank (GTK_ENTRY(compose->reply_to), id->reply_to);
 
-	if (is_nonempty_string (id->attribution) && !is_original(mw->type))
-		prepend_attribution_line (mw, id->attribution);
+	if (is_nonempty_string (id->attribution) && !is_original(compose->type))
+		prepend_attribution_line (compose, id->attribution);
 
 	if (is_nonempty_string (id->signature))
-		append_signature (mw->body_buffer, id->signature);
+		append_signature (compose->text_massager, compose->body_buffer, id->signature);
 
-	add_custom_headers (mw->custom_headers_text, id);
+	add_custom_headers (compose->custom_headers_text, id);
+
+	if (is_nonempty_string (id->posting_charset))
+		pan_charset_picker_set_charset (compose->charset_om, id->posting_charset);
 }
 
 
@@ -1956,7 +1986,7 @@ populate_post_info_pane (Compose *mw)
 		/* editor */
 		g_assert (mw->body_view!=NULL);
 		pch = g_mime_message_get_body (m, TRUE, &is_html);
-		update_body_pane (mw->body_buffer, pch, FALSE);
+		update_body_pane (mw->text_massager, mw->body_buffer, pch, FALSE);
 		g_free (pch);
 
 	}
@@ -1972,7 +2002,7 @@ populate_post_info_pane (Compose *mw)
 		g_assert (mw->body_view!=NULL);
 		if (m != NULL) {
 			p = create_reply_body (m);
-			update_body_pane (mw->body_buffer, p, FALSE);
+			update_body_pane (mw->text_massager, mw->body_buffer, p, FALSE);
 			g_free (p);
 		} 
 
@@ -2133,7 +2163,7 @@ create_body_pane_nolock (Compose * mw)
 	gtk_text_buffer_create_tag (buf, "quote_3", "foreground_gdk", NULL, NULL);
 	gtk_text_buffer_create_tag (buf, "signature", "foreground_gdk", NULL, NULL);
 
-	pan_widget_set_font (mw->body_view, message_body_font_fixed);
+	pan_widget_set_font (mw->body_view, body_pane_monospace_font);
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(mw->body_view), GTK_WRAP_NONE);
 	gtk_text_view_set_editable (GTK_TEXT_VIEW(mw->body_view), TRUE);
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
@@ -2171,7 +2201,8 @@ message_post_window_create (Compose * compose)
 	w = create_post_info_pane (compose);
 	populate_post_info_pane (compose);
 
-	experimental_wrap_handler (compose->body_buffer,
+	experimental_wrap_handler (compose,
+	                           compose->body_buffer,
 	                           GTK_TOGGLE_BUTTON(compose->do_wrap_togglebutton));
 
 	pan_lock ();
@@ -2252,7 +2283,8 @@ message_reply_window_create_impl (Compose * compose, const char * title)
 	w = create_post_info_pane (compose);
 	populate_post_info_pane (compose);
 
-	experimental_wrap_handler (compose->body_buffer,
+	experimental_wrap_handler (compose,
+	                           compose->body_buffer,
 	                           GTK_TOGGLE_BUTTON(compose->do_wrap_togglebutton));
 
 	pan_lock ();
@@ -2280,6 +2312,7 @@ message_window_new (GMimeMessage * message, ComposeType type)
 	compose->message = message;
 	compose->type = type;
 	compose->server = serverlist_get_active_server ();
+	compose->text_massager = text_massager_new ();
 
 	switch (type)
 	{
@@ -2448,9 +2481,9 @@ populate_message_from_mw (GMimeMessage* message, const Compose * mw)
 	int i;
 	char ** sd;
 	char * pch;
-	char buf[512];
 	Identity * id;
 	GMimePart * part;
+	InternetAddress *ia;
 
 	/***
 	****  HEADERS
@@ -2463,8 +2496,13 @@ populate_message_from_mw (GMimeMessage* message, const Compose * mw)
 
 	/* author */
 	id = identity_manager_get_identity (mw->identity_name);
-	article_format_author_str (id->author_addr, id->author_real, buf, sizeof(buf));
-	g_mime_message_set_sender (message, buf);
+	ia = internet_address_new ();
+	internet_address_set_name (ia, id->author_real);
+	internet_address_set_addr (ia, id->author_addr);
+	pch = internet_address_to_string (ia, TRUE);
+	g_mime_message_set_sender (message, pch);
+	g_free (pch);
+	internet_address_unref (ia);
 
 	/* organization */
 	pch = message_window_get_organization(mw);
@@ -2587,6 +2625,7 @@ populate_message_from_mw (GMimeMessage* message, const Compose * mw)
 	/* charset */
 	if (1) {
 		const char * charset = pan_charset_picker_get_charset (mw->charset_om);
+		Identity   * id_to_change = identity_manager_get_identity_nocopy (mw->identity_name);
 
 		/* let the part know what its charset is. */
 		g_mime_object_set_content_type_parameter (GMIME_OBJECT(part), "charset", charset);
@@ -2611,6 +2650,14 @@ populate_message_from_mw (GMimeMessage* message, const Compose * mw)
 			g_mime_stream_unref (original_stream);
 			g_object_unref (data_wrapper);
 		}
+
+		/* remember the charset */
+		if (id_to_change != NULL)
+		{
+			identity_set_posting_charset (id_to_change, charset);
+			identity_manager_save ();
+		}
+
 	}
 
 	/* cleanup */
@@ -2705,7 +2752,7 @@ post_checked_cb (Server        * server,
 	if (goodness == OKAY)
 	{
 		const char * pch;
-		char * group_name;
+		GString * group_name;
 		Group * folder;
 
 		/* set the server to use, if not already set */
@@ -2722,14 +2769,15 @@ post_checked_cb (Server        * server,
 
 		/* remember which identity we used */
 		pch = g_mime_message_get_header (message, HEADER_NEWSGROUPS);
-		while ((group_name = get_next_token_str (pch, ',', &pch))!=NULL) {
+		group_name = g_string_new (NULL);
+		while (get_next_token_g_str (pch, ',', &pch, group_name)) {
 			Group * group;
-			g_strstrip (group_name);
-			group = server_get_named_group (server, group_name);
+			pan_g_string_strstrip (group_name);
+			group = server_get_named_group (server, group_name->str);
 			if (group!=NULL && !group_is_folder (group))
 				group_set_identity (group, mw->identity_name);
-			g_free (group_name);
 		}
+		g_string_free (group_name, TRUE);
 
 		/* close this window */
 		message_window_destroy (mw);
@@ -2906,6 +2954,8 @@ warn_wrong_charset (gpointer p)
 static void*
 run_external_editor (void * compose_voidp)
 {
+	int argc;
+	int i;
 	FILE * fp = NULL;
 	char * fname = NULL;
 	char ** argv = NULL;
@@ -2983,9 +3033,8 @@ run_external_editor (void * compose_voidp)
 
 	/* parse the command line */
 	if (ok) {
-		int argc;
 		GError * err = NULL;
-		char * cmd = pan_substitute (external_editor, "%t", fname);
+		char * cmd = g_strdup (external_editor);
 		g_shell_parse_argv (cmd, &argc, &argv, &err);
 		if (err != NULL) {
 			log_add_va (LOG_ERROR, _("Error parsing \"external editor\" command line: %s"), err->message);
@@ -2995,6 +3044,9 @@ run_external_editor (void * compose_voidp)
 		}
 		g_free (cmd);
 	}
+	for (i=0; i<argc; ++i)
+		if (strstr (argv[i], "%t"))
+			replace_gstr (&argv[i], pan_substitute(argv[i], "%t", fname));
 
 	/* spawn off the external editor */
 	if (ok) {
