@@ -615,7 +615,7 @@ nntp_article_download (StatusItem         * status,
 			if (group_ref_articles_if_loaded (group)) {
 				Article * a = group_get_article_by_message_id (group, mid->message_id);
 				if (a != NULL)
-					articles_add_flag (&a, 1, STATE_ERROR);
+					article_set_error_flag (a, TRUE);
 				group_unref_articles (group, NULL);
 			}
 			return TASK_FAIL_HOPELESS;
@@ -833,7 +833,7 @@ nntp_download_headers (StatusItem       * status,
 			((char*)tok)[tok_len] = '\0';
 			g_strstrip ((char*)tok);
 			cvt = pan_header_to_utf8 (tok, tok_len, charset);
-			article_init_header (a, HEADER_SUBJECT, cvt, DO_CHUNK_SHARE);
+			article_set_subject (a, cvt);
 			g_free (cvt);
 		}
 
@@ -841,7 +841,7 @@ nntp_download_headers (StatusItem       * status,
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
 			((char*)tok)[tok_len] = '\0';
 			g_strstrip ((char*)tok);
-			article_init_author_from_header (a, tok);
+			article_set_author (a, tok);
 		}
 
 		/* get date */
@@ -858,7 +858,7 @@ nntp_download_headers (StatusItem       * status,
 			((char*)tok)[tok_len] = '\0';
 			g_strstrip ((char*)tok);
 			if (is_nonempty_string(tok) && *tok=='<')
-				article_init_header (a, HEADER_MESSAGE_ID, tok, DO_CHUNK);
+				article_set_message_id (a, tok);
 		}
 
 		/* get references */
@@ -866,7 +866,7 @@ nntp_download_headers (StatusItem       * status,
 			((char*)tok)[tok_len] = '\0';
 			g_strstrip ((char*)tok);
 			if (is_nonempty_string(tok) && *tok=='<')
-				article_init_header (a, HEADER_REFERENCES, tok, DO_CHUNK_SHARE);
+				article_set_references (a, tok);
 		}
 
 		/* get byte qty */
@@ -881,15 +881,8 @@ nntp_download_headers (StatusItem       * status,
 			g_strstrip ((char*)tok);
 			if (tok_len>6 && !strncmp(tok,"Xref: ", 6)) {
 				const char * header = tok + 6;
-				const char * pch;
-				int crosspost_qty = 0;
-				for (pch=header; *pch; ++pch)
-					if (*pch==':')
-						++crosspost_qty;
-				if (crosspost_qty > 1) {
-					g_strstrip ((char*)header); /* remove trailing linefeed */
-					article_init_header (a, HEADER_XREF, header, DO_CHUNK);
-				}
+				g_strstrip ((char*)header); /* remove trailing linefeed */
+				article_set_xrefs (a, header);
 			}
 		}
 
