@@ -118,6 +118,26 @@ pan_charset_picker_free (GtkWidget * w)
 	return NULL;
 }
 
+gboolean
+pan_charset_picker_set_charset (GtkWidget * w, const char * charset)
+{
+	gboolean retval = FALSE;
+	gint     i;
+
+	g_return_val_if_fail (w!=NULL, FALSE);
+
+	for (i=0; i<G_N_ELEMENTS(charsets); i++)
+		if (strcmp (charsets[i].charset, charset) == 0)
+			break;
+
+	if (i < G_N_ELEMENTS(charsets)) {
+		gtk_option_menu_set_history (GTK_OPTION_MENU(w), i);
+		retval = TRUE;
+	}
+
+	return retval;
+}
+
 const char *
 pan_charset_picker_get_charset (GtkWidget * w)
 {
@@ -140,6 +160,9 @@ pan_charset_picker_has (const char * charset)
 	gint i;
 	gint n = G_N_ELEMENTS(charsets);
 
+	if (charset == NULL)
+		return FALSE;
+
 	for (i=0; i<G_N_ELEMENTS(charsets); i++)
 		if (!g_strcasecmp (charsets[i].charset, charset))
 			break;
