@@ -31,7 +31,6 @@
 #include <pan/base/pan-glib-extensions.h>
 
 #include <pan/globals.h>
-#include <pan/gui.h>
 #include <pan/prefs.h>
 #include <pan/print.h>
 #include <pan/text.h>
@@ -124,10 +123,11 @@ print_dialog_cb (GtkDialog * dialog, int response, gpointer user_data)
 
 	switch (response)
 	{
-		case GTK_RESPONSE_OK: /* Print */
+		case GTK_RESPONSE_OK:
 			print_dialog_print_clicked (user_data);
 			break;
-		case GTK_RESPONSE_CANCEL: /* Cancel */
+		case GTK_RESPONSE_CANCEL:
+		case GTK_RESPONSE_DELETE_EVENT:
 			print_dialog_destroy (user_data);
 			break;
 		default:
