@@ -212,7 +212,7 @@ queue_init (gboolean online, gboolean remove_failed_tasks)
 	_online = online;
 
 	_remove_failed_tasks = remove_failed_tasks;
-	_tpool = g_thread_pool_new (queue_run_thread, NULL, 8, TRUE, NULL);
+	_tpool = g_thread_pool_new (queue_run_thread, NULL, 5, TRUE, NULL);
 
 	/* fire up the queue thread */
 	g_thread_create (queue_mainloop, NULL, FALSE, NULL);
@@ -656,6 +656,23 @@ real_queue_insert_tasks (GSList * new_tasks, int index, gboolean * do_wakeup)
 ***
 **/
 
+void
+queue_remove_last_task (void)
+{
+        GPtrArray * tasks;
+                                                                                                                     
+        /* remove the last task, if one exists */
+        tasks = queue_get_tasks ();
+        if (tasks->len) {
+                Task * task = TASK(g_ptr_array_index (tasks, tasks->len-1));
+                queue_remove_tasks (g_slist_append(NULL,task));
+        }
+                                                                                                                     
+        /* cleanup */
+        pan_g_ptr_array_foreach (tasks, (GFunc)pan_object_unref, NULL);
+        g_ptr_array_free (tasks, TRUE);
+}
+
 void
 queue_remove_tasks (GSList * tasks)
 {
@@ -1031,7 +1048,7 @@ queue_run_what_we_can (void)
 			case TASK_NEED_SOCKET:
 				if (runnable) {
 					pool = get_socket_pool_from_server (state.server);
-					socket_pool_request_connections (pool, 1);
+					socket_pool_request_connection (pool);
 					sock = socket_pool_checkout (pool);
 					if (sock != NULL)
 						queue_run (task, state.func, sock, state.server);
@@ -1049,7 +1066,10 @@ queue_run_what_we_can (void)
 
 			case TASK_SUCCESS:
 			case TASK_FAIL_HOPELESS:
-				queue_remove_tasks (g_slist_append (NULL, task));
+				if (state.state==TASK_SUCCESS || _remove_failed_tasks)
+					queue_remove_tasks (g_slist_append (NULL, task));
+				else
+					queue_set_task_status (task, QUEUE_TASK_STATUS_STOPPED);
 				pan_callback_call (task->task_ran_callback, task, GINT_TO_POINTER(state.state));
 				break;
 
