@@ -33,9 +33,9 @@
 #include <pan/base/log.h>
 #include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
+#include <pan/base/util-file.h>
 
 #include <pan/globals.h>
-#include <pan/gui.h>
 #include <pan/prefs.h>
 #include <pan/queue.h>
 #include <pan/pan-file-entry.h>
@@ -101,10 +101,7 @@ save_attachment_as_dialog_response_cb (GtkDialog   * dialog,
 	***  Get Path
 	**/
 
-	path = g_strdup (pan_file_entry_get (save->directory_entry));
-	g_strstrip (path);
-	if (path!=NULL && *path=='~')
-		replace_gstr (&path, g_strconcat (g_get_home_dir(), path+1, NULL));
+	path = pan_file_normalize (pan_file_entry_get (save->directory_entry), NULL);
 
 	/**
 	***  Get Body Filename
@@ -135,7 +132,7 @@ save_attachment_as_dialog_response_cb (GtkDialog   * dialog,
 		task = task_save_new_from_articles ((const Article**)tmp->pdata, tmp->len);
 		g_ptr_array_free (tmp, TRUE);
 
-		expanded_path = expand_download_dir (path, article->group->name);
+		expanded_path = expand_download_dir (path, group_get_name(article->group));
 
 		if (_save_attachments)
 			task_save_set_attachments (TASK_SAVE(task), expanded_path, NULL);
@@ -216,11 +213,13 @@ static void
 create_save_attachment_as_dialog (Group * group,
                                   GSList * gslists_of_unowned_articles)
 {
+	int row;
+	int items_in_section;
+	char buf[512];
+	GtkWidget * t;
 	GtkWidget * w;
 	GtkWidget * l;
-	GtkWidget * hbox;
-	GtkWidget * vbox;
-	GtkWidget * frame;
+	GtkWidget * h;
 	GtkWidget * dialog;
 	SaveAsDialogStruct * data;
 	const char * cpch;
@@ -241,63 +240,109 @@ create_save_attachment_as_dialog (Group * group,
 	data->dialog = dialog;
 	gtk_dialog_set_default_response (GTK_DIALOG(dialog), GTK_RESPONSE_CANCEL);
 
-	/**
-	***  "Files" Pane
-	**/
+	row = 0;
+	t = gtk_table_new (4, 18, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER(t), 12);
+	gtk_table_set_row_spacings (GTK_TABLE(t), 6);
+	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), t, TRUE, TRUE, 0);
 
-	pan_hig_frame_new (_("Files"), &frame, &vbox);
-	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, FALSE, FALSE, GUI_PAD);
+	/* title for files section */
+	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Files"));
+	w = gtk_label_new (buf);
+	gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.5f);
+	gtk_label_set_use_markup (GTK_LABEL(w), TRUE);
+	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 4, row, row+1);
+	++row;
 
-	w = gtk_check_button_new_with_mnemonic (_("Save Article Te_xt"));
-	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), _save_text);
-	data->save_text_check = w;
+		items_in_section = 2;
 
-	w = gtk_check_button_new_with_mnemonic (_("Save Article _Attachments"));
-	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), _save_attachments);
-	data->save_attachments_check = w;
+		/* spacer to move the fields a little to the right of the name header */
+		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+		gtk_widget_set_usize (w, 18u, 0u);
+		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+items_in_section, 0, 0, 0, 0);
 
-	hbox = gtk_hbox_new (FALSE, GUI_PAD_SMALL);
-	gtk_box_pack_start (GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-	l = gtk_label_new_with_mnemonic (_("_Directory:"));
-	gtk_box_pack_start (GTK_BOX(hbox), l, FALSE, FALSE, 0);
-	w = pan_file_entry_new (_("Save Files in Directory"));
-	gtk_box_pack_start (GTK_BOX(hbox), w, TRUE, TRUE, 0);
-	gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
-	if (is_nonempty_string(_path))
-		cpch = _path;
-	else
-	if (group && is_nonempty_string (group->download_dir))
-		cpch = group->download_dir;
-	else
-		cpch = download_dir;
-	pan_file_entry_set (w, cpch);
-	gtk_widget_set_usize (GTK_WIDGET(w), 400, 0);
-	data->directory_entry = w;
-	w = gtk_button_new_with_mnemonic (_("_Help"));
-	gtk_box_pack_start (GTK_BOX(hbox), w, FALSE, FALSE, 0);
-	g_signal_connect_swapped (w, "clicked", G_CALLBACK (show_group_substitution_help_dialog), dialog);
+		/* spacer between the controls and their labels */
+		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+		gtk_widget_set_usize (w, 12u, 0u);
+		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+items_in_section, 0, 0, 0, 0);
 
-	/**
-	***  "Priority" Pane
-	**/
+		/* "save article text" togglebutton */
+		w = gtk_check_button_new_with_mnemonic (_("Save Te_xt"));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), _save_text);
+		data->save_text_check = w;
+		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
+		++row;
 
-	pan_hig_frame_new (_("Priority"), &frame, &vbox);
-	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), frame, FALSE, FALSE, GUI_PAD);
+		/* "save article attachments" togglebutton */
+		w = gtk_check_button_new_with_mnemonic (_("Save _Attachments"));
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), _save_attachments);
+		data->save_attachments_check = w;
+		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
+		++row;
 
-	/* front of queue */
-	w = gtk_radio_button_new_with_mnemonic (NULL, _("Add to the _top of the Task Manager's List"));
-	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), _high_priority);
-	data->priority_high_radio = w;
+		/* directory label */
+		l = gtk_label_new_with_mnemonic (_("_Directory:"));
+		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
 
-	/* back of queue (default) */
-	w = gtk_radio_button_new_with_mnemonic_from_widget (
-		GTK_RADIO_BUTTON(w), _("Add to the b_ottom of the Task Manager's List"));
-	gtk_box_pack_start (GTK_BOX(vbox), w, FALSE, FALSE, 0);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), !_high_priority);
-	data->priority_low_radio = w;
+		/* directory entry */
+		h = gtk_hbox_new (FALSE, GUI_PAD_SMALL);
+		w = data->directory_entry = pan_file_entry_new (_("Save Files in Directory"));
+		gtk_box_pack_start (GTK_BOX(h), w, TRUE, TRUE, 0);
+		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
+		if (is_nonempty_string(_path))
+			cpch = _path;
+		else
+		if (group && is_nonempty_string (group->download_dir))
+			cpch = group->download_dir;
+		else
+			cpch = download_dir;
+		pan_file_entry_set (w, cpch);
+		gtk_widget_set_usize (GTK_WIDGET(w), 400, 0);
+		w = gtk_button_new_with_mnemonic (_("_Help"));
+		gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
+		g_signal_connect_swapped (w, "clicked", G_CALLBACK (show_group_substitution_help_dialog), dialog);
+		gtk_table_attach_defaults (GTK_TABLE(t), h, 3, 4, row, row+1);
+		++row;
+
+	/* section spacer */
+	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+	gtk_widget_set_usize (w, 0u, 6u);
+	gtk_table_attach (GTK_TABLE(t), w, 0, 4, row, row+1, 0, 0, 0, 0);
+	++row;
+
+	/* title for files section */
+	g_snprintf (buf, sizeof(buf), "<b>%s</b>", _("Priority"));
+	w = gtk_label_new (buf);
+	gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.5f);
+	gtk_label_set_use_markup (GTK_LABEL(w), TRUE);
+	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 4, row, row+1);
+	++row;
+
+		items_in_section = 2;
+
+		/* spacer to move the fields a little to the right of the name header */
+		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+		gtk_widget_set_usize (w, 18u, 0u);
+		gtk_table_attach (GTK_TABLE(t), w, 0, 1, row, row+items_in_section, 0, 0, 0, 0);
+
+		/* spacer between the controls and their labels */
+		w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+		gtk_widget_set_usize (w, 12u, 0u);
+		gtk_table_attach (GTK_TABLE(t), w, 2, 3, row, row+items_in_section, 0, 0, 0, 0);
+
+		/* front of queue */
+		w = data->priority_high_radio = gtk_radio_button_new_with_mnemonic (NULL, _("Add to the _top of the Task Manager's List"));
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), _high_priority);
+		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
+		++row;
+
+		/* back of queue (default) */
+		w = gtk_radio_button_new_with_mnemonic_from_widget (
+			GTK_RADIO_BUTTON(w), _("Add to the b_ottom of the Task Manager's List"));
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), !_high_priority);
+		data->priority_low_radio = w;
+		gtk_table_attach_defaults (GTK_TABLE(t), w, 1, 4, row, row+1);
+		++row;
 
 	/**
 	***  Wire it up
