@@ -29,7 +29,6 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
-#include <pan/base/base-prefs.h>
 #include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-config.h>
@@ -204,9 +203,7 @@ edit_dialog_populate_server (ServerEditDialog * d, Server * s)
 	g_return_if_fail (server_is_valid(s));
 
 	/* server section */
-	cpch = gtk_entry_get_text (GTK_ENTRY(d->name_entry));
-	replace_gstr (&s->name, g_strdup(cpch));
-	g_strstrip (s->name);
+	server_set_name (s, gtk_entry_get_text (GTK_ENTRY(d->name_entry)));
 	cpch = gtk_entry_get_text (GTK_ENTRY(d->address_entry));
 	replace_gstr (&s->address, g_strdup(cpch));
 	g_strstrip (s->address);
@@ -242,6 +239,7 @@ server_edit_response_cb (GtkDialog * w, int response, gpointer user_data)
 		if (d->new_server)
 			serverlist_add_server (d->server);
 
+		/* wake up the queue in case user changed "max connections" field */
 		queue_wakeup ();
 	}
 	else /* cancel, destroy */
@@ -264,7 +262,7 @@ server_edit_dialog (GtkWidget * window, Server * server)
 	GtkWidget * l;
 	ServerEditDialog * d;
 	int row;
-	const int max_per_server = pan_config_get_int ("/Pan/General/Max_Connections_Per_Server=4");
+	const int DEFAULT_MAX_PER_SERVER = 4;
 
 	d = g_new0 (ServerEditDialog, 1);
 
@@ -363,7 +361,7 @@ server_edit_dialog (GtkWidget * window, Server * server)
 		gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
 		gtk_table_attach_defaults (GTK_TABLE(t), l, 1, 2, row, row+1);
 
-		a = gtk_adjustment_new (1.0, 1.0, max_per_server, 1.0, 1.0, 1.0);
+		a = gtk_adjustment_new (1.0, 1.0, DEFAULT_MAX_PER_SERVER, 1.0, 1.0, 1.0);
 		w = gtk_spin_button_new (GTK_ADJUSTMENT(a), 1.0, 0u);
 		d->connection_limit_spin = w;
 		gtk_label_set_mnemonic_widget (GTK_LABEL(l), w);
