@@ -254,7 +254,7 @@ smtp_send_message (StatusItem    * item,
 
 		while ((get_next_token_run (march, '\n', &march, &line_start, &line_len)))
 		{
-			char * line = g_alloca (line_len+1);
+			char * line = g_newa (char, line_len+1);
 			memcpy (line, line_start, line_len);
 			line[line_len] = '\0';
 
