@@ -25,8 +25,9 @@
 
 #include <errno.h>
 #include <string.h>
-#include <signal.h> /* for sigignore(SIGPIPE) on unix */
+#include <signal.h> /* for signal (SIGPIPE, SIG_IGN) */
 #include <unistd.h>
+#include <time.h> /* for time_t */
 
 #include <glib.h>
 #include <gnet/gnet.h>
@@ -65,8 +66,8 @@ struct _PanSocket
 	/* statistics */
 	char * host_address;
 	char * host_name;
-	unsigned int bytes_read;
-	unsigned int bytes_written;
+	guint bytes_read;
+	guint bytes_written;
 	time_t byte_count_start_time;
 };
 
@@ -92,9 +93,9 @@ pan_socket_new (const char     * server_name,
 	g_return_val_if_fail (is_nonempty_string (server_address), NULL);
 	g_return_val_if_fail (port>=0, NULL);
 
-	/* needed on Unix */
-#	ifdef SIGPIPE
-	sigignore (SIGPIPE);
+	/* normally SIGPIPE, from writes on closed sockets, kill the app */
+#	ifndef G_OS_WIN32
+	signal (SIGPIPE, SIG_IGN);
 #	endif
 
 	/* create the socket */
@@ -122,10 +123,10 @@ pan_socket_constructor (PanSocket             * sock,
 	pan_object_constructor (PAN_OBJECT(sock), destructor);
 	sock->host_address = g_strdup (server_address);
 	sock->host_name = g_strdup (server_name);
-	sock->bytes_read = 0;
-	sock->bytes_written = 0;
-	sock->byte_count_start_time = time(0);
-	sock->line_buffer = g_string_new (NULL);
+	sock->bytes_read = 0u;
+	sock->bytes_written = 0u;
+	sock->byte_count_start_time = time(NULL);
+	sock->line_buffer = g_string_sized_new (512);
 	sock->error = FALSE;
 	sock->need_auth = FALSE;
 	sock->nntp_group_name = NULL;
@@ -149,6 +150,7 @@ pan_socket_constructor (PanSocket             * sock,
         debug1 (DEBUG_PAN_OBJECT, "pan_socket_constructor: %p", sock);
 }
 
+#define DEAD_POINTER ((void*)(0xDEADBEEF))
 
 void
 pan_socket_destructor (PanObject *obj)
@@ -160,16 +162,20 @@ pan_socket_destructor (PanObject *obj)
 	debug1 (DEBUG_SOCKET, "socket closed: %p", sock);
         debug1 (DEBUG_PAN_OBJECT, "pan_socket_destructor: %p", sock);
 
-	/* free up our pieces */
 	g_io_channel_unref (sock->gio_channel);
 	gnet_tcp_socket_delete (sock->gnet_socket);
-	g_free (sock->nntp_group_name);
-	g_free (sock->nntp_username);
-	g_free (sock->nntp_password);
-	g_free (sock->host_address);
-	g_free (sock->host_name);
 	g_string_free (sock->line_buffer, TRUE);
 
+	sock->gio_channel = DEAD_POINTER;
+	sock->gnet_socket = DEAD_POINTER;
+	sock->line_buffer = DEAD_POINTER;
+
+	replace_gstr (&sock->nntp_group_name, DEAD_POINTER);
+	replace_gstr (&sock->nntp_username, DEAD_POINTER);
+	replace_gstr (&sock->nntp_password, DEAD_POINTER);
+	replace_gstr (&sock->host_address, DEAD_POINTER);
+	replace_gstr (&sock->host_name, DEAD_POINTER);
+
 	/* pass along to superclass */
 	pan_object_destructor (obj);
 }
