@@ -33,7 +33,6 @@
 #include <pan/base/argset.h>
 #include <pan/base/debug.h>
 
-#include <pan/gui.h>
 #include <pan/status-item-view.h>
 #include <pan/util.h>
 
@@ -102,42 +101,66 @@ extern GtkTooltips * ttips;
 *****/
 
 static char*
-string_shorten_nolock (GdkFont          * font,
+string_shorten_nolock (GtkWidget        * w,
                        int                max_width,
                        const char       * original,
                        int                original_len)
 {
 	gboolean i_turn = TRUE;
-	int i, j, original_midpoint;
+	int i, j, len, original_midpoint, width=0;
 	const int target_width = max_width - 4 - 30; /* 30 is just a magic safety number */
 	GString * tmp;
+	PangoLayout * pango_layout;
+	const char * cpch_midpoint;
 
 	/* handle cases where the string isn't too long */
 	if (!original)
 		return NULL;
-	if (gdk_text_width (font, original, original_len) < target_width)
+
+	pango_layout = gtk_widget_create_pango_layout (w, NULL);
+	pango_layout_set_text (pango_layout, original, original_len);
+	pango_layout_get_pixel_size (pango_layout, &width, NULL);
+	if (width < target_width) {
+		g_object_unref (G_OBJECT(pango_layout));
 		return g_strndup (original, original_len);
+	}
 
 	/* build the clone with elipses in the middle */
-	original_midpoint = original_len / 2;
+	len = g_utf8_strlen (original, original_len);
+	original_midpoint = len / 2;
+	cpch_midpoint = g_utf8_offset_to_pointer (original, original_midpoint);
        	tmp = g_string_sized_new (128);
-	g_string_append_len (tmp, original, original_midpoint);
+	g_string_append_len (tmp, original, cpch_midpoint - original);
 	g_string_append (tmp, "...");
-	g_string_append (tmp, original+original_midpoint);
+	g_string_append (tmp, cpch_midpoint);
 
 	/* start trimming out the characters next to the ellipses */
 	i = original_midpoint - 1;
-	j = original_midpoint + 3; /* "..." */
-	while (gdk_text_width (font, tmp->str, tmp->len) >= target_width)
+	j = original_midpoint + 2; /* "..." */
+	for (;;)
 	{
-		int *pi = i_turn ? &i : &j;
-		*pi -= 1;
-		if (*pi<0 || *pi>=tmp->len)
+		int * pi;
+		const char * begin, * end;
+
+		/* check the width */
+		pango_layout_set_text (pango_layout, tmp->str, tmp->len);
+		pango_layout_get_pixel_size (pango_layout, &width, NULL);
+		if (width < target_width)
 			break;
-		g_string_erase (tmp, *pi, 1);
+
+		pi = i_turn ? &i : &j;
+		begin = g_utf8_offset_to_pointer (tmp->str, *pi);
+		end = i_turn ? g_utf8_prev_char (begin) : g_utf8_next_char (begin);
+
+		*pi -= 1;
+		if (*pi<0 || *pi>=len)
+			break;
+		g_string_erase (tmp, (i_turn ? end : begin) - tmp->str, abs (end-begin));
+		len--;
 		i_turn = !i_turn;
 	}
 
+	g_object_unref (G_OBJECT(pango_layout));
 	return g_string_free (tmp, FALSE);
 }
 
@@ -185,10 +208,10 @@ status_cb_mainthread (gpointer data)
 		char * s;
 
 		g_mutex_lock (impl->last_status_mutex);
-		s = string_shorten_nolock (gtk_style_get_font(gtk_widget_get_style(impl->progress)),
-		                                   impl->progress->allocation.width,
-		                                   impl->last_status->str,
-		                                   impl->last_status->len);
+		s = string_shorten_nolock (impl->progress,
+		                           impl->progress->allocation.width,
+		                           impl->last_status->str,
+		                           impl->last_status->len);
 		g_mutex_unlock (impl->last_status_mutex);
 
 		/* update progressbar w/o a resize */
