@@ -183,7 +183,7 @@ task_bodies_new_from_articles (const Article       ** articles,
 	g_return_val_if_fail (articles_are_valid (articles, article_qty), NULL);
 
 	/* build the task */
-	mids = g_alloca (sizeof(MessageIdentifier*) * article_qty);
+	mids = g_newa (MessageIdentifier*, article_qty);
 	for (i=0; i<article_qty; ++i)
 		mids[i] = message_identifier_new_from_article (articles[i]);
 	retval = task_bodies_new (articles[0]->group->server, mids, article_qty);
@@ -209,6 +209,9 @@ task_bodies_describe (const StatusItem* si)
 	else {
 		MessageIdentifier * mid = MESSAGE_IDENTIFIER (g_ptr_array_index (mids, 0));
 		retval = g_strdup (message_identifier_get_readable_name (mid));
+
+		if (!g_utf8_validate (retval, -1, NULL))
+			replace_gstr (&retval, g_strdup (_("Downloading 1 article")));
 	}
 
 	return retval;
