@@ -259,6 +259,8 @@ nntp_grouplist_download_all (TaskGroupList * item, PanSocket * sock)
 	{
 		gint val;
 		const gchar * response = NULL;
+		GString * name;
+		GString * permission;
 
 		val = *abort ? -1 : nntp_command (status, sock, &response, "LIST");
 		if (val != 215) {
@@ -267,26 +269,26 @@ nntp_grouplist_download_all (TaskGroupList * item, PanSocket * sock)
 		}
 
 		/* read the group list from the server. */
+		name = g_string_new (NULL);
+		permission = g_string_new (NULL);
 		while (!*abort && !(read_err=pan_socket_getline(sock,&response)))
 		{
-			const gchar * march = response;
+			const char * march = response;
 			Group * group;
-			gchar * name;
-			gchar * permission;
 
 			/* check for end of list */
 			if (!strcmp(response,".\r\n"))
 				break;
 
 			/* parse the data line */
-			name = get_next_token_str (march, ' ', &march);
+			get_next_token_g_str (march, ' ', &march, name);
 			skip_next_token (march, ' ', &march); /* skip low number */
 			skip_next_token (march, ' ', &march); /* skip high number */
-			permission = get_next_token_str (march, ' ', &march);
+			get_next_token_g_str (march, ' ', &march, permission);
 
 			/* create a new group object */
-			group = group_new (server, name);
-			group->permission = *permission;
+			group = group_new (server, name->str);
+			group->permission = *permission->str;
 			g_ptr_array_add (groups, group);
 
 			/* periodic status feedback */
@@ -295,11 +297,9 @@ nntp_grouplist_download_all (TaskGroupList * item, PanSocket * sock)
 					_("Got %d groups from server \"%s\""),
 					(int)groups->len,
 					server_get_name (server));
-
-			/* cleanup */
-			g_free (name);
-			g_free (permission);
 		}
+		g_string_free (permission, TRUE);
+		g_string_free (name, TRUE);
 
 		/* make sure the new groups get saved */
 		if (groups->len)
@@ -408,6 +408,7 @@ nntp_grouplist_download_all (TaskGroupList * item, PanSocket * sock)
 static int
 nntp_grouplist_download_new (TaskGroupList * item, PanSocket * sock)
 {
+	GString * name;
 	GPtrArray * groups = item->groups;
 	const gboolean * const abort = &TASK(item)->hint_abort;
 	Server *server = TASK(item)->server;
@@ -451,15 +452,13 @@ nntp_grouplist_download_new (TaskGroupList * item, PanSocket * sock)
 	if (*abort || pan_socket_getline (sock, &response))
 		return TASK_FAIL;
 
+	name = g_string_new (NULL);
 	while (strncmp(response, ".\r\n", 3))
 	{
-		gchar * name;
-
 		/* create a Group */
-		name = get_next_token_str (response, ' ', NULL);
-		group = group_new (server, name);
+		get_next_token_g_str (response, ' ', NULL, name);
+		group = group_new (server, name->str);
 		group_set_dirty (group); /* make sure it gets saved */
-		g_free (name);
 		group->flags |= GROUP_NEW;
 		g_ptr_array_add (groups, group);
 
@@ -473,9 +472,11 @@ nntp_grouplist_download_new (TaskGroupList * item, PanSocket * sock)
 		}
 
 		if (*abort || pan_socket_getline (sock, &response)) {
+			g_string_free (name, TRUE);
 			return TASK_FAIL;
 		}
 	}
+	g_string_free (name, TRUE);
 
 	if (1) {
 		char buf[512];
