@@ -31,15 +31,15 @@
 
 #include <pan/base/acache.h>
 #include <pan/base/argset.h>
-#include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
+#include <pan/base/gnksa.h>
 #include <pan/base/log.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-object.h>
 #include <pan/base/pan-i18n.h>
 #include <pan/base/serverlist.h>
+#include <pan/base/text-massager.h>
 #include <pan/base/util-mime.h>
-#include <pan/base/util-wrap.h>
 
 #include <pan/articlelist.h>
 #include <pan/gui.h>
@@ -55,7 +55,6 @@
 
 
 static GtkTextBuffer * _text_buffer = NULL;
-static GtkWidget * header_table = NULL;
 static GtkWidget * scrolled_window = NULL;
 static GtkTextMark * _begin_mark = NULL;
 
@@ -81,7 +80,9 @@ PanCallback * current_article_changed = NULL;
  * @return a const string for the line's quote tag.  Never NULL.
  */
 static const char *
-get_quote_tag (const char * utf8_line, int utf8_byte_len)
+get_quote_tag (const TextMassager     * text_massager,
+               const char             * utf8_line,
+               int                      utf8_byte_len)
 {
 	const char * str = utf8_line;
 	const char * line_end = utf8_line + utf8_byte_len;
@@ -99,7 +100,7 @@ get_quote_tag (const char * utf8_line, int utf8_byte_len)
 		for (;;) {
 			if (str == line_end)
 				break;
-			else if (is_quote_char((guchar)*str))
+			else if (text_massager_is_quote_char(text_massager, (guchar)*str))
 			       ++depth;
 			else if (!g_unichar_isspace(g_utf8_get_char(str)))
 				break;
@@ -160,9 +161,10 @@ find_next_url (const char * searchme)
  * quoted and URL areas in the GtkTextBuffer.
  */
 static void
-append_text_buffer_nolock (GtkTextBuffer * text_buffer,
-                           const char    * body,
-		           gboolean        mute_quotes)
+append_text_buffer_nolock (const TextMassager  * text_massager,
+                           GtkTextBuffer       * text_buffer,
+                           const char          * body,
+		           gboolean              mute_quotes)
 {
 	int line_len;
 	char * freeme1 = NULL;
@@ -186,7 +188,7 @@ append_text_buffer_nolock (GtkTextBuffer * text_buffer,
 
 	/* mute the quoted text, if desired */
 	if (mute_quotes)
-		body = freeme1 = pan_text_mute_quoted (body);
+		body = freeme1 = text_massager_mute_quoted (text_massager, body);
 
 	body = pan_utf8ize (body, -1, &freeme2);
 
@@ -207,13 +209,9 @@ append_text_buffer_nolock (GtkTextBuffer * text_buffer,
 			continue;
 		
 		if (!is_sig)
-		{
-			int is = pan_is_signature_delimitor (line, line_len);
-			/*support both standard and non standard sigs when highlighting*/
-			is_sig = (is==SIG_STANDARD)||(is==SIG_NON_STANDARD);
-		}
-		quote_tag = get_quote_tag (line, line_len);
-		
+			is_sig = pan_is_signature_delimiter (line, line_len) != SIG_NONE;
+
+		quote_tag = get_quote_tag (text_massager, line, line_len);
 		if (strcmp (quote_tag, "quote_0"))
 			is_sig = FALSE;
 		
@@ -259,20 +257,22 @@ append_text_buffer_nolock (GtkTextBuffer * text_buffer,
 
 /**
  * Clears out the old buffer contents and populates it with body.
+ * @param text_massager holds information on how to wrap/quote the text
  * @param text_buffer the text buffer to repopulate.
  * @param body the new text to be placed in the buffer
  * @param mute_quotes true if quotes are to be muted in the buffer, false otherwise.
  */
 void
-update_body_pane (GtkTextBuffer  * text_buffer,
-                  const char     * body,
-		  gboolean         mute_quotes)
+update_body_pane (const TextMassager  * text_massager,
+                  GtkTextBuffer       * text_buffer,
+                  const char          * body,
+		  gboolean              mute_quotes)
 {
 	debug_enter ("update_body_pane");
 
 	pan_lock ();
 	clear_text_buffer_nolock (text_buffer);
-	append_text_buffer_nolock (text_buffer, body, mute_quotes);
+	append_text_buffer_nolock (text_massager, text_buffer, body, mute_quotes);
 	pan_unlock ();
 
 	debug_exit ("update_body_pane");
@@ -286,21 +286,21 @@ update_body_pane (GtkTextBuffer  * text_buffer,
 void
 text_set_font (void)
 {
-	if (text_use_fixed_font)
+	if (body_pane_monospace_font_enabled)
 	{
-		pan_widget_set_font (GTK_WIDGET(Pan.text), message_body_font_fixed);
+		pan_widget_set_font (GTK_WIDGET(Pan.text), body_pane_monospace_font);
 	}
-	else if (!use_custom_fonts)
+	else if (body_pane_custom_font_enabled)
+	{
+		pan_widget_set_font (GTK_WIDGET(Pan.text), body_pane_custom_font);
+	}
+	else
 	{
 		GtkStyle * style = gtk_widget_get_default_style ();
 		char * name = pango_font_description_to_string (style->font_desc);
 		pan_widget_set_font (Pan.text, name);
 		g_free (name);
 	}
-	else
-	{
-		pan_widget_set_font (GTK_WIDGET(Pan.text), message_body_font);
-	}
 
 	text_refresh ();
 }
@@ -464,7 +464,8 @@ get_pixbuf_from_gmime_part (const GMimePart * part)
 	gdk_pixbuf_loader_close (l, NULL);
 
 	/* cleanup */
-	g_object_ref (G_OBJECT(pixbuf));
+	if (pixbuf != NULL)
+		g_object_ref (G_OBJECT(pixbuf));
 	g_object_unref (G_OBJECT(l));
 
 	return pixbuf;
@@ -472,6 +473,7 @@ get_pixbuf_from_gmime_part (const GMimePart * part)
 
 typedef struct
 {
+	const TextMassager * text_massager;
 	const char * default_charset;
 	const GMimeMessage * message;
 	GtkTextBuffer * buffer;
@@ -495,13 +497,17 @@ insert_part_partfunc (GMimeObject * obj, gpointer ips_gpointer)
 	type = g_mime_object_get_content_type (GMIME_OBJECT (part));
 	yenc = g_object_get_data (G_OBJECT(obj), "yenc");
 	
-	if (yenc!= NULL && part->content!=NULL && part->content->stream!=NULL)
+	if (yenc!= NULL
+		&& part->content!=NULL
+		&& part->content->stream!=NULL
+		&& g_object_get_data(G_OBJECT(part),"Y_DECODER_INSTALLED")==NULL)
 	{
 		GMimeStream * stream;
 	       
 		stream = g_mime_stream_filter_new_with_stream (part->content->stream);
 		g_mime_stream_filter_add (GMIME_STREAM_FILTER(stream), g_mime_filter_yenc_new (GMIME_FILTER_YENC_DIRECTION_DECODE));
 		g_mime_data_wrapper_set_stream (part->content, stream);
+		g_object_set_data (G_OBJECT(part), "Y_DECODER_INSTALLED", GINT_TO_POINTER(1));
 
 		g_mime_stream_unref (stream);
 	}
@@ -514,8 +520,9 @@ insert_part_partfunc (GMimeObject * obj, gpointer ips_gpointer)
 		{
 			GtkTextIter iter;
 			gtk_text_buffer_get_end_iter (ips->buffer, &iter);
+			gtk_text_buffer_insert (ips->buffer, &iter, "\n", -1);
 			gtk_text_buffer_insert_pixbuf (ips->buffer, &iter, pixbuf);
-			gtk_text_buffer_insert (ips->buffer, &iter, "\n\n", -1);
+			gtk_text_buffer_insert (ips->buffer, &iter, "\n", -1);
 			g_object_unref (pixbuf);
 		}
 	}
@@ -524,49 +531,215 @@ insert_part_partfunc (GMimeObject * obj, gpointer ips_gpointer)
 		char * str = pan_body_to_utf8 (part, ips->default_charset);
 
 		if (text_get_wrap() && is_nonempty_string(str))
-			replace_gstr (&str, pan_text_fill (str,wrap_column));      
+			replace_gstr (&str, text_massager_fill (ips->text_massager, str));
 
-       		append_text_buffer_nolock (ips->buffer, str, text_get_mute_quoted());
+       		append_text_buffer_nolock (ips->text_massager, ips->buffer, str, text_get_mute_quoted());
 
 		g_free (str);
 	}
 }
+static void
+add_header (GtkTextBuffer * tbuf, int key_width, const char * header, const char * value, iconv_t iconv_utf8)
+{
+	if (is_nonempty_string (value))
+	{
+		char buf1[64];
+		char buf2[64];
+		char buf3[1024];
+		GtkTextIter end;
+		char * utf8_value;
+		glong utf8_len;
+
+		utf8_value  = g_mime_iconv_strdup (iconv_utf8, value);
+		if (!utf8_value)
+		{
+			const char * charset = get_charset_from_locale ();
+			iconv_t cd = g_mime_iconv_open ("UTF-8", charset);
+			utf8_value = g_mime_iconv_strdup (cd, value);
+			g_mime_iconv_close (cd);
+		}
+		if (!utf8_value)
+			utf8_value = g_strdup ("?");
+		utf8_len = g_utf8_strlen (utf8_value, -1);
+
+
+		if (!text_get_show_all_headers() && g_strstr_len (utf8_value, utf8_len, "=?"))
+			replace_gstr (&utf8_value, g_mime_utils_8bit_header_decode (utf8_value));
+
+		gtk_text_buffer_get_end_iter (tbuf, &end);
+
+		g_snprintf (buf1, sizeof(buf1), "%s:", header);
+		g_snprintf (buf2, sizeof(buf2), "%%-%ds", key_width+3);
+		g_snprintf (buf3, sizeof(buf3), buf2, buf1);
+		gtk_text_buffer_insert_with_tags_by_name (tbuf, &end, buf3, -1, "header_key", NULL);
+
+		g_snprintf (buf3, sizeof(buf3), "%s\n", utf8_value);
+		gtk_text_buffer_insert_with_tags_by_name (tbuf, &end, buf3, -1, "header_val", NULL);
+
+		g_free (utf8_value);
+	}
+}
+
+typedef struct
+{
+	const char * key;
+	const char * value;
+}
+AddHeaderStruct;
 
 static void
-set_text_from_message_nolock (GMimeMessage * message)
+add_headers (GtkTextBuffer * tbuf, AddHeaderStruct * headers, int header_qty, const char * default_charset)
+{
+	int i;
+	int max_key_len = 0;
+
+	/* sanity clause */
+	g_return_if_fail (tbuf!=NULL);
+	g_return_if_fail (headers!=NULL);
+	g_return_if_fail (header_qty>=0);
+	for (i=0; i<header_qty; ++i)
+		g_return_if_fail (is_nonempty_string (headers[i].key));
+
+	/* find the maximum header name length */
+	for (i=0; i<header_qty; ++i) {
+		const size_t len = strlen (headers[i].key);
+		if (len > max_key_len)
+			max_key_len = len;
+	}
+
+	/* add the headers */
+	if (1) {
+		iconv_t cd = g_mime_iconv_open ("UTF-8", default_charset);
+		for (i=0; i<header_qty; ++i)
+			if (is_nonempty_string(headers[i].key))
+				add_header (tbuf, max_key_len, headers[i].key, headers[i].value, cd);
+		g_mime_iconv_close (cd);
+	}
+}
+
+static void
+add_newline (GtkTextBuffer * tbuf)
+{
+	GtkTextIter end;
+	gtk_text_buffer_get_end_iter (tbuf, &end);
+	gtk_text_buffer_insert (tbuf, &end, "\n", -1);
+}
+
+static void
+add_header_info_nolock (GMimeMessage * message, GtkTextBuffer * tbuf, gulong header_flags, const char * default_charset)
+{
+	int header_qty = 0;
+	AddHeaderStruct headers[128];
+
+	if (header_flags & UI_HEADER_SUBJECT) {
+		headers[header_qty].key = _("Subject");
+		headers[header_qty++].value = g_mime_message_get_header (message, HEADER_SUBJECT);
+	}
+	if (header_flags & UI_HEADER_AUTHOR) {
+		headers[header_qty].key = _("From");
+		headers[header_qty++].value = g_mime_message_get_header (message, HEADER_FROM);
+	}
+	if (header_flags & UI_HEADER_REPLY_TO) {
+		headers[header_qty].key = _("Reply-To");
+		headers[header_qty++].value = g_mime_message_get_header (message, HEADER_REPLY_TO);
+	}
+	if (header_flags & UI_HEADER_NEWSGROUPS) {
+		headers[header_qty].key = _("Newsgroups");
+		headers[header_qty++].value = g_mime_message_get_header (message, HEADER_NEWSGROUPS);
+	}
+	if (header_flags & UI_HEADER_FOLLOWUP_TO) {
+		headers[header_qty].key = _("Followup-To");
+		headers[header_qty++].value = g_mime_message_get_header (message, HEADER_FOLLOWUP_TO);
+	}
+	if (header_flags & UI_HEADER_MESSAGE_ID) {
+		headers[header_qty].key = _("Message-ID");
+		headers[header_qty++].value = g_mime_message_get_header (message, HEADER_MESSAGE_ID);
+	}
+	if (header_flags & UI_HEADER_REFERENCES) {
+		headers[header_qty].key = _("References");
+		headers[header_qty++].value = g_mime_message_get_header (message, HEADER_REFERENCES);
+	}
+	if (header_flags & UI_HEADER_NEWSREADER) {
+		headers[header_qty].key = _("X-Newsreader");
+		headers[header_qty++].value = g_mime_message_get_header (message, HEADER_X_NEWSREADER);
+		headers[header_qty].key = _("X-Mailer");
+		headers[header_qty++].value = g_mime_message_get_header (message, HEADER_X_MAILER);
+		headers[header_qty].key = _("User-Agent");
+		headers[header_qty++].value = g_mime_message_get_header (message, HEADER_USER_AGENT);
+	}
+	if (header_flags & UI_HEADER_DATE) {
+		headers[header_qty].key = _("Date");
+		headers[header_qty++].value = g_mime_message_get_header (message, HEADER_DATE);
+	}
+
+	if (header_qty > 0) {
+		add_headers (tbuf, headers, header_qty, default_charset);
+		add_newline (tbuf);
+	}
+}
+
+static void
+add_to_headerstruct_func (const char *name, const char *value, gpointer data)
+{
+	AddHeaderStruct h;
+	h.key = name;
+	h.value = value;
+	g_array_append_val ((GArray*)data, h);
+}
+
+static int
+compare_headerstructs (gconstpointer va, gconstpointer vb)
+{
+	const AddHeaderStruct * a = (const AddHeaderStruct*) va;
+	const AddHeaderStruct * b = (const AddHeaderStruct*) vb;
+	return strcmp (a->key, b->key);
+}
+ 
+static void
+set_text_from_message_nolock (const TextMassager   * text_massager,
+                              GMimeMessage         * message)
 {
 	const char * default_charset = (const char*) message ? g_object_get_data (G_OBJECT(message), "default_charset") : NULL;
 	debug_enter ("set_text_from_message_nolock");
 
-	/* upper headers */
-	gui_headers_set_default_nolock (header_table, message);
-
 	/* clear */
 	clear_text_buffer_nolock (_text_buffer);
 
 	/* add headers */
-	if (message!=NULL && text_get_show_all_headers())
+	if (message!=NULL)
 	{
-		char * pch;
-		iconv_t cd;
+		if (!text_get_show_all_headers())
+		{
+			add_header_info_nolock (message, _text_buffer, header_flags, default_charset);
+		}
+		else
+		{
+			GArray * headers = g_array_new (FALSE, FALSE, sizeof(AddHeaderStruct));
+			g_mime_header_foreach (GMIME_OBJECT(message)->headers, add_to_headerstruct_func, headers);
+			g_array_sort (headers, compare_headerstructs);
+        		if (message->mime_part) {
+				add_to_headerstruct_func ("MIME-Version", "1.0", headers);
+				g_mime_header_foreach (message->mime_part->headers, add_to_headerstruct_func, headers);
+			}
+			add_headers (_text_buffer, (AddHeaderStruct*)headers->data, headers->len, default_charset);
+			add_newline (_text_buffer);
+			g_array_free (headers, TRUE);
+		}
+	}
 
-		/* get the headers in a UTF-8 friendly form */
-	       	pch = g_mime_object_get_headers (GMIME_OBJECT(message));
-		cd = g_mime_iconv_open ("UTF-8", default_charset);
-		replace_gstr (&pch, g_mime_iconv_strdup (cd, pch));
-		g_mime_iconv_close (cd);
-
-		/* add the headers to the text buffer */
-		append_text_buffer_nolock (_text_buffer, pch, FALSE);
-		append_text_buffer_nolock (_text_buffer, "\n\n", FALSE);
-
-		g_free (pch);
+	/* make a mark _after_ the headers so that we know where to get the body
+	   from when the user hits reply/follow-up */
+	if (1) {
+		GtkTextIter iter;
+		gtk_text_buffer_get_end_iter (_text_buffer, &iter);
+		gtk_text_buffer_create_mark (_text_buffer, "body_begin", &iter, TRUE);
 	}
 
 	/* add body */
 	if (message)
 	{
 		InsertPartStruct ips;
+		ips.text_massager = text_massager;
 		ips.default_charset = default_charset;
 		ips.message = message;
 		ips.buffer = _text_buffer;
@@ -635,6 +808,8 @@ set_current_message_impl (GMimeMessage * message)
 static int
 set_current_message_mainthread (GMimeMessage * message)
 {
+	const TextMassager * text_massager = text_pane_get_text_massager ();
+
 	debug_enter ("set_current_message_mainthread");
 
 	/* update the current message */
@@ -644,7 +819,7 @@ set_current_message_mainthread (GMimeMessage * message)
 	if (message != NULL)
 	{
 		pan_lock ();
-		set_text_from_message_nolock (message);
+		set_text_from_message_nolock (text_massager, message);
 		pan_unlock ();
 		pan_g_mime_message_mark_read (message);
 
@@ -653,9 +828,7 @@ set_current_message_mainthread (GMimeMessage * message)
 	else
 	{
 		pan_lock ();
-		gui_headers_set_default_nolock (header_table, NULL);
-		set_text_from_message_nolock (NULL);
-
+		set_text_from_message_nolock (text_massager, NULL);
 		pan_unlock ();
 	}
 
@@ -684,8 +857,7 @@ text_clear_nolock (void)
 	set_current_message_impl (NULL);
 
 	/* update gui */
-	gui_headers_set_default_nolock (header_table, NULL);
-	set_text_from_message_nolock (NULL);
+	set_text_from_message_nolock (text_pane_get_text_massager(), NULL);
 }
 
 void
@@ -716,7 +888,12 @@ task_bodies_ran_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
 		if (message != NULL)
 		{
 			char * default_charset = (char*) PAN_OBJECT(task)->user_data;
-			g_object_set_data_full (G_OBJECT(message), "default_charset", g_strdup(default_charset), g_free);
+			const char * charset = pan_g_mime_message_get_charset (message);
+
+			if (!is_nonempty_string (charset))
+				charset = default_charset;
+
+			g_object_set_data_full (G_OBJECT(message), "default_charset", g_strdup(charset), g_free);
 			set_current_message (message);
 			g_object_unref (message);
 			g_free (default_charset);
@@ -748,7 +925,12 @@ text_set_from_identifiers (Server             * server,
 
 		if (message)
 		{
-			g_object_set_data_full (G_OBJECT(message), "default_charset", g_strdup(default_charset), g_free);
+			const char * charset = pan_g_mime_message_get_charset (message);
+
+			if (!is_nonempty_string (charset))
+				charset = default_charset;
+
+			g_object_set_data_full (G_OBJECT(message), "default_charset", g_strdup(charset), g_free);
 			set_current_message (message);
 			g_object_unref (message);
 			message = NULL;
@@ -829,7 +1011,7 @@ text_rot13_selected_text_nolock (void)
 	if (gtk_text_buffer_get_selection_bounds (_text_buffer, &sel_start, &sel_end))
 	{
 		char * text = gtk_text_iter_get_text (&sel_start, &sel_end);
-		pan_text_rot13_inplace (text);
+		text_massager_rot13_inplace (text_pane_get_text_massager(), text);
 		gtk_text_buffer_delete (_text_buffer, &sel_start, &sel_end);
 		gtk_text_buffer_insert (_text_buffer, &sel_end, text, -1);
 		g_free (text);
@@ -947,8 +1129,16 @@ text_button_pressed (GtkWidget * w, GdkEventButton * event, gpointer unused)
 		char * url = get_url_from_location (w, (int)event->x, (int)event->y);
 		if (url != NULL)
 		{
-			pan_url_show (url);
-			g_free (url);
+			/* this is kind of a crude way of making sure that double-click
+			 * doesn't open two or three browser windows. */
+			static time_t last_url_time = 0;
+			const time_t this_url_time = time (NULL);
+			if (this_url_time != last_url_time)
+			{
+				last_url_time = this_url_time;
+				pan_url_show (url);
+				g_free (url);
+			}
 		}
 	}
 
@@ -1005,32 +1195,6 @@ motion_notify_event (GtkWidget * w, GdkEventMotion * event, gpointer user_data)
 	return FALSE;
 }
 
-static char welcome[] =
-{
-"Pan " VERSION "\n"
-"Copyright  2002 by Charles Kerr \n"
-"\n"
-"If you find a bug, please report it.\n"
-"\n"
-"http://pan.rebelbase.com/ - Pan Homepage\n"
-"http://pan.rebelbase.com/bugs/ - Report a Bug\n"
-"http://pan.rebelbase.com/download/ - Upgrade\n"
-"\n"
-"This program is free software; you can redistribute it \n"
-"and/or modify it under the terms of the GNU General Public \n"
-"License as published by the Free Software Foundation; \n"
-"version 2 of the License.\n"
-"\n"
-"This program is distributed in the hope that it will be \n"
-"useful, but WITHOUT ANY WARRANTY; without even the implied \n"
-"warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR \n"
-"PURPOSE.  See the GNU General Public License for more \n"
-"details. \n"
-"\n"
-"The GNU Public License can be found from the menu above \n"
-"in Help|About|License. \n"
-};
-
 /*
  * The following code was "stolen" from the gtk-devel archives
  */
@@ -1078,31 +1242,6 @@ gtk_text_tag_copy_values (GtkTextTag *dest, GtkTextTag *src)
 }
 
 
-#if 0
-/**
- ** gtk_text_tag_copy: -duplicate a tag
- ** @tag: the #GtkTextTag to duplicate
- ** 
- ** Create a new #GtkTextTag with same values as @tag. The returned
- ** #GtkTextTag doesn't belong to any $GtkTextTagTable.
- ** 
- ** Return value: the newly created #GtkTextTag
- ***/
-static GtkTextTag*
-gtk_text_tag_copy (GtkTextTag *tag)
-{
-	GtkTextTag *ntag;
-
-	g_return_val_if_fail (tag, NULL);
-	g_return_val_if_fail (GTK_IS_TEXT_TAG (tag), NULL);
-
-	ntag = gtk_text_tag_new (tag->name);
-	gtk_text_tag_copy_values (ntag, tag);
-	return ntag;
-}
-#endif
-
-
 /**
  ** gtk_text_tag_table_install:
  ** @table: a #GtkTextTagTable
@@ -1141,7 +1280,15 @@ text_set_text_buffer_tags (GtkTextBuffer * buffer)
 {
 	GtkTextTag * tag;
 	GtkTextTagTable * table = gtk_text_buffer_get_tag_table (buffer);
-	
+
+	tag = gtk_text_tag_new ("header_key");
+	g_object_set (tag, "font", "courier bold", NULL);
+	gtk_text_tag_table_install (table, tag);
+
+	tag = gtk_text_tag_new ("header_val");
+	g_object_set (tag, "font", "courier", NULL);
+	gtk_text_tag_table_install (table, tag);
+
 	tag = gtk_text_tag_new ("center");
 	g_object_set (tag, "justification", GTK_JUSTIFY_CENTER, NULL);
 	gtk_text_tag_table_install (table, tag);
@@ -1169,7 +1316,6 @@ text_set_text_buffer_tags (GtkTextBuffer * buffer)
 	tag = gtk_text_tag_new ("signature");
 	g_object_set (tag, "foreground_gdk", &signature_color, NULL);
 	gtk_text_tag_table_install (table, tag);
-	
 }
 
 	
@@ -1195,6 +1341,7 @@ text_create (void)
 	/* set up the buffer tags */
 	_text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(text_view));
 	text_set_text_buffer_tags (_text_buffer);
+
 	/* set a beginning mark that we can always scroll to */
 	if (1) {
 		GtkTextIter start;
@@ -1203,6 +1350,7 @@ text_create (void)
 	}
 
 
+
 	current_article_changed = pan_callback_new ();
 
 	/* the larger scrolled window */
@@ -1214,18 +1362,41 @@ text_create (void)
 	gtk_container_add (GTK_CONTAINER(scrolled_window), text_view);
 
 	if (1) {
-		iconv_t cd = g_mime_iconv_open ("UTF-8", "ISO-8859-1");
-		char * pch = g_mime_iconv_strdup (cd, welcome);
-		g_mime_iconv_close (cd);
-		append_text_buffer_nolock (_text_buffer, pch, FALSE);
-		g_free (pch);
+		GString * str = g_string_new (NULL);
+
+		g_string_append_printf (str,
+			_("Pan %s\nCopyright (c) %d by Charles Kerr\n"
+			  "\n"
+			  "If you find a bug, please report it.\n"
+			  "\n"), VERSION, 2003);
+		g_string_append (str,
+			_("http://pan.rebelbase.com/ - Pan Homepage\n"
+			  "http://pan.rebelbase.com/bugs/ - Report a Bug\n"
+			  "http://pan.rebelbase.com/download/ - Upgrade\n"
+			  "\n"));
+		g_string_append (str,
+			_("This program is free software; you can redistribute it\n"
+			  "and/or modify it under the terms of the GNU General Public\n"
+			  "License as published by the Free Software Foundation;\n"
+			  "version 2 of the License.\n"
+			  "\n"
+			  "This program is distributed in the hope that it will be\n"
+			  "useful, but WITHOUT ANY WARRANTY; without even the implied\n"
+			  "warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR\n"
+			  "PURPOSE.  See the GNU General Public License for more\n"
+			  "details.\n"
+			  "\n"
+			  "The GNU Public License can be found from the menu above\n"
+			  "in Help|About|License."));
+		append_text_buffer_nolock (text_pane_get_text_massager(), _text_buffer, str->str, FALSE);
+
+		/* cleanup */
+		g_string_free (str, TRUE);
 	}
 
 	/* text_box holds the header info and the scrolled text window */
         text_box = gtk_vbox_new (FALSE, 4);
 	gtk_container_set_border_width (GTK_CONTAINER(text_box), 4);
-        header_table = gtk_table_new ( 0, 0, FALSE);
-        gtk_box_pack_start (GTK_BOX(text_box), header_table, FALSE, FALSE, 0);
         gtk_box_pack_start (GTK_BOX(text_box), scrolled_window, TRUE, TRUE, 0); 
 
 	pan_callback_add (group_get_articles_removed_callback(),
@@ -1259,7 +1430,8 @@ text_get_message_to_reply_to (void)
 		body = gtk_text_iter_get_text (&sel_start, &sel_end);
 	else
 	{
-		gtk_text_buffer_get_start_iter (_text_buffer, &sel_start);
+		GtkTextMark * mark = gtk_text_buffer_get_mark (_text_buffer, "body_begin");
+		gtk_text_buffer_get_iter_at_mark (_text_buffer, &sel_start, mark);
 		gtk_text_buffer_get_end_iter (_text_buffer, &sel_end);
 		body = gtk_text_buffer_get_text (_text_buffer, &sel_start, &sel_end, FALSE);
 	}
@@ -1424,3 +1596,18 @@ text_deselect_all (void)
 	gtk_text_buffer_move_mark (_text_buffer, gtk_text_buffer_get_selection_bound (_text_buffer), &end);
 	pan_unlock();
 }
+
+/**
+***
+**/
+
+TextMassager* 
+text_pane_get_text_massager (void)
+{
+	static TextMassager * text_massager = NULL;
+
+	if (text_massager == NULL)
+		text_massager = text_massager_new ();
+
+	return text_massager;
+}
