@@ -28,6 +28,7 @@
 
 #include <pan/base/pan-callback.h>
 #include <pan/base/message-identifier.h>
+#include <pan/base/text-massager.h>
 
 /**
 ***  CURRENT ARTICLE
@@ -70,6 +71,9 @@ void          text_set_mute_quoted                           (gboolean);
 gboolean      text_get_mute_quoted                           (void);
 PanCallback*  text_get_mute_quoted_changed_callback          (void);
 
+
+TextMassager*   text_pane_get_text_massager                  (void);
+
 /**
 ***  Rot 13
 **/
@@ -114,8 +118,9 @@ void         text_set_font                   (void);
 
 gchar*       text_get_message_to_reply_to    (void);
 
-void         update_body_pane                (GtkTextBuffer * buffer,
-	                                      const gchar   * body,
-					      gboolean        mute_quotes);
+void         update_body_pane                (const TextMassager  * text_massager,
+                                              GtkTextBuffer       * buffer,
+	                                      const gchar         * body,
+					      gboolean              mute_quotes);
 
 #endif /* __TEXT_H__ */
