@@ -29,18 +29,18 @@
 
 #include <pan/base/argset.h>
 #include <pan/base/debug.h>
+#include <pan/base/pan-config.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-i18n.h>
 #include <pan/base/log.h>
 
 #include <pan/globals.h>
-#include <pan/gui.h>
 #include <pan/prefs.h>
 #include <pan/util.h>
 
 /***
 ****
-****  GUI
+****  DIALOG
 ****
 ***/
 
@@ -144,9 +144,8 @@ pan_error_dialog (const char *format, ...)
  * need to call gdk_threads_enter(), but within the other types of
  * callbacks, you do."
  *
- * pan_timeout_add() and run_in_main_thread_idle() are wrappers around
- * glib-level callbacks that ensure that pan_lock() works properly
- * to give them a gdk_threads_enter() lock.
+ * pan_timeout_add() is a wrapper around a glib-level callbacks that
+ * ensures pan_lock() works properly to a gdk_threads_enter() lock.
  */
 
 static gboolean main_thread_is_in_glib_callback = FALSE;
@@ -173,9 +172,85 @@ pan_timeout_wrapper (gpointer data)
 	return retval;
 }
 guint
-pan_timeout_add (guint32 interval, GtkFunction func, gpointer arg)
+pan_timeout_add (guint32 interval, GSourceFunc func, gpointer arg)
 {
-	return gtk_timeout_add (interval, pan_timeout_wrapper, argset_new2 (func, arg));
+	return g_timeout_add (interval, pan_timeout_wrapper, argset_new2 (func, arg));
+}
+
+/***
+****  GUI QUEUE
+***/
+
+static unsigned int gui_queue_timer_id = 0;
+static GStaticMutex gui_queue_mutex = G_STATIC_MUTEX_INIT;
+static GSList * gui_queue_slist = NULL;
+
+typedef struct
+{
+	GSourceFunc func;
+	gpointer user_data;
+}
+GuiQueueItem;
+
+static int
+gui_queue_timer_cb (gpointer user_data)
+{
+	GSList * list;
+
+	/* get the work list */
+	g_static_mutex_lock (&gui_queue_mutex);
+	list = gui_queue_slist;
+	gui_queue_slist = NULL;
+	g_static_mutex_unlock (&gui_queue_mutex);
+
+	if (list != NULL)
+	{
+		GSList * l;
+
+		/* gui_queue_slist is LIFO for efficiency of adding nodes;
+		 * reverse the nodes to make list FIFO */
+		list = g_slist_reverse (list);
+
+		/* run the work functions */
+		for (l=list; l!=NULL; l=l->next)
+		{
+			GuiQueueItem * queue_item = (GuiQueueItem*) l->data;
+			(*queue_item->func)(queue_item->user_data);
+			g_free (queue_item);
+		}
+		g_slist_free (list);
+	}
+
+	return 1;
+}
+
+void
+gui_queue_init (void)
+{
+	gui_queue_timer_id = pan_timeout_add (100, gui_queue_timer_cb, NULL);
+}
+
+void
+gui_queue_shutdown (void)
+{
+	g_source_remove (gui_queue_timer_id);
+	gui_queue_timer_id = 0;
+}
+
+void
+gui_queue_add (GSourceFunc func, gpointer user_data)
+{
+	GuiQueueItem * item;
+
+	/* create the new queue item */	
+	item = g_new (GuiQueueItem, 1);
+	item->func = func;
+	item->user_data = user_data;
+
+	/* add it to the queue */
+	g_static_mutex_lock (&gui_queue_mutex);
+	gui_queue_slist = g_slist_prepend (gui_queue_slist, item);
+	g_static_mutex_unlock (&gui_queue_mutex);
 }
 
 /***
@@ -200,7 +275,7 @@ pan_lock_from (const gchar * file, const gchar * func, int line)
 	 * then we don't need to lock.
 	 *
 	 * However if pan_lock() is called from a worker thread, or the main
-	 * thread inside a glib idle function (via run_in_main_thread_nolock())
+	 * thread inside a glib idle function (via pan_timeout_add())
 	 * then we _do_ need to obtain a gtk lock.
 	 */
 	if (thr==Pan.main_t && !main_thread_is_in_glib_callback)
@@ -235,7 +310,7 @@ pan_unlock_from (const gchar* file, const gchar * func, int line)
 
 	if (thr==Pan.main_t && !has_lock_thr)
 	{
-		debug4 (DEBUG_LOCK,"mainthread %p attempted unneccessary unlock from %s:%d (%s)", thr, file, line, func);
+		debug4 (DEBUG_LOCK,"mainthread %p attempted unnecessary unlock from %s:%d (%s)", thr, file, line, func);
 	}
 	else if (has_lock_thr != thr)
 	{
@@ -274,29 +349,56 @@ get_date_display_string (time_t date, const char * fmt, char * fillme, int len)
 }
 
 static gboolean
-try_to_execute_url (const gchar * template, const gchar * url)
+try_to_execute_url (const char * template, const char * url)
 {
-	gboolean retval;
-	char * str;
+	int i;
+	int argc = 0;
+	char * freeme;
+	char ** argv = NULL;
+	gboolean retval = FALSE;
+	gboolean ok = TRUE;
 
 	/* sanity clause */
 	g_return_val_if_fail (is_nonempty_string(url), FALSE);
 	if (template == NULL)
 		template = "netscape %s";
 
-	/* add the URL to the template */
-	if (strstr(template,"%s"))
-		str = pan_substitute (template, "%s", url);
-	else
-		str = g_strdup_printf ("%s %s", template, url);
-	replace_gstr (&str, pan_substitute(str, "%%%%", "%%"));
+	/* make sure the backslashes are escaped */
+	freeme = pan_substitute (template, "\\", "\\\\");
+	template = freeme;
 
-	/* execute it */
-	g_message ("%s", str);
-	retval = g_spawn_command_line_async (str, NULL);
+	/* parse the command line */
+	if (ok) {
+		GError * err = NULL;
+		g_shell_parse_argv (template, &argc, &argv, &err);
+		if (err != NULL) {
+			log_add_va (LOG_ERROR, _("Error parsing \"web browser\" command line: %s"), err->message);
+			log_add_va (LOG_ERROR, _("The command line was: %s"), template);
+			g_error_free (err);
+			ok = FALSE;
+		}
+	}
+
+	/* make sure the URL is substitued _after_ the command-line
+	 * parser; those path slashes and backslashes make Windows
+	 * very sad */
+	for (i=0; i<argc; ++i)
+		if (strstr (argv[i], "%s") != NULL) /* filename */
+			replace_gstr (&argv[i], pan_substitute (argv[i], "%s", url));
+
+	/* spawn off the external editor */
+	if (ok) {
+		GError * err = NULL;
+		g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &err);
+		if (err != NULL) {
+			log_add_va (LOG_ERROR, _("Error starting external browser: %s"), err->message);
+			g_error_free (err);
+			ok = FALSE;
+		}
+	}
 
 	/* cleanup */
-	g_free (str);
+	g_free (freeme);
 	return retval;
 }
 
@@ -319,8 +421,13 @@ pan_url_show (const char * url)
 		int i;
 		char ** tries;
 		gboolean result = FALSE;
-		
-		tries = g_strsplit (external_web_browser, ":", -1);
+	
+#ifdef G_OS_WIN32
+#define DELIMITER ";"	
+#else
+#define DELIMITER ":"
+#endif
+		tries = g_strsplit (external_web_browser, DELIMITER, -1);
 		for (i=0; !result && tries[i]!=NULL; ++i)
 			result = try_to_execute_url (tries[i], url);
 
@@ -419,15 +526,6 @@ menu_is_checked (GtkItemFactory   * ifactory,
 	return gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(w));
 }
 
-void
-menu_set_insensitive_all(GtkMenuShell * menu_shell)
-{
-	GList * cur;
-
-	for (cur = menu_shell->children; cur != NULL; cur = cur->next)
-		gtk_widget_set_sensitive(GTK_WIDGET(cur->data), FALSE);
-}
-
 void
 pan_gtk_entry_set_text (GtkWidget            * w,
                         const char           * text)
@@ -441,104 +539,277 @@ pan_gtk_entry_set_text (GtkWidget            * w,
 }
 
 void
-pan_hig_frame_new        (const char           * title,
-                          GtkWidget           ** setme_container,
-                          GtkWidget           ** setme_vbox)
+pan_widget_set_font (GtkWidget * w, const char * font_name)
 {
-	char * pch;
-	GtkWidget * frame;
-	GtkWidget * h;
-	GtkWidget * v;
-	GtkWidget * w;
+	PangoFontDescription * font_desc;
 
-	pch = g_strdup_printf ("<b>%s</b>", title);
-	frame = gtk_frame_new (pch);
-	g_free (pch);
-	gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_NONE);
-	gtk_label_set_use_markup (GTK_LABEL(gtk_frame_get_label_widget (GTK_FRAME(frame))), TRUE);
+	g_return_if_fail (GTK_IS_WIDGET(w));
+	g_return_if_fail (is_nonempty_string(font_name));
 
-	h = gtk_hbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER(frame), h);
-
-	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
-	gtk_widget_set_usize (w, 12u, 0u);
-	gtk_box_pack_start (GTK_BOX(h), w, FALSE, FALSE, 0);
-
-	v = gtk_vbox_new (FALSE, GUI_PAD_SMALL);
-	gtk_box_pack_start (GTK_BOX(h), v, TRUE, TRUE, 0);
-
-	*setme_container = frame;
-	*setme_vbox = v;
+	font_desc = pango_font_description_from_string (font_name);
+	gtk_widget_modify_font (w, font_desc);
+	pango_font_description_free (font_desc);
 }
 
-/*
- * Check for sig delims
- */
+/***
+****
+***/
 
-int
-pan_is_signature_delimitor (const char * line, const int line_len)
+void
+gui_save_column_widths (GtkWidget    * clist,
+                        const char   * type)
 {
-	switch (line_len)
+	if (GTK_IS_CLIST (clist))
 	{
-		case 2: if (!strncmp (line,"--"   ,2)) return SIG_NON_STANDARD;
-					break;
+		int i;
+		int cols = GTK_CLIST (clist)->columns;
+		GtkCList * list = GTK_CLIST (clist);
+		char buf[1024];
 		
-		case 3: if (!strncmp (line,"--\r" ,3)) return SIG_NON_STANDARD;
-				if (!strncmp (line,"-- "  ,3)) return SIG_STANDARD;
-				break;
-		
-		case 4: if (!strncmp (line,"-- \r",4)) return SIG_STANDARD;
-		
-		default:
-			return SIG_NONE;
-	}
-	return SIG_NONE;
-}
-
-gboolean
-pan_remove_signature (char * body)
-{
-	const char * march = body;
-	char * line = NULL;
-	char * sig_point = NULL;
-	int sig_type = SIG_NONE;
-	int lines_below = 0;
-	int line_len = 0;
-
-	/*iterate through the text, line by line*/
-	while (get_next_token_run (march, '\n', &march, (const char **) &line, &line_len))
-	{
-		int st;
-		st = pan_is_signature_delimitor (line, line_len);
-		if (st == SIG_STANDARD || st == SIG_NON_STANDARD)
+		for (i=0; i<cols; i++)
 		{
-			sig_type = st;
-			sig_point = line;
-			lines_below = 0;
-			
-		} else 
-		{
-			if (sig_point)
-				lines_below++;
+			g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
+			pan_config_set_int (buf, (int)list->column[i].width);
 		}
 	}
-	if (sig_type == SIG_NONE) return FALSE;
-	if (sig_point == NULL) return FALSE;
-	if (sig_type == SIG_STANDARD)
+}
+
+void
+gui_save_column_widths_tree_view (GtkWidget    * tree_view,
+				  const char   * type)
+{
+	GList * columns;
+	char buf[1024];	
+	int i;
+	int length;
+
+	debug_enter ("gui_save_column_widths_tree_view");
+
+	columns = gtk_tree_view_get_columns(GTK_TREE_VIEW(tree_view));
+	length = g_list_length(columns);
+
+	for (i = 0; i < length; i++) {
+		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
+		pan_config_set_int (buf, (int)gtk_tree_view_column_get_width(gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), i)));
+	}
+
+	/* cleanup */
+	g_list_free(columns);
+
+	debug_exit ("gui_save_column_widths_tree_view");	
+}
+
+
+ 
+void
+gui_save_window_size (GtkWidget     * widget,
+                      const char    * key)
+{
+	char buf[1024];
+	int x, y, w, h;
+	GtkWindow * window;
+
+	/* sanity clause */
+	g_return_if_fail (GTK_IS_WINDOW(widget));
+	g_return_if_fail (widget->window!=NULL);
+	g_return_if_fail (is_nonempty_string(key));
+
+	window = GTK_WINDOW(widget);
+
+	if (window->maximize_initially)
+	{
+		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_max", key);
+		pan_config_set_bool (buf, TRUE);
+	}
+	else
+	{
+		gtk_window_get_position (window, &x, &y);
+		gtk_window_get_size (window, &w, &h);
+
+		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_x", key);
+		pan_config_set_int (buf, x);
+		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_y", key);
+		pan_config_set_int (buf, y);
+		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_width", key);
+		pan_config_set_int (buf, w);
+		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_height", key);
+		pan_config_set_int (buf, h);
+	}
+}
+
+void
+gui_restore_column_widths (GtkWidget     * clist,
+                           const char   * type)
+{
+	if (GTK_IS_CLIST (clist))
 	{
-		*sig_point = '\0';
-		return TRUE;
+		int i;
+		GtkCList * list = GTK_CLIST (clist);
+		const int cols = list->columns;
+		int * widths = g_newa (int, cols);
+		char buf[1024];
+		debug_enter ("gui_restore_column_widths");
+
+		/* get width from config... */
+		for (i=0; i!=cols; ++i) {
+			g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
+			widths[i] = pan_config_get_int (buf);
+		}
+
+		/* set ui.. */
+		pan_lock();
+		for (i=0; i!=cols; ++i)
+			if (widths[i] != 0)
+				gtk_clist_set_column_width (list, i, widths[i]);
+		pan_unlock();
+
+		/* cleanup */
+		debug_exit ("gui_restore_column_widths");
 	}
+}
+
+void
+gui_restore_column_widths_tree_view (GtkWidget     * tree_view,
+				     const char   * type)
+{
+	GList * columns;
+	char buf[1024];	
+	int * widths;
+	int i;
+	int length;
+		
+	debug_enter ("gui_restore_column_widths_tree_view");
+	
+	columns = gtk_tree_view_get_columns(GTK_TREE_VIEW(tree_view));
+	length = g_list_length(columns);
+	widths = g_newa (int, length);
 	
+	/* get width from config... */
+	for (i = 0; i != length; ++i) {
+		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_column_%d", type, i);
+		widths[i] = pan_config_get_int (buf);
+	}
+
+	/* set ui.. */
+	pan_lock();
+	for (i = 0; i != length; ++i) {
+		if (widths[i] != 0)
+			gtk_tree_view_column_set_fixed_width(gtk_tree_view_get_column(GTK_TREE_VIEW(tree_view), i), widths[i]);
+	}
 	
-	/*if we have a non-standard sig, make sure it's the last one and that
-	 * there are less than SIG_THRESHOLD lines
-	 */
-	if (sig_type == SIG_NON_STANDARD && lines_below <= SIG_THRESHOLD )
+	pan_unlock();
+	
+	/* cleanup */
+	g_list_free(columns);
+	
+	debug_exit ("gui_restore_column_widths_tree_view");
+}
+
+
+
+gboolean
+gui_restore_window_size (GtkWidget     * widget,
+                         const char    * key)
+{
+	int x, y, w, h;
+	char buf[1024];
+	int screen_w = gdk_screen_width ();
+	int screen_h = gdk_screen_height ();
+	const int fuzz = 10;
+	gboolean retval = FALSE;
+
+	/* sanity clause */
+	g_return_val_if_fail (GTK_IS_WIDGET(widget), FALSE);
+	g_return_val_if_fail (is_nonempty_string(key), FALSE);
+
+	/* get the dimensions */
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_x", key);
+	x = pan_config_get_int (buf);
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_y", key);
+	y = pan_config_get_int (buf);
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_width", key);
+	w = pan_config_get_int (buf);
+	g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_height", key);
+	h = pan_config_get_int (buf);
+
+	/* move & resize */
+	if ((0<=x && x<screen_w-fuzz) && (0<=y && y<screen_h-fuzz))
+		gtk_window_move (GTK_WINDOW(widget), x, y);
+	if (w>0 && h>0) {
+		gtk_window_set_default_size (GTK_WINDOW(widget), w, h);
+		retval = TRUE;
+	}
+
+	if (widget == Pan.window)
 	{
-		*sig_point = '\0';
-		return TRUE;
+		gboolean maximize;
+		g_snprintf (buf, sizeof(buf), "/Pan/Geometry/%s_max", key);
+		maximize = pan_config_get_bool (buf);
+
+		if (maximize)
+			gtk_window_maximize (GTK_WINDOW(widget));
+		else
+			gtk_window_unmaximize (GTK_WINDOW(widget));
+	}
+
+	return retval;
+}
+
+/***
+****
+***/
+
+void
+launch_external_editor (const char    * edit_command,
+                        const char    * filename,
+                        int             line_number,
+                        GSourceFunc     finished_callback,
+                        gpointer        finished_user_data)
+{
+	int i;
+	int argc = 0;
+	char ** argv = NULL;
+	gboolean ok = TRUE;
+
+	/* sanity checks */
+	g_return_if_fail (filename != NULL);
+
+	/* parse the command line */
+	if (ok) {
+		GError * err = NULL;
+		g_shell_parse_argv (edit_command, &argc, &argv, &err);
+		if (err != NULL) {
+			log_add_va (LOG_ERROR, _("Error parsing \"external editor\" command line: %s"), err->message);
+			log_add_va (LOG_ERROR, _("The command line was: %s"), edit_command);
+			g_error_free (err);
+			ok = FALSE;
+		}
 	}
-	
-	return FALSE;
+
+	/* make sure the filename is substitued _after_ the command-line parser;
+	 * those path slashes and backslashes make Windows very sad */
+	for (i=0; i<argc; ++i) {
+		if (strstr (argv[i], "%t")) /* filename */
+			replace_gstr (&argv[i], pan_substitute (argv[i], "%t", filename));
+		if (strstr (argv[i], "%n")) { /* line number */
+			char buf[32];
+			g_snprintf (buf, sizeof(buf), "%d", line_number);
+			replace_gstr (&argv[i], pan_substitute (argv[i], "%n", buf));
+		}
+	}
+
+	/* spawn off the external editor */
+	if (ok) {
+		GError * err = NULL;
+		g_spawn_sync (NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL, NULL, &err);
+		if (err != NULL) {
+			log_add_va (LOG_ERROR, _("Error starting external scorefile editor: %s"), err->message);
+			g_error_free (err);
+			ok = FALSE;
+		}
+	}
+
+	/* cleanup */
+	g_strfreev (argv);
+	gui_queue_add (finished_callback, finished_user_data);
 }
