@@ -30,30 +30,29 @@
 
 #include <pan/base/pan-glib-extensions.h> /* for PRETTY_FUNCTION */
 
-/*---------------------------------------------------------------------------
-**
-**  UI LOCKING FOR THREADS
-*/
-
 /**
- * Used to lock the UI thread.
- *
- * @see pan_unlock
- */
+***  Living with Thread in a GUI app
+**/
+
 #define pan_lock() pan_lock_from(__FILE__, PRETTY_FUNCTION, __LINE__)
 void pan_lock_from (const char * file, const char * func, int line);
 
-
-/**
- * Used to unlock the UI thread after pan_lock() has been called.
- *
- * @see pan_lock
- * @see pan_unlock_unconditional
- */
 #define pan_unlock() pan_unlock_from(__FILE__, PRETTY_FUNCTION, __LINE__)
 void pan_unlock_from (const char * file, const char * func, int line);
 
+guint pan_timeout_add   (guint32            interval,
+                         GSourceFunc        function,
+                         gpointer           data);
 
+void gui_queue_add (GSourceFunc func, gpointer user_data);
+
+void gui_queue_init (void);
+
+void gui_queue_shutdown (void);
+
+/**
+***  Dialogs
+**/
 
 void pan_info_dialog (const char * fmt, ...);
 
@@ -61,18 +60,9 @@ void pan_error_dialog (const char * fmt, ...);
 
 void pan_error_dialog_parented (gpointer gtkwindow, const char * fmt, ...);
 
-void replace_gstr (char ** target_free_old, char * assign_from_me);
-
-void get_date_display_string (time_t date, const char * strftime_fmt, char * buf, int size);
-
-void pan_url_show (const char * url);
-
-guint pan_timeout_add   (guint32            interval,
-                         GtkFunction        function,
-			 gpointer           data);
 
 /**
-***  Menu Building
+***  Menus
 **/
 
 GtkWidget* menubar_create     (GtkWidget             * window,
@@ -98,30 +88,56 @@ void menu_set_checked         (GtkItemFactory       * ifactory,
 gboolean menu_is_checked      (GtkItemFactory       * ifactory,
                                const char           * path);
 
-void menu_set_insensitive_all (GtkMenuShell         * menu_shell);
+/**
+***  Column Widths and Window Geometry
+**/
+
+void     gui_save_column_widths               (GtkWidget    * clist,
+                                               const char   * key);
+
+void     gui_save_column_widths_tree_view     (GtkWidget    * tree_view,
+                                               const char   * key);
+
+void     gui_save_window_size                 (GtkWidget    * window,
+                                               const char   * key);
+
+void     gui_restore_column_widths            (GtkWidget    * clist,
+                                               const char   * key);
+
+void     gui_restore_column_widths_tree_view  (GtkWidget    * tree_view,
+                                               const char   * key);
+
+gboolean gui_restore_window_size              (GtkWidget    * window,
+                                               const char   * key);
+
+
+/**
+***
+**/
+
+void launch_external_editor (const char    * edit_command,
+                             const char    * filename,
+                             int             line_number,
+                             GSourceFunc     finished_callback,
+                             gpointer        finished_user_data);
+
+
+void get_date_display_string (time_t date, const char * strftime_fmt, char * buf, int size);
+
+void pan_widget_set_font (GtkWidget* w, const char* font_name);
+
+void pan_url_show (const char * url);
 
 void pan_gtk_entry_set_text   (GtkWidget            * w,
                                const char           * text);
 
-void pan_hig_frame_new        (const char           * title,
-                               GtkWidget           ** setme_container,
-                               GtkWidget           ** setme_vbox);
 
-#define GUI_PAD_SMALL 3
-#define GUI_PAD 6
-#define GUI_PAD_BIG 12
-#define GUI_PAD_LARGE 12
+enum
+{
+	GUI_PAD_SMALL = 3,
+	GUI_PAD = 6,
+	GUI_PAD_BIG = 12,
+	GUI_PAD_LARGE = 12
+};
 
-/*Signature parsing code*/
-#define SIG_STANDARD 		1
-#define SIG_NON_STANDARD 	2
-#define SIG_NONE 		  (-1)
-
-/* Maximum number of lines below an ill-formed signature */
-#define SIG_THRESHOLD 6
-
-int pan_is_signature_delimitor (const char			* line,
-								const int			line_len);
-
-gboolean pan_remove_signature (char 				* body);
 #endif /* __UTIL_H__ */
