@@ -33,6 +33,7 @@ extern const guint8 icon_layout_2[];
 extern const guint8 icon_layout_3[]; 
 extern const guint8 icon_layout_4[]; 
 extern const guint8 icon_layout_5[]; 
+extern const guint8 icon_layout_6[]; 
 extern const guint8 icon_newsgroup[]; 
 extern const guint8 icon_online[]; 
 extern const guint8 icon_offline[]; 
@@ -43,6 +44,7 @@ extern const guint8 icon_read_more[];
 extern const guint8 icon_read_less[]; 
 extern const guint8 icon_read_new_article[]; 
 extern const guint8 icon_read_new_thread[]; 
+extern const guint8 icon_score[]; 
 extern const guint8 icon_server[]; 
 extern const guint8 icon_stock_mail_forward[]; 
 extern const guint8 icon_stock_mail_receive[]; 
