@@ -4,6 +4,7 @@
 #include <time.h>
 
 #include <pan/base/pan-glib-extensions.h>
+#include <pan/base/util-file.h>
 
 static gint
 compare_pint_pint_by_value (const void * va, const void *vb)
@@ -624,6 +625,62 @@ int main (void)
 		g_message ("PASS lower_bound");
 	}
 
+	/* pan_file_normalize */
+	++num;
+	if (1)
+	{
+		const char * home_dir = g_get_home_dir ();
+		const char * in;
+		char * expected;
+		char * out;
+
+		in = "~/.signature";
+		expected = g_build_filename (home_dir, ".signature", NULL);
+		out = pan_file_normalize (in, NULL);
+		if (pan_strcmp (expected, out))
+			return num;
+		g_free (expected);
+		g_free (out);
+
+		in = "~//.signature";
+		expected = g_build_filename (home_dir, ".signature", NULL);
+		out = pan_file_normalize (in, NULL);
+		if (pan_strcmp (expected, out))
+			return num;
+		g_free (expected);
+		g_free (out);
+
+		in = ".signature";
+		expected = g_build_filename (home_dir, ".signature", NULL);
+		out = pan_file_normalize (in, NULL);
+		if (pan_strcmp (expected, out))
+			return num;
+		g_free (expected);
+		g_free (out);
+
+		in = "/var/tmp/.signature";
+		expected = "/var/tmp/.signature";
+		out = pan_file_normalize (in, NULL);
+		if (pan_strcmp (expected, out))
+			return num;
+		g_free (out);
+
+		in = "///var//tmp/////.signature";
+		expected = "/var/tmp/.signature";
+		out = pan_file_normalize (in, NULL);
+		if (pan_strcmp (expected, out))
+			return num;
+		g_free (out);
+
+		in = "  ///var//tmp/////.signature     ";
+		expected = "/var/tmp/.signature";
+		out = pan_file_normalize (in, NULL);
+		if (pan_strcmp (expected, out))
+			return num;
+		g_free (out);
+
+		g_message ("PASS pan_file_normalize");
+	}
 
 	/* success */
 	g_message ("All pan-glib-extensions tests passed");
