@@ -38,7 +38,7 @@ int main (void)
 	server_saving_enabled = FALSE;
 	if (1) {
 		gchar * download_dir = g_strdup_printf ("%s/data/save/", getenv("PWD"));
-		base_prefs_init (download_dir, ">", 1000, FALSE, FALSE);
+		base_prefs_init (download_dir, 1000, FALSE, FALSE);
 		g_free (download_dir);
 	}
 
