@@ -5,6 +5,8 @@
 
 #include <unistd.h>
 
+#include <glib.h>
+
 #include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
 #include <pan/base/newsrc.h>
@@ -32,18 +34,19 @@ files_differ (const gchar * filename1, const gchar * filename2)
 static int
 import_test (void)
 {
-	gint test = 0;
-	gchar *base, *noisy, *less_noisy, *in, *in_sub_only, *out, *out_bak;
+	int test = 0;
+	char *base, *noisy, *less_noisy, *in, *in_sub_only, *out, *out_bak;
 	Server * server;
 
 	base = getenv ("srcdir");
-	base = g_strdup_printf ("%s/", base!=NULL ? base : getenv("PWD"));
-	noisy       = g_strdup_printf ("%s/data/newsrc_noise", base);
-	less_noisy  = g_strdup_printf ("%s/data/newsrc_noise_cleaned_up", base);
-	in          = g_strdup_printf ("%s/data/newsrc_1", base);
-	in_sub_only = g_strdup_printf ("%s/data/newsrc_1_subscribed_only", base);
-	out         = g_strdup_printf ("%s/newsrc_2", g_get_tmp_dir());
-	out_bak     = g_strdup_printf ("%s/newsrc_2.bak", g_get_tmp_dir());
+	if (!g_path_is_absolute (base))
+		base = g_build_filename (g_get_current_dir(), base, NULL);
+	noisy       = g_build_filename (base, "data", "newsrc_noise", NULL);
+	less_noisy  = g_build_filename (base, "data", "newsrc_noise_cleaned_up", NULL);
+	in          = g_build_filename (base, "data", "newsrc_1", NULL);
+	in_sub_only = g_build_filename (base, "data", "newsrc_1_subscribed_only", NULL);
+	out         = g_build_filename (g_get_tmp_dir(), "newsrc_2", NULL);
+	out_bak     = g_build_filename (g_get_tmp_dir(), "newsrc_2.bak", NULL);
 
 	/* create server */
 	server = server_new ();
@@ -102,7 +105,7 @@ init_pan (void)
 	base = getenv ("srcdir");
 	base = g_strdup_printf ("%s/", base!=NULL ? base : getenv("PWD"));
 	download_dir = g_strdup ("/tmp/");
-	base_prefs_init (download_dir, ">", 100, FALSE, FALSE);
+	base_prefs_init (download_dir, 100, FALSE, FALSE);
 
 	g_free (download_dir);
 	g_free (base);
